@@ -48,6 +48,14 @@ my $testCc;
 #
 #*****************************************************************************
 
+sub autoFlushStdout() {
+    my $oldFh = select(STDOUT);
+    $OUTPUT_AUTOFLUSH = $TRUE;
+    select($oldFh);
+}
+
+
+
 sub prompt($$) {
 
     my ($prompt, $default) = @_;
@@ -100,7 +108,7 @@ sub tempFile($) {
 #    my ($cFile, $cFileName) = tempfile("netpbmXXXX", 
 #                                       SUFFIX=>".c", 
 #                                       DIR=>File::Spec->tmpdir(),
-#                                       UNLINK=>1);
+#                                       UNLINK=>0);
     my ($suffix) = @_;
     my $fileName;
     local *file;  # For some inexplicable reason, must be local, not my
@@ -137,22 +145,14 @@ sub chooseTestCompiler($$) {
 
     my $cc;
 
-    if (!defined($compiler)) {
-        if ($ENV{"CC"}) {
-            $cc = $ENV{"CC"};
-        } else {
-            if (commandExists("cc")) {
-                $cc = "cc";
-            } elsif (commandExists("gcc")) {
-                $cc = "gcc";
-            }
-        }
-    } elsif ($compiler eq "cc") {
-        $cc = "cc";
-    } elsif ($compiler eq "gcc") {
-        $cc = "gcc";
+    if ($ENV{'CC'}) {
+        $cc = $ENV{'CC'};
     } else {
-        die("Internal error: invalid value \"$compiler\" for \$compiler");
+        if (commandExists('cc')) {
+            $cc = 'cc';
+        } elsif (commandExists("gcc")) {
+            $cc = 'gcc';
+        }
     }
     $$testCcR = $cc;
 }
@@ -225,7 +225,7 @@ sub displayIntroduction() {
     print("opportunity to make a lot of choices, but you don't have to.  " .
           "If \n");
     print("you don't have reason to believe you're smarter than Configure,\n");
-    print("Just take the defaults (hit ENTER) and don't sweat it.\n");
+    print("just take the defaults (hit ENTER) and don't sweat it.\n");
     print("\n");
 
     print("If you are considering having a program feed answers to the " .
@@ -423,7 +423,8 @@ sub getCompiler($$) {
 #    need different options.
 #
 #  - There are basically two choices on any system:  native compiler or
-#    GNU compiler.  That's all this program recognizes, anyway.
+#    GNU compiler.  That's all this program recognizes, anyway.  On some,
+#    native _is_ GNU, and we return 'gcc'.
 #
 #  - A user may well have various compilers.  Different releases, using
 #    different standard libraries, for different target machines, etc.
@@ -442,15 +443,24 @@ sub getCompiler($$) {
 #
 # The value this subroutine returns is NOT the command name to invoke the
 # compiler.  It is simply "cc" to mean native compiler or "gcc" to mean
-# GNU compiler or undefined to express no preference.
+# GNU compiler.
 #-----------------------------------------------------------------------------
-    my %gccCapablePlatform = ("SOLARIS" => 1,
-                              "TRU64"   => 1,
-                              "SCO"     => 1,
-                              "AIX"     => 1,
-                              "HP"      => 1);
+    my %gccOptionalPlatform = ("SOLARIS" => 1,
+                               "TRU64"   => 1,
+                               "SCO"     => 1,
+                               "AIX"     => 1,
+                               "HP-UX"   => 1);
 
-    if ($gccCapablePlatform{$platform}) {
+    my %gccUsualPlatform = ("GNU"     => 1,
+                            "NETBSD"  => 1,
+                            "OPENBSD" => 1,
+                            "FREEBSD" => 1,
+                            "DARWIN"  => 1,
+                            );
+
+    if ($gccUsualPlatform{$platform}) {
+        $$compilerR = "gcc";
+    } elsif ($gccOptionalPlatform{$platform}) {
         print("GNU compiler or native operating system compiler (cc)?\n");
         print("\n");
 
@@ -474,7 +484,7 @@ sub getCompiler($$) {
             exit 12;
         }
 
-        if ($$compilerR eq "gcc" && !commandExists("gcc")) {
+        if ($$compilerR eq 'gcc' && !commandExists('gcc')) {
             print("WARNING: You selected the GNU compiler, " .
                   "but do not appear to have a program " .
                   "named 'gcc' in your PATH.  This may " .
@@ -483,6 +493,8 @@ sub getCompiler($$) {
                   "makefile variable or install 'gcc'\n");
         }
         print("\n");
+    } else {
+        $$compilerR = 'cc';
     }
 }
 
@@ -693,14 +705,26 @@ sub inttypesDefault() {
 
         my $works;
 
-        my @candidateList = ("<inttypes.h>", "<types.h>", 
-                             "<sys/inttypes.h>", "<sys/types.h>");
+        # We saw a system (Irix 5.3 with native IDO, December 2005) on
+        # which sys/types.h defines uint32_t, but not int32_t and other
+        # similar types.  We saw a Mac OS X system (January 2006) on which
+        # sys/types sometimes defines uint32_t, but sometimes doesn't.
+        # So we make those last resorts.
+
+        # int_fastXXX_t are the least likely of all the types to be
+        # defined, so we look for that.
+
+        # Solaris 8, for example, has a <sys/inttypes.h> that defines
+        # int32_t and uint32_t, but nothing the defines int_fast32_t.
+
+        my @candidateList = ("<inttypes.h>", "<sys/inttypes.h>",
+                             "<types.h>", "<sys/types.h>");
         
         for (my $i = 0; $i < @candidateList && !$works; ++$i) {
             my $candidate = $candidateList[$i];
             my @cSourceCode = (
                                "#include $candidate\n",
-                               "uint32_t testvar;\n"
+                               "int_fast32_t testvar;\n"
                                );
             
             testCompile($cflags, \@cSourceCode, \my $success);
@@ -712,7 +736,7 @@ sub inttypesDefault() {
         if ($works) {
             $retval = $works;
         } else {
-            testCompile($cflags, ["uint32_t testvar;"], \my $success);
+            testCompile($cflags, ["int_fast32_t testvar;"], \my $success);
             if ($success) {
                 $retval = "NONE";
             } else {
@@ -751,29 +775,52 @@ sub getInttypes($) {
                       "statement.  It must be something in <> or \"\".\n");
             } else {
                 $gotit = $TRUE;
-                if ($response ne '<inttypes.h>') {
-                    $$inttypesHeaderFileR = $response;
-                }
+                $$inttypesHeaderFileR = $response;
             }
         }
     }
 }
 
 
+sub getInt64($$) {
+
+    my ($inttypes_h, $haveInt64R) = @_;
+
+    if (defined($testCc)) {
+
+        print("(Doing test compiles to determine if you have int64 type -- " .
+              "ignore errors)\n");
+
+        my $cflags = testCflags($FALSE);
+
+        my $works;
+
+        my @cSourceCode = (
+                           "#include $inttypes_h\n",
+                           "int64_t testvar;\n"
+                           );
+            
+        testCompile($cflags, \@cSourceCode, \my $success);
+            
+        if ($success) {
+            print("You do.\n");
+            $$haveInt64R = 'Y';
+        } else {
+            print("You do not.  64-bit code won't be built.\n");
+            $$haveInt64R = 'N';
+        }
+        print("\n");
+    } else {
+        $$haveInt64R = "N";
+    }
+}
+
+
+
 # TODO: These should do test compiles to see if the headers are in the
 # default search path, both to create a default to offer and to issue a
 # warning after user has chosen.  Also test links to test the link library.
 
-# Before we had prompts for library locations, we set values based on 
-# the platform type, as follows:
-#
-#    TRU64: /usr/local1/DEC/include, /usr/local1/DEC/lib
-#    IRIX:  default                  /usr/lib32
-#                                        this goes with our -n32 option
-#    NETBSD: $(LOCALBASE)/include    $(LOCALBASE)/lib
-#            where LOCALBASE is a build-time environment variable.
-#    DARWIN: same as NETBSD
-#
 # It looks like these should all be in the default search paths and were there
 # just to override defaults in Makefile.config.in.  Since Configure now
 # creates a default of "standard search path," I'm guessing we don't need
@@ -870,39 +917,40 @@ sub getPngLibrary($@) {
 
     my ($platform, @suggestedHdrDir) = @_;
 
-    my $pnglib;
-    {
-        my $default;
-        if (-d("/usr/link/libpng")) {
-            $default = "/usr/link/libpng/libpng" . libSuffix($platform);
-        } else {
-            $default = "libpng" . libSuffix($platform); 
-        }
+    my ($pnglib, $pnghdr_dir);
 
-        print("What is your PNG (graphics format) library?\n");
-        
-        my $response = prompt("library filename or 'none'", $default);
-        
-        if ($response ne "none") {
-            $pnglib = $response;
-        }
-    }
-    my $pnghdr_dir;
-    if (defined($pnglib)) {
-        my $default;
+    if (commandExists('libpng-config')) {
+        # We don't need to ask where Libpng is; there's a 'libpng-config'
+        # That tells exactly how to access it, and the make files will use
+        # that.
+    } else {
+        {
+            my $default = "libpng" . libSuffix($platform);
 
-        if (-d("/usr/include/libpng")) {
-            $default = "/usr/include/libpng";
-        } else {
-            $default = "default";
+            print("What is your PNG (graphics format) library?\n");
+            
+            my $response = prompt("library filename or 'none'", $default);
+            
+            if ($response ne "none") {
+                $pnglib = $response;
+            }
         }
-        
-        print("Where are the interface headers for it?\n");
-        
-        my $response = prompt("PNG header directory", $default);
+        if (defined($pnglib)) {
+            my $default;
 
-        if ($response ne "default") {
-            $pnghdr_dir = $response;
+            if (-d("/usr/include/png")) {
+                $default = "/usr/include/libpng";
+            } else {
+                $default = "default";
+            }
+            
+            print("Where are the interface headers for it?\n");
+            
+            my $response = prompt("PNG header directory", $default);
+
+            if ($response ne "default") {
+                $pnghdr_dir = $response;
+            }
         }
     }
     return($pnglib, $pnghdr_dir);
@@ -914,7 +962,8 @@ sub getZLibrary($@) {
 
     my ($platform, @suggestedHdrDir) = @_;
 
-    my $zlib;
+    my ($zlib, $zhdr_dir);
+
     {
         my $default = "libz" . libSuffix($platform);
 
@@ -926,7 +975,6 @@ sub getZLibrary($@) {
             $zlib = $response;
         }
     }
-    my $zhdr_dir;
     if (defined($zlib)) {
         my $default;
 
@@ -949,6 +997,110 @@ sub getZLibrary($@) {
 
 
 
+sub getX11Library($@) {
+
+    my ($platform, @suggestedHdrDir) = @_;
+
+    my $x11lib;
+    {
+        my $default;
+
+        if (-d('/usr/link/X11')) {
+            $default = '/usr/link/X11/libX11' . libSuffix($platform);
+        } elsif (-d('/usr/X11R6/lib')) {
+            $default = '/usr/X11R6/lib/libX11' . libSuffix($platform);
+        } else {
+            $default = "libX11" . libSuffix($platform);
+        }
+        print("What is your X11 (X client) library?\n");
+        
+        my $response = prompt("library filename or 'none'", $default);
+        
+        if ($response ne "none") {
+            $x11lib = $response;
+        }
+    }
+    my $x11hdr_dir;
+    if (defined($x11lib)) {
+        my $default;
+
+        $default = "default";
+
+        print("Where are the interface headers for it?\n");
+        
+        my $response = prompt("X11 header directory", $default);
+        
+        if ($response ne "default") {
+            $x11hdr_dir = $response;
+        }
+        if (defined($x11hdr_dir)) {
+            if (!-d($x11hdr_dir)) {
+                printf("WARNING: No directory named '$x11hdr_dir' exists.\n");
+            } elsif (!-d("$x11hdr_dir/X11")) {
+                printf("WARNING: Directory '$x11hdr_dir' does not contain " .
+                       "the requisite 'X11' subdirectory\n");
+            }
+        }
+    }
+    return($x11lib, $x11hdr_dir);
+}
+
+
+
+sub getLinuxsvgaLibrary($@) {
+
+    my ($platform, @suggestedHdrDir) = @_;
+
+    my ($svgalib, $svgalibhdr_dir);
+
+    if ($platform eq "GNU") {
+        my $default;
+
+        if (-d('/usr/link/svgalib')) {
+            $default = '/usr/link/svgalib/libvga.so';
+        } elsif (-d('/usr/lib/svgalib')) {
+            $default = '/usr/lib/svgalib/libvga.so';
+        } elsif (system('ldconfig -p | grep libvga &>/dev/null') == 0) {
+            $default = 'libvga.so';
+        } else {
+            $default = 'none';
+        }
+            
+        print("What is your Svgalib library?\n");
+        
+        my $response = prompt("library filename or 'none'", $default);
+            
+        if ($response ne 'none') {
+            $svgalib = $response;
+        }
+    }
+    if (defined($svgalib) && $svgalib ne 'none') {
+        my $default;
+        
+        if (-d('/usr/include/svgalib')) {
+            $default = '/usr/include/svgalib';
+        } else {
+            $default = "default";
+        }
+        print("Where are the interface headers for it?\n");
+        
+        my $response = prompt("Svgalib header directory", $default);
+        
+        if ($response ne "default") {
+            $svgalibhdr_dir = $response;
+        }
+        if (defined($svgalibhdr_dir)) {
+            if (!-d($svgalibhdr_dir)) {
+                printf("WARNING: No directory named " .
+                       "'$svgalibhdr_dir' exists.\n");
+            }
+        }
+    }
+    return($svgalib, $svgalibhdr_dir);
+}
+
+
+
 sub symlink_command() {
 
     my $retval;
@@ -978,61 +1130,6 @@ sub symlink_command() {
     return $retval;
 }
 
-sub getLinuxsvgaLibrary($) {
-#-----------------------------------------------------------------------------
-#  Get the file name of the Linux SVGA library and directory of its
-#  interface headers.  If there isn't an SVGA library, return "undefined"
-#  values.
-#-----------------------------------------------------------------------------
-    my ($platform) = @_;
-
-    my ($linuxsvgahdr_dir, $linuxsvgalib);
-
-    if ($platform eq "GNU") {
-        # Look for SVGALIB 
-        if (-f "/usr/include/vga.h") {
-            $linuxsvgahdr_dir = "/usr/include";
-        } elsif (-d "/usr/include/svgalib") {
-            $linuxsvgahdr_dir = "/usr/include/svgalib";
-        } else {
-            print("Cannot find the include files for SVGALIB.  Enter the " .
-                  "directory where they reside, or none.  " .
-                  "If none, a default " .
-                  "make will not build programs that require " .
-                  "the SVGALIB library.\n");
-
-            print("Svgalib header location (none)=> ");
-
-            my $response = <STDIN>;
-            chomp($response);
-            if ($response ne "" && $response ne "none") {
-                $linuxsvgahdr_dir = $response;
-            }
-        }
-        if (defined($linuxsvgahdr_dir)) {
-            if (-f "/usr/lib/libvga.so") {
-                $linuxsvgalib = "/usr/lib/libvga.so";
-            } elsif (-f "/usr/link/svgalib/libvga.so") {
-                $linuxsvgalib = "/usr/link/svgalib/libvga.so";
-            } else {
-                print("Cannot find the link library for SVGALIB.  " .
-                      "Enter the full pathname of the link library, " .
-                      "or 'none'.  If 'none', " .
-                      "a default make will not build programs that require " .
-                      "the SVGALIB library.\n");
-
-                print("Svgalib link lib location (none)=> ");
-
-                my $response = <STDIN>;
-                chomp($response);
-                if ($response ne "" && $response ne "none") {
-                    $linuxsvgalib = $response;
-                }
-            }
-        }
-    }
-    return($linuxsvgalib, $linuxsvgahdr_dir);
-}
 
 
 sub help() {
@@ -1141,6 +1238,33 @@ sub needLocal($) {
 
 
 
+sub findProcessManagement($) {
+    my ($dontHaveProcessMgmtR) = @_;
+#-----------------------------------------------------------------------------
+#  Return $TRUE iff the system does not have <sys/wait.h> in its default
+#  search path.
+#-----------------------------------------------------------------------------
+    my $cflags = testCflags($FALSE);
+
+    my @cSourceCode = (
+                       "#include <sys/wait.h>\n",
+                       );
+    
+    testCompile($cflags, \@cSourceCode, \my $success);
+
+    if (!$success) {
+        print("Your system does not appear to have <sys/wait.h> in its " .
+              "standard compiler include path.  Therefore, we will build " .
+              "Netpbm with some function missing (e.g. the pm_system() " .
+              "function in libnetpbm and most of 'pamlookup'\n");
+        $$dontHaveProcessMgmtR = $TRUE;
+    } else {
+        $$dontHaveProcessMgmtR = $FALSE;
+    }
+}
+
+
+
 sub validateLibraries(@) {
     my @libList = @_;
 #-----------------------------------------------------------------------------
@@ -1305,7 +1429,7 @@ sub testCompilePngH($$) {
     my ($cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if we can see png.h, assuming we can see
-#  png.h, which png.h #includes.
+#  zlib.h, which png.h #includes.
 #-----------------------------------------------------------------------------
     my @cSourceCode = (
                        "#include <png.h>\n",
@@ -1348,6 +1472,34 @@ sub testPngHdr($$$) {
 
 
 
+sub testLibpngConfig($) {
+    my ($needLocal) = @_;
+#-----------------------------------------------------------------------------
+#  Issue a warning if the instructions 'libpng-config' give for compiling
+#  with Libpng don't work.
+#-----------------------------------------------------------------------------
+    my $generalCflags = testCflags($needLocal);
+
+    my $pngCflags = qx{libpng-config --cflags};
+    chomp($pngCflags);
+
+    testCompilePngH("$generalCflags $pngCflags", \my $success);
+
+    if (!$success) {
+        print("\n");
+        print("Unable to compile a test PNG program using the compiler " .
+              "flags that libpng-config says to use: '$pngCflags'.\n");
+        print("This indicates that you have Libpng installed incorrectly.\n");
+        print("libpng-config gets installed as part of the Libpng package.\n");
+    } else {
+        
+
+
+    }
+}
+
+
+
 sub testConfiguration($$$$$$$) {
 
     my ($needLocal, $jpeglib, $jpeghdr_dir,
@@ -1358,6 +1510,8 @@ sub testConfiguration($$$$$$$) {
     }
     if (defined($pnglib) && defined($zlib)) {
         testPngHdr($needLocal, $pnghdr_dir, $zhdr_dir);
+    } elsif (commandExists('libpng-config')) {
+        testLibpngConfig($needLocal);
     }
 
     # TODO: We ought to validate other libraries too.  But it's not
@@ -1375,7 +1529,7 @@ sub testConfiguration($$$$$$$) {
 #
 #*****************************************************************************
 
-
+autoFlushStdout();
 
 my $configInPathArg;
 if (@ARGV > 0) {
@@ -1480,6 +1634,10 @@ getLibTypes($platform, $subplatform, $default_target,
 
 getInttypes(\my $inttypesHeaderFile);
 
+getInt64($inttypesHeaderFile, \my $haveInt64);
+
+findProcessManagement(\my $dontHaveProcessMgmt);
+
 #******************************************************************************
 #
 #  FIND THE PREREQUISITE LIBRARIES
@@ -1525,8 +1683,12 @@ my ($zlib, $zhdr_dir)       = getZLibrary($platform,
                                           $pnghdr_dir,
                                           $tiffhdr_dir,
                                           $jpeghdr_dir);
+print("\n");
+my ($x11lib, $x11hdr_dir) = getX11Library($platform); 
+
 print("\n");
 my ($linuxsvgalib, $linuxsvgahdr_dir) = getLinuxsvgaLibrary($platform); 
+
 print("\n");
 
 # We should add the JBIG and URT libraries here too.  They're a little
@@ -1631,9 +1793,9 @@ sub gnuCflags($) {
     my ($gccCommandName) = @_;
 
     return("CFLAGS = " . gnuOptimizeOpt($gccCommandName) . " -ffast-math " .
-           " -pedantic " . 
+           " -pedantic -fno-common " . 
            "-Wall -Wno-uninitialized -Wmissing-declarations -Wimplicit " .
-           "-Wwrite-strings -Wmissing-prototypes\n");
+           "-Wwrite-strings -Wmissing-prototypes -Wundef\n");
 }
 
 my @Makefile_config;
@@ -1731,6 +1893,7 @@ if ($platform eq "GNU") {
         # We don't know what to do here.  We used to (before 10.20) just
         # just assume the compiler was gcc.  We know that the gcc stuff
         # above does NOT work for HP native compiler.
+        push(@Makefile_config, "LDSHLIB =\n");
     }
 } elsif ($platform eq "AIX") {
     push(@Makefile_config, 'LDFLAGS = -L /usr/pubsw/lib', "\n");
@@ -1791,8 +1954,6 @@ if ($platform eq "GNU") {
          '-shared -Wl,--image-base=0x10000000 -Wl,--enable-auto-import', "\n");
 } elsif ($platform eq "BEOS") {
     push(@Makefile_config, "LDSHLIB = -nostart\n");
-} elsif ($platform eq "NETBSD") {
-    push(@Makefile_config, 'CFLAGS_SHLIB = -fpic', "\n");
 } elsif ($platform eq "OPENBSD") {
     # vedge@vedge.com.ar says on 2001.04.29 that there are a ton of 
     # undefined symbols in the Fiasco stuff on OpenBSD.  So we'll just
@@ -1809,13 +1970,12 @@ if ($platform eq "GNU") {
     if ($compiler eq "cc") {
         push(@Makefile_config, "CFLAGS = -O\n");
         push(@Makefile_config, "CFLAGS_SHLIB = -O -K pic\n");
-        push(@Makefile_config, "LD_SHLIB = -G\n");
+        push(@Makefile_config, "LDSHLIB = -G\n");
         push(@Makefile_config, "SHLIB_CLIB =\n");
     } else {
         makeCompilerGcc(\@Makefile_config);
+        push(@Makefile_config, "LDSHLIB = -shared\n"); 
     }
-    push(@Makefile_config, "CFLAGS_SHLIB = -fPIC\n");
-    push(@Makefile_config, "LDSHLIB = -shared\n"); 
     push(@Makefile_config, "NETWORKLD = -lsocket -lresolve\n");
 } elsif ($platform eq "DARWIN") {
     push(@Makefile_config, "CC = cc -no-cpp-precomp\n");
@@ -1844,22 +2004,33 @@ if (!$flex_result) {
     # make rules for Thinkjettopbm for information on our experiences
     # with Lexes besides Flex.
 
-    print("\n");
-    print("Using 'lex' as the pattern matcher generator, since we cannot\n");
-    print("find 'flex' on your system.\n");
-    print("\n");
-    push(@Makefile_config, "LEX = lex\n"); 
+    my $systemRc = system('lex </dev/null &>/dev/null');
 
-    # We could also set LEX to null, and then the one part that requires
-    # it -- thinkjettopbm -- would simply not get built.
+    if ($systemRc >> 8 == 127) {
+        print("\n");
+        print("You do not appear to have the 'flex' or 'lex' pattern \n");
+        print("matcher generator on your system, so we will not build \n");
+        print("programs that need it (Thinkjettopbm)\n");
+        
+        print("\n");
+        print("Press ENTER to continue.\n");
+        my $key = <STDIN>;
+        print("\n");
 
-#    print("\n");
-#    print("There is no 'flex' command in your environment, so we will not\n");
-#    print("Build the part that requires it: thinkjettopbm.\n");
+        push(@Makefile_config, "LEX=\n");
+    } else {
+        print("\n");
+        print("Using 'lex' as the pattern matcher generator, " .
+              "since we cannot\n");
+        print("find 'flex' on your system.\n");
+        print("\n");
 
-#    print("\n");
-#    print("Press ENTER to continue.\n");
-#    my $key = <STDIN>;
+        push(@Makefile_config, "LEX = lex\n"); 
+    }
+}
+
+if ($compiler eq 'gcc') {
+    push(@Makefile_config, "CFLAGS_SHLIB += -fPIC\n");
 }
 
 if (defined($tiffhdr_dir)) {
@@ -1890,6 +2061,13 @@ if (defined($zlib)) {
     push(@Makefile_config, "ZLIB = $zlib\n");
 }
 
+if (defined($x11hdr_dir)) {
+    push(@Makefile_config, "X11HDR_DIR = $x11hdr_dir\n");
+}
+if (defined($x11lib)) {
+    push(@Makefile_config, "X11LIB = $x11lib\n");
+}
+
 if (defined($linuxsvgahdr_dir)) {
     push(@Makefile_config, "LINUXSVGAHDR_DIR = $linuxsvgahdr_dir\n");
 }
@@ -1901,10 +2079,17 @@ if (defined($netpbm_docurl)) {
     push(@Makefile_config, "NETPBM_DOCURL = $netpbm_docurl\n");
 }
 
-if (defined($inttypesHeaderFile)) {
+if ($inttypesHeaderFile ne '<inttypes.h>') {
     push(@Makefile_config, "INTTYPES_H = $inttypesHeaderFile\n");
 }
 
+if ($haveInt64 ne 'Y') {
+    push(@Makefile_config, "HAVE_INT64 = $haveInt64\n");
+}
+
+if ($dontHaveProcessMgmt) {
+    push(@Makefile_config, "DONT_HAVE_PROCESS_MGMT = Y\n");
+}
 
 #******************************************************************************
 #
