@@ -31,6 +31,8 @@
  */
 
 #include <string.h>
+
+#include "pm_c_util.h"
 #include "pnm.h"
 
 /* Do what you have to, to get a sensible value here.  This may not be so */
@@ -357,6 +359,22 @@ main(int argc, char * argv[]) {
     exit( 0 );
 }
 
+
+static void
+swapbytes(void *       const p,
+          unsigned int const nbytes) {
+#if BYTE_ORDER == LITTLE_ENDIAN
+    unsigned char * const c = p;
+    unsigned int i;
+    for (i = 0; i < nbytes/2; ++i) {
+        unsigned char const orig = c[i];
+        c[i] = c[nbytes-(i+1)];
+        c[nbytes-(i+1)] = orig;
+    }
+#endif
+}
+
+
 /*
  ** This code will deal properly with integers, no matter what the byte order
  ** or integer size of the host machine.  Sign extension is handled manually
@@ -416,6 +434,7 @@ read_val (fp, bitpix, vp)
         else
             lval = c[0]<<24 | c[1]<<16 | c[2]<<8 | c[3];
         *vp = lval;
+        break;
       
     case -32:
         for (i=0; i<4; i++) {
@@ -424,6 +443,7 @@ read_val (fp, bitpix, vp)
                 pm_error( "EOF / read error" );
             c[i] = ich;
         }
+        swapbytes(c, 4);
         *vp = *( (float *) c);
         break;
       
@@ -434,6 +454,7 @@ read_val (fp, bitpix, vp)
                 pm_error( "EOF / read error" );
             c[i] = ich;
         }
+        swapbytes(c, 8);
         *vp = *( (double *) c);
         break;
       
