@@ -58,11 +58,11 @@ ReadOK(FILE *          const fileP,
 
 static int const maxnum_lzwCode = (1<<MAX_LZW_BITS);
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *input_filespec;  /* Filespecs of input files */
+    const char * input_filespec;  /* Filespecs of input files */
     unsigned int verbose;    /* -verbose option */
     unsigned int comments;   /* -comments option */
     bool all_images;  /* He wants all the images */
@@ -70,19 +70,20 @@ struct cmdline_info {
         /* image number he wants from input, starting at 0.  Undefined
            if all_images is TRUE
         */
-    const char *alpha_filename;
+    const char * alpha_filename;
+    unsigned int quitearly;
 };
 
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdlineP) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def;
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     
@@ -100,8 +101,10 @@ parse_command_line(int argc, char ** argv,
             &cmdlineP->verbose,         0);
     OPTENT3(0, "comments",    OPT_FLAG, NULL,
             &cmdlineP->comments,        0);
+    OPTENT3(0, "quitearly",    OPT_FLAG, NULL,
+            &cmdlineP->quitearly,       0);
     OPTENT3(0, "image",       OPT_STRING, &image,
-            &imageSpec,                  0);
+            &imageSpec,                 0);
     OPTENT3(0, "alphaout",    OPT_STRING, &cmdlineP->alpha_filename, 
             &alphaSpec,                 0);
 
@@ -511,9 +514,14 @@ doGetCode(FILE *                const ifP,
           struct getCodeState * const gsP,
           int *                 const retvalP) {
 
-    if ((gsP->curbit+codeSize) > gsP->bufCount*8 && !gsP->streamExhausted) 
+    while (gsP->curbit + codeSize > gsP->bufCount * 8 &&
+           !gsP->streamExhausted) 
         /* Not enough left in buffer to satisfy request.  Get the next
            data block into the buffer.
+
+           Note that a data block may be as small as one byte, so we may need
+           to do this multiple times to get the full code.  (This probably
+           never happens in practice).
         */
         getAnotherBlock(ifP, gsP);
 
@@ -1362,6 +1370,7 @@ static void
 convertImages(FILE * const ifP, 
               bool   const allImages,
               int    const requestedImageSeq, 
+              bool   const drainStream,
               FILE * const imageout_file, 
               FILE * const alphafile) {
 /*----------------------------------------------------------------------------
@@ -1374,6 +1383,12 @@ convertImages(FILE * const ifP,
    'requestedImageSeq' is meaningful only when 'allImages' is FALSE.  It 
    is the sequence number of the one image Caller wants from the stream,
    with the first image being 0.
+
+   'drainInput' means to read the entire GIF stream, even after
+   reading the image Caller asked for.  We read the stream, not just
+   the file it's in, so we still recognize certain errors in the GIF
+   format in the tail of the stream and there may yet be more stuff in
+   the file when we return.
 -----------------------------------------------------------------------------*/
     int imageSeq;
         /* Sequence within GIF stream of image we are currently processing.
@@ -1388,8 +1403,10 @@ convertImages(FILE * const ifP,
 
     readGifHeader(ifP, &gifScreen);
 
-    eod = FALSE;  /* initial value */
-    for (imageSeq=0; !eod; ++imageSeq) {
+    for (imageSeq = 0, eod = FALSE;
+         !eod && (imageSeq <= requestedImageSeq || allImages || drainStream);
+         ++imageSeq) {
+
         readExtensions(ifP, &gif89, &eod);
 
         if (eod) {
@@ -1413,13 +1430,13 @@ convertImages(FILE * const ifP,
 int
 main(int argc, char **argv) {
 
-    struct cmdline_info cmdline;
+    struct cmdlineInfo cmdline;
     FILE *ifP;
     FILE *alpha_file, *imageout_file;
 
     pnm_init(&argc, argv);
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
     verbose = cmdline.verbose;
     showComment = cmdline.comments;
    
@@ -1436,7 +1453,7 @@ main(int argc, char **argv) {
         imageout_file = stdout;
 
     convertImages(ifP, cmdline.all_images, cmdline.image_no, 
-                  imageout_file, alpha_file);
+                  !cmdline.quitearly, imageout_file, alpha_file);
 
     pm_close(ifP);
     if (imageout_file != NULL) 
