@@ -61,12 +61,12 @@ static void remove_adjacent_corners (index_list_type *, unsigned, bool,
                      at_exception_type * exception);
 static void filter (curve_type, fitting_opts_type *);
 static void find_vectors
-  (unsigned, pixel_outline_type, vector_type *, vector_type *, unsigned);
+  (unsigned const, pixel_outline_type const, vector_type * const, vector_type * const, unsigned const);
 static float find_error (curve_type, spline_type, unsigned *,
                at_exception_type * exception);
 static vector_type find_half_tangent (curve_type, bool start, unsigned *, unsigned);
 static void find_tangent (curve_type, bool, bool, unsigned);
-static void remove_knee_points (curve_type, bool);
+static void remove_knee_points (curve_type const, bool const);
 static void set_initial_parameter_values (curve_type);
 static float distance (float_coord, float_coord);
 
@@ -1852,6 +1852,7 @@ find_error (curve_type curve, spline_type spline, unsigned *worst_point,
 
   if (*worst_point == CURVE_LENGTH (curve) + 1)
     { /* Didn't have any ``worst point''; the error should be zero.  */
+      *worst_point = 0;
       if (epsilon_equal (total_error, 0.0))
         LOG ("  Every point fit perfectly.\n");
       else
