@@ -276,7 +276,7 @@ computeOutName(struct cmdlineInfo const cmdline,
 
     if (cmdline.outName)
         workarea = strdup(cmdline.outName);
-    else if (STREQ(cmdline.inputFilespec, "-"))
+    else if (streq(cmdline.inputFilespec, "-"))
         workarea = NULL;
     else {
         char * cp;
@@ -301,18 +301,18 @@ computeOutName(struct cmdlineInfo const cmdline,
 static void
 validateTupleType(struct pam * const pamP) {
 
-    if (STREQ(pamP->tuple_type, "RGB_ALPHA")) {
+    if (streq(pamP->tuple_type, "RGB_ALPHA")) {
         if (pamP->depth < 4)
             pm_error("Invalid depth for tuple type RGB_ALPHA.  "
                      "Should have at least 4 planes, but has %d.", 
                      pamP->depth);
-    } else if (STREQ(pamP->tuple_type, "RGB")) {
+    } else if (streq(pamP->tuple_type, "RGB")) {
         if (pamP->depth < 3)
             pm_error("Invalid depth for tuple type RGB.  "
                      "Should have at least 3 planes, but has %d.", 
                      pamP->depth);
-    } else if (STREQ(pamP->tuple_type, "GRAYSCALE")) {
-    } else if (STREQ(pamP->tuple_type, "BLACKANDWHITE")) {
+    } else if (streq(pamP->tuple_type, "GRAYSCALE")) {
+    } else if (streq(pamP->tuple_type, "BLACKANDWHITE")) {
     } else 
         pm_error("Invalid type of input.  PAM tuple type is '%s'.  "
                  "This programs understands only RGB_ALPHA, RGB, GRAYSCALE, "
@@ -337,7 +337,7 @@ computeImageType_cht(struct pam *            const pamP,
 
     validateTupleType(pamP);
 
-    withAlpha = (STREQ(pamP->tuple_type, "RGB_ALPHA"));
+    withAlpha = (streq(pamP->tuple_type, "RGB_ALPHA"));
 
     if (cmdline.defaultFormat) {
         /* default the image type */
@@ -461,7 +461,8 @@ computeTgaHeader(struct pam *          const pamP,
 static void
 releaseTgaHeader(struct ImageHeader const tgaHeader) {
 
-    strfree(tgaHeader.Id);
+    if (tgaHeader.IdLength > 0)
+        strfree(tgaHeader.Id);
 }
 
 
