@@ -276,17 +276,17 @@ parseCommandLine(int                        argc,
         cmdlineP->yresolution = -1;
 
     if (resolutionUnitSpec) {
-        if (STREQ(resolutionUnit, "inch"))
+        if (streq(resolutionUnit, "inch"))
             cmdlineP->resolutionUnit = RESUNIT_INCH;
-        else if (STREQ(resolutionUnit, "in"))
+        else if (streq(resolutionUnit, "in"))
             cmdlineP->resolutionUnit = RESUNIT_INCH;
-        else if (STREQ(resolutionUnit, "centimeter"))
+        else if (streq(resolutionUnit, "centimeter"))
             cmdlineP->resolutionUnit = RESUNIT_CENTIMETER;
-        else if (STREQ(resolutionUnit, "cm"))
+        else if (streq(resolutionUnit, "cm"))
             cmdlineP->resolutionUnit = RESUNIT_CENTIMETER;
-        else if (STREQ(resolutionUnit, "none"))
+        else if (streq(resolutionUnit, "none"))
             cmdlineP->resolutionUnit = RESUNIT_NONE;
-        else if (STREQ(resolutionUnit, "no"))
+        else if (streq(resolutionUnit, "no"))
             cmdlineP->resolutionUnit = RESUNIT_NONE;
         else
             pm_error("The only acceptable values for -resolutionunit are "
@@ -351,10 +351,10 @@ putSample(sample           const s,
     if (maxval != tiff_maxval)
         s2 = s * tiff_maxval / maxval;
     if (bitspersample > 8) {
-        *((unsigned short *)(*tPP)) = s;
+        *((unsigned short *)(*tPP)) = s2;
         (*tPP) += sizeof(short);
     } else
-        *(*tPP)++ = s & 0xff;
+        *(*tPP)++ = s2 & 0xff;
 }
 
 
@@ -1067,7 +1067,7 @@ main(int argc, char *argv[]) {
     
     ifP = pm_openr_seekable(cmdline.input_filespec);
 
-    if (STREQ(cmdline.input_filespec, "-"))
+    if (streq(cmdline.input_filespec, "-"))
         inputFileDescription = "Standard Input";
     else 
         inputFileDescription = cmdline.input_filespec;
