@@ -42,9 +42,9 @@
 
 
 typedef struct InputSource {
-    const char *name; 
-    struct InputSource *next;   
-}InputSource;
+    const char *         name; 
+    struct InputSource * next;
+} InputSource;
 
 
 
@@ -115,9 +115,9 @@ parseCommandLine(int argc, char ** argv,
             &cmdlineP->colorok, 0);
     OPTENT3(0, "center",    OPT_FLAG,    NULL,                  
             &cmdlineP->center, 0 );
-    OPTENT3(0, "feeder",    OPT_STRING,  &cmdlineP->feeder,
+    OPTENT3(0, "feeder",    OPT_UINT,    &cmdlineP->feeder,
             &cmdlineP->feederSpec,        0);
-    OPTENT3(0, "outtray",   OPT_STRING,  &cmdlineP->outtray,
+    OPTENT3(0, "outtray",   OPT_UINT,    &cmdlineP->outtray,
             &cmdlineP->outtraySpec,       0);
     OPTENT3(0, "verbose",   OPT_FLAG,    NULL,                  
             &cmdlineP->verbose, 0);
@@ -176,6 +176,7 @@ parseCommandLine(int argc, char ** argv,
     if (argc-1 < 1) {
         MALLOCVAR(cmdlineP->sourceP);
         cmdlineP->sourceP->name = "-";
+        cmdlineP->sourceP->next = NULL;
     } else {
         int i;
         InputSource ** nextLinkP;
@@ -329,16 +330,16 @@ XY_Write(int fd, const void *buf,int cnt) {
 }
 #define XY_Puts(fd,str)  XY_Write(fd,str,strlen(str))
 
-typedef struct pcl_img {
+typedef struct pclGenerator {
     enum ColorDepth colorDepth;
     enum Colorspace colorSpace;
     int width,height;
     int linelen; /* bytes per line */
     unsigned char *data;
-} XL_pcl_img;
+    void (*getnextrow)(const struct pclGenerator *, struct pam *);
+} pclGenerator;
 
-struct tPrinter 
-{ 
+struct tPrinter { 
     const char *name;
     float topmargin;
     float bottommargin;
@@ -348,7 +349,9 @@ struct tPrinter
     { "lj2200",0,0,0,0 }
 };
 
-static  int
+
+
+static int
 out_ubyte(int fd,unsigned char data) {
     return XY_Write(fd,&data,1);
 }
@@ -762,32 +765,35 @@ positionCursor(int            const outFd,
 
 
 static void
-XL_WriteRleBlock(int                const outFd,
-                 const XL_pcl_img * const img,
-                 int                const firstLine,
-                 int                const nlines,
-                 XY_rle *           const rle) {
+convertAndWriteRleBlock(int                  const outFd,
+                        const pclGenerator * const pclGeneratorP,
+                        struct pam *         const pamP,
+                        int                  const firstLine,
+                        int                  const nlines,
+                        XY_rle *             const rle) {
 
-    const unsigned char pad[4] = {0,0,0,0};
-    unsigned int const paddedLinelen = ((img->linelen+3)/4)*4;
+    unsigned char const pad[4] = {0,0,0,0};
+    unsigned int const paddedLinelen = ((pclGeneratorP->linelen+3)/4)*4;
     int rlelen;
     unsigned int line;
     
     XY_RLEreset(rle);
 
-    for(line=firstLine;line<firstLine+nlines;++line) {
-        XY_RLEput(rle,img->data+img->linelen*line,img->linelen);
-        XY_RLEput(rle,pad,paddedLinelen - img->linelen);
+    for (line = firstLine; line < firstLine + nlines; ++line) {
+        pclGeneratorP->getnextrow(pclGeneratorP, pamP);
+        XY_RLEput(rle, pclGeneratorP->data, pclGeneratorP->linelen);
+        XY_RLEput(rle, pad, paddedLinelen - pclGeneratorP->linelen);
     }
-    rlelen=XY_RLEfinish(rle);
-    if(rlelen<0) 
+    rlelen = XY_RLEfinish(rle);
+    if (rlelen<0) 
         pm_error("Error on Making rle");
 
-    xl_dataLength(outFd,rlelen); 
-    XY_Write(outFd,rle->fbuf,rlelen);
+    xl_dataLength(outFd, rlelen); 
+    XY_Write(outFd, rle->fbuf, rlelen);
 }
 
 
+
 /*
  * ------------------------------------------------------------
  * XL_WriteImage
@@ -795,41 +801,45 @@ XL_WriteRleBlock(int                const outFd,
  * ------------------------------------------------------------
  */
 static void 
-XL_WriteImage(int outFd,XL_pcl_img *img) 
-{
+convertAndWriteImage(int                  const outFd,
+                     const pclGenerator * const pclGenP,
+                     struct pam *         const pamP) {
+
     int blockStartLine;
-    XY_rle *rle;
+    XY_rle * rle;
 
-    xl_ubyte(outFd,eDirectPixel); xl_attr_ubyte(outFd,aColorMapping);
-    xl_ubyte(outFd,img->colorDepth); xl_attr_ubyte(outFd,aColorDepth);
-    xl_uint16(outFd,img->width); xl_attr_ubyte(outFd,aSourceWidth);  
-    xl_uint16(outFd,img->height); xl_attr_ubyte(outFd,aSourceHeight);    
-    xl_uint16_xy(outFd,img->width*1,img->height*1); 
-    xl_attr_ubyte(outFd,aDestinationSize);   
-    XL_Operator(outFd,oBeginImage);
+    xl_ubyte(outFd, eDirectPixel); xl_attr_ubyte(outFd, aColorMapping);
+    xl_ubyte(outFd, pclGenP->colorDepth); xl_attr_ubyte(outFd, aColorDepth);
+    xl_uint16(outFd, pclGenP->width); xl_attr_ubyte(outFd, aSourceWidth);  
+    xl_uint16(outFd, pclGenP->height); xl_attr_ubyte(outFd, aSourceHeight);    
+    xl_uint16_xy(outFd, pclGenP->width*1, pclGenP->height*1); 
+    xl_attr_ubyte(outFd, aDestinationSize);   
+    XL_Operator(outFd, oBeginImage);
 
-    rle = XY_RLEnew(img->linelen*20);
-    if(!rle) 
+    rle = XY_RLEnew(pclGenP->linelen*20);
+    if (!rle) 
         pm_error("Unable to allocate %d bytes for the RLE buffer",
-                 img->linelen*20);
+                 pclGenP->linelen * 20);
 
-    blockStartLine=0;
-    while (blockStartLine<img->height) {
-        int const blockHeight=MIN(20,img->height-blockStartLine);
-        xl_uint16(outFd,blockStartLine); xl_attr_ubyte(outFd,aStartLine); 
-        xl_uint16(outFd,blockHeight); xl_attr_ubyte(outFd,aBlockHeight);
-        xl_ubyte(outFd,eRLECompression); xl_attr_ubyte(outFd,aCompressMode);
+    blockStartLine = 0;
+    while (blockStartLine < pclGenP->height) {
+        unsigned int const blockHeight =
+            MIN(20, pclGenP->height-blockStartLine);
+        xl_uint16(outFd, blockStartLine); xl_attr_ubyte(outFd, aStartLine); 
+        xl_uint16(outFd, blockHeight); xl_attr_ubyte(outFd, aBlockHeight);
+        xl_ubyte(outFd, eRLECompression); xl_attr_ubyte(outFd, aCompressMode);
         /* In modern PCL-XL, we could use a PadBytesMultiple attribute
            here to avoid having to pad the data to a multiple of 4
            bytes.  But PCL-XL 1.1 didn't have PadBytesMultiple.
-        xl_ubyte(outFd,1); xl_attr_ubyte(outFd,aPadBytesMultiple); 
+           xl_ubyte(outFd, 1); xl_attr_ubyte(outFd, aPadBytesMultiple); 
         */
-        XL_Operator(outFd,oReadImage);
-        XL_WriteRleBlock(outFd,img,blockStartLine,blockHeight,rle);
-        blockStartLine+=blockHeight;
+        XL_Operator(outFd, oReadImage);
+        convertAndWriteRleBlock(outFd, pclGenP, pamP,
+                                blockStartLine, blockHeight, rle);
+        blockStartLine += blockHeight;
     }
     XY_RLEdelete(rle);
-    XL_Operator(outFd,oEndImage);
+    XL_Operator(outFd, oEndImage);
 }
 
 
@@ -851,22 +861,22 @@ endPage(int          const outFd,
 
 
 static void
-printPage(int                 const outFd,
-          XL_pcl_img *        const img,
-          enum MediaSize      const format,
-          int                 const dpi,
-          bool                const center,
-          float               const xoffs,
-          float               const yoffs,
-          bool                const doDuplex,
-          enum DuplexPageMode const duplex,
-          bool                const doCopies,
-          unsigned int        const copies,
-          bool                const doMediaSource,
-          int                 const mediaSource,
-          bool                const doMediaDestination,
-          int                 const mediaDestination) {
-
+convertAndPrintPage(int                  const outFd,
+                    const pclGenerator * const pclGeneratorP,
+                    struct pam *         const pamP,
+                    enum MediaSize       const format,
+                    int                  const dpi,
+                    bool                 const center,
+                    float                const xoffs,
+                    float                const yoffs,
+                    bool                 const doDuplex,
+                    enum DuplexPageMode  const duplex,
+                    bool                 const doCopies,
+                    unsigned int         const copies,
+                    bool                 const doMediaSource,
+                    int                  const mediaSource,
+                    bool                 const doMediaDestination,
+                    int                  const mediaDestination) {
 
     beginPage(outFd, doDuplex, duplex, doMediaSource, mediaSource,
               doMediaDestination, mediaDestination, format);
@@ -881,12 +891,12 @@ printPage(int                 const outFd,
 
        So we don't use a palette at all now.  
     */
-    setColorSpace(outFd, img->colorSpace, NULL, 0, 0);
+    setColorSpace(outFd, pclGeneratorP->colorSpace, NULL, 0, 0);
 
     positionCursor(outFd, center, xoffs, yoffs, 
-                   img->width, img->height, dpi, format);
+                   pclGeneratorP->width, pclGeneratorP->height, dpi, format);
 
-    XL_WriteImage(outFd,img);   
+    convertAndWriteImage(outFd, pclGeneratorP, pamP);
 
     endPage(outFd, doCopies, copies);
 }
@@ -918,45 +928,84 @@ endSession(int outFd) {
 
 
 static void
-pnm_to_pclimg_packbits(struct pam * pamP,XL_pcl_img * img) {
-    /* Samples are black or white and packed 8 to a byte */
+pnmToPcllinePackbits(const pclGenerator * const pclGeneratorP,
+                     struct pam *         const pamP) {
 
     tuple * tuplerow;
+    unsigned int pcl_cursor;
+    unsigned char accum;
+    unsigned char bitmask;
+    unsigned int col;
         
-    img->colorDepth=e1Bit;
-    img->colorSpace=eGray;
-    img->linelen=((pamP->width+7)/8);
-    img->height=pamP->height;
-    img->width=(pamP->width);
+    tuplerow = pnm_allocpamrow(pamP);
 
-    img->data=(unsigned char *)malloc(((pamP->width+7)/8)
-                                      *pamP->height*pamP->depth);
-    tuplerow=pnm_allocpamrow(pamP);
-    {
-        int row;
-        unsigned int pcl_cursor;
+    pnm_readpamrow(pamP, tuplerow);
 
-        pcl_cursor=0;
-        for (row=0;row<pamP->height;++row) {
-            unsigned char accum;
-            unsigned char bitmask;
-            int col;
+    pcl_cursor = 0; bitmask = 0x80; accum = 0x00;
+    for (col = 0; col < pamP->width; ++col) {
+        if (tuplerow[col][0] == PAM_PBM_WHITE)
+            accum |= bitmask;
+        bitmask >>= 1;
+        if (bitmask == 0) {
+            pclGeneratorP->data[pcl_cursor++] = accum;
+            bitmask = 0x80; accum = 0x0;
+        } 
+    }
+    if (bitmask != 0x80)
+        pclGeneratorP->data[pcl_cursor++] = accum;
 
-            pnm_readpamrow(pamP, tuplerow);
+    pnm_freepamrow(tuplerow);
+}
 
-            bitmask=0x80; accum=0x00;
-            for (col=0;col<pamP->width;++col) {
-                if (tuplerow[col][0]==PAM_PBM_WHITE)
-                    accum |= bitmask;
-                bitmask>>=1;
-                if (bitmask==0) {
-                    img->data[pcl_cursor++]=accum;
-                    bitmask=0x80; accum=0x0;
-                } 
-            }
-	    if(bitmask!=0x80) {
-                    img->data[pcl_cursor++]=accum;
-	    }
+
+
+static void
+createPclGeneratorPackbits(struct pam *    const pamP,
+                           pclGenerator ** const pclGeneratorPP) {
+
+    /* Samples are black or white and packed 8 to a byte */
+
+    pclGenerator * pclGeneratorP;
+
+    MALLOCVAR_NOFAIL(pclGeneratorP);
+
+    pclGeneratorP->colorDepth = e1Bit;
+    pclGeneratorP->colorSpace = eGray;
+    pclGeneratorP->linelen = (pamP->width+7)/8;
+    pclGeneratorP->height = pamP->height;
+    pclGeneratorP->width = (pamP->width);
+
+    pclGeneratorP->data = malloc(pclGeneratorP->linelen);
+    
+    if (pclGeneratorP->data == NULL)
+        pm_error("Unable to allocate row buffer.");
+
+    pclGeneratorP->getnextrow = pnmToPcllinePackbits;
+
+    *pclGeneratorPP = pclGeneratorP;
+}
+
+
+
+static void
+pnmToPcllineWholebytes(const pclGenerator * const pclGeneratorP,
+                       struct pam *         const pamP) {
+
+    tuple * tuplerow;
+    unsigned int pcl_cursor;
+    unsigned int col;
+
+    tuplerow = pnm_allocpamrow(pamP);
+
+    pnm_readpamrow(pamP, tuplerow);
+
+    pcl_cursor = 0; /* initial value */
+    
+    for (col = 0; col < pamP->width; ++col) {
+        unsigned int plane;
+        for (plane = 0; plane < pamP->depth; ++plane) {
+            pclGeneratorP->data[pcl_cursor++] = 
+                pnm_scalesample(tuplerow[col][plane], pamP->maxval, 255);
         }
     }
     pnm_freepamrow(tuplerow);
@@ -964,55 +1013,54 @@ pnm_to_pclimg_packbits(struct pam * pamP,XL_pcl_img * img) {
 
 
 
-
 static void
-pnm_to_pclimg_wholebytes(struct pam * pamP,XL_pcl_img *img) {
+createPclGeneratorWholebytes(struct pam *    const pamP,
+                             pclGenerator ** const pclGenPP) {
     /* One sample per byte */
+
+    pclGenerator * pclGenP;
+
+    MALLOCVAR_NOFAIL(pclGenP);
     
-    tuple * tuplerow;
-        
     if (pamP->depth <  3)
-        img->colorSpace = eGray;
+        pclGenP->colorSpace = eGray;
     else
-        img->colorSpace = eRGB;
+        pclGenP->colorSpace = eRGB;
 
-    img->colorDepth = e8Bit;
-    img->height     = pamP->height;
-    img->width      = pamP->width;
-    img->linelen    = pamP->width * pamP->depth;
+    pclGenP->colorDepth = e8Bit;
+    pclGenP->height     = pamP->height;
+    pclGenP->width      = pamP->width;
+    pclGenP->linelen    = pamP->width * pamP->depth;
 
-    if (UINT_MAX / pamP->height / pamP->width < pamP->depth)
+    if (UINT_MAX / pamP->width < pamP->depth)
         pm_error("Image to big to process");
     else
-        img->data = malloc(pamP->width * pamP->height * pamP->depth);
+        pclGenP->data = malloc(pamP->width * pamP->depth);
 
-    tuplerow=pnm_allocpamrow(pamP);
-    {
-        unsigned int row;
-        unsigned int pcl_cursor;
+    if (pclGenP->data == NULL)
+        pm_error("Unable to allocate row buffer.");
+    
+    pclGenP->getnextrow = pnmToPcllineWholebytes;
 
-        pcl_cursor=0;
-        for (row=0;row<pamP->height;++row) {
-            unsigned int col;
-            pnm_readpamrow(pamP, tuplerow);
-            for (col=0;col<pamP->width;++col) {
-                unsigned int plane;
-                for (plane=0;plane<pamP->depth;++plane)
-                    img->data[pcl_cursor++] = 
-                        pnm_scalesample(tuplerow[col][plane], 
-                                        pamP->maxval, 255);
-            }
-        }
-    }
-    pnm_freepamrow(tuplerow);
+    *pclGenPP = pclGenP;
+}
+
+
+
+static void
+destroyPclGenerator(pclGenerator * const pclGenP) {
+
+    free(pclGenP->data);
+
+    free(pclGenP);
 }
 
 
 
 static void 
-pnm_to_pclimg(struct pam *        const pamP,
-              XL_pcl_img *        const img,
-              bool                const colorok) {
+createPclGenerator(struct pam *        const pamP,
+                   pclGenerator **     const pclGeneratorPP,
+                   bool                const colorok) {
 
     if (pamP->depth > 1 && !colorok)
         pm_message("WARNING: generating a color print stream because the "
@@ -1021,10 +1069,10 @@ pnm_to_pclimg(struct pam *        const pamP,
                    "through Ppmtopgm.  To suppress this warning, use the "
                    "-colorok option.");
 
-    if (pamP->depth==1 && pamP->maxval==1) 
-        pnm_to_pclimg_packbits(pamP,img);
+    if (pamP->depth == 1 && pamP->maxval == 1) 
+        createPclGeneratorPackbits(pamP, pclGeneratorPP);
     else 
-        pnm_to_pclimg_wholebytes(pamP,img);
+        createPclGeneratorWholebytes(pamP, pclGeneratorPP);
 }
 
 
@@ -1075,27 +1123,28 @@ printPages(int                 const outFd,
         while(!eof) {
             pnm_nextimage(in_file, &eof);
             if (!eof) {
-                XL_pcl_img pcl_img;
+                pclGenerator * pclGeneratorP;
 
                 ++pageNum;
                 pm_message("Processing File %u, Page %u", sourceNum, pageNum);
 
                 pnm_readpaminit(in_file, &pam, PAM_STRUCT_SIZE(tuple_type));
                 
-                pnm_to_pclimg(&pam, &pcl_img, colorok);
+                createPclGenerator(&pam, &pclGeneratorP, colorok);
+                
+                convertAndPrintPage(
+                    outFd, pclGeneratorP, &pam,
+                    format, dpi, center, xoffs, yoffs, doDuplex, duplex,
+                    doCopies, copies, doMediaSource, mediaSource,
+                    doMediaDestination, mediaDestination);
 
-                printPage(outFd, &pcl_img,
-                          format, dpi, center, xoffs, yoffs, doDuplex, duplex,
-                          doCopies, copies, doMediaSource, mediaSource,
-                          doMediaDestination, mediaDestination);
-
-                free(pcl_img.data);
+                destroyPclGenerator(pclGeneratorP);
             }
         }
         pm_close(in_file);
         sourceP = sourceP->next; 
     }
-    XL_Operator(outFd,oCloseDataSource);
+    XL_Operator(outFd, oCloseDataSource);
 }
 
 
