@@ -40,11 +40,11 @@
 #include "nstring.h"
 #include "bmepsoe.h"
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *input_filespec;  /* Filespecs of input file */
+    const char * inputFileName;  /* Filespecs of input file */
     float scale;
     unsigned int dpiX;     /* horiz component of DPI option */
     unsigned int dpiY;     /* vert component of DPI option */
@@ -109,9 +109,9 @@ parseDpi(const char *   const dpiOpt,
 
 static void
 parseCommandLine(int argc, char ** argv,
-                 struct cmdline_info *cmdlineP) {
+                 struct cmdlineInfo * const cmdlineP) {
 
-    unsigned int imagewidth_spec, imageheight_spec;
+    unsigned int imagewidthSpec, imageheightSpec;
     float imagewidth, imageheight;
     unsigned int center, nocenter;
     unsigned int nosetpage;
@@ -140,8 +140,8 @@ parseCommandLine(int argc, char ** argv,
     OPTENT3(0, "center",      OPT_FLAG,  NULL, &center,                  0);
     OPTENT3(0, "nocenter",    OPT_FLAG,  NULL, &nocenter,                0);
     OPTENT3(0, "equalpixels", OPT_FLAG,  NULL, &cmdlineP->equalpixels,   0);
-    OPTENT3(0, "imagewidth",  OPT_FLOAT, &imagewidth,  &imagewidth_spec, 0);
-    OPTENT3(0, "imageheight", OPT_FLOAT, &imageheight, &imageheight_spec,0);
+    OPTENT3(0, "imagewidth",  OPT_FLOAT, &imagewidth,  &imagewidthSpec,  0);
+    OPTENT3(0, "imageheight", OPT_FLOAT, &imageheight, &imageheightSpec, 0);
     OPTENT3(0, "nosetpage",   OPT_FLAG,  NULL, &nosetpage,               0);
     OPTENT3(0, "setpage",     OPT_FLAG,  NULL, &cmdlineP->setpage,       0);
     OPTENT3(0, "noshowpage",  OPT_FLAG,  NULL, &noshowpage,              0);
@@ -187,11 +187,11 @@ parseCommandLine(int argc, char ** argv,
     cmdlineP->width  = width * 72;
     cmdlineP->height = height * 72;
 
-    if (imagewidth_spec)
+    if (imagewidthSpec)
         cmdlineP->imagewidth = imagewidth * 72;
     else
         cmdlineP->imagewidth = 0;
-    if (imageheight_spec)
+    if (imageheightSpec)
         cmdlineP->imageheight = imageheight * 72;
     else
         cmdlineP->imageheight = 0;
@@ -201,16 +201,13 @@ parseCommandLine(int argc, char ** argv,
         pm_error("You must specify -psfilter in order to specify "
                  "-flate or -ascii85");
 
-    if (cmdlineP->rle && cmdlineP->flate)
-        pm_error("You cannot specify both -rle and -flate");
-
     if (argc-1 == 0) 
-        cmdlineP->input_filespec = "-";
+        cmdlineP->inputFileName = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdlineP->input_filespec = argv[1];
+        cmdlineP->inputFileName = argv[1];
 
 }
 
@@ -498,11 +495,18 @@ destroyBmepsOutputEncoder(struct bmepsoe * const bmepsoeP) {
 
 static void
 outputBmepsSample(struct bmepsoe * const bmepsoeP,
-                  unsigned int     const sampleValue) {
+                  unsigned int     const sampleValue,
+          unsigned int     const bitsPerSample) {
 
-    assert(sampleValue < 256);
+    if (bitsPerSample == 8)
+        oe_byte_add(bmepsoeP->oeP, sampleValue);
+    else {
+        unsigned int m;
 
-    oe_byte_add(bmepsoeP->oeP, sampleValue);
+        for (m = 1 << (bitsPerSample-1); m != 0; m >>= 1)
+            /* depends on oe_bit_add accepting any value !=0 as 1 */
+            oe_bit_add(bmepsoeP->oeP, sampleValue & m); 
+    }
 }
 
 
@@ -518,41 +522,6 @@ flushBmepsOutput(struct bmepsoe * const bmepsoeP) {
 ============================================================================*/
 
 
-static void
-putEnd(bool const showpage, 
-       bool const psfilter,
-       bool const ascii85,
-       bool const dict,
-       bool const vmreclaim) {
-
-    if (psfilter) {
-        if (ascii85)
-            printf("%s\n", "~>");
-        else
-            printf("%s\n", ">");
-    }
-
-    printf("currentdict /inputf undef\n");
-    printf("currentdict /picstr undef\n");
-    printf("currentdict /rpicstr undef\n");
-    printf("currentdict /gpicstr undef\n");
-    printf("currentdict /bpicstr undef\n");
-
-    if (dict)
-        printf("end\n");
-
-    if (vmreclaim)
-        printf("1 vmreclaim\n");
-
-    printf( "grestore\n" );
-
-    if (showpage)
-        printf( "showpage\n" );
-    printf( "%%%%Trailer\n" );
-}
-
-
-
 static void
 computeImagePosition(int     const dpiX, 
                      int     const dpiY, 
@@ -563,7 +532,7 @@ computeImagePosition(int     const dpiX,
                      bool    const center,
                      int     const pagewid, 
                      int     const pagehgt, 
-                     float   const requested_scale,
+                     float   const requestedScale,
                      float   const imagewidth,
                      float   const imageheight,
                      bool    const equalpixels,
@@ -598,7 +567,7 @@ computeImagePosition(int     const dpiX,
    'equalpixels' means the user wants one printed pixel per input pixel.
    It is inconsistent with imagewidth or imageheight != 0
 
-   'requested_scale' is meaningful only when imageheight/imagewidth == 0
+   'requestedScale' is meaningful only when imageheight/imagewidth == 0
    and equalpixels == FALSE.  It tells how many inches the user wants
    72 pixels of input to occupy, if it fits on the page.
 -----------------------------------------------------------------------------*/
@@ -651,19 +620,19 @@ computeImagePosition(int     const dpiX,
         const float pixfacY = 72.0 / dpiY * devpixY;  /* 1, approx. */
         float scale;
 
-        scale = MIN(requested_scale, 
+        scale = MIN(requestedScale, 
                     MIN((float)pagewid/cols, (float)pagehgt/rows));
 
         *scolsP = scale * cols * pixfacX;
         *srowsP = scale * rows * pixfacY;
         
-        if (scale != requested_scale)
+        if (scale != requestedScale)
             pm_message("warning, image too large for page, rescaling to %g", 
                        scale );
 
         /* Before May 2001, Pnmtops enforced a 5% margin around the page.
            If the image would be too big to leave a 5% margin, Pnmtops would
-           scale it down.  But people have images that are exacty the size
+           scale it down.  But people have images that are exactly the size
            of a page, e.g. because they created them with Sane's 'scanimage'
            program from a full page of input.  So we removed the gratuitous
            5% margin.  -Bryan.
@@ -683,35 +652,21 @@ computeImagePosition(int     const dpiX,
 
 
 
-
 static void
-setupReadstringPsFilter(bool         const rle,
-                        bool         const flate,
-                        bool         const ascii85,
-                        bool         const color,
-                        unsigned int const width) {
-/*----------------------------------------------------------------------------
-   Output to Standard Output Postscript statements to set up the arguments
-   to /readstring for using built in Postscript input filters.
+determineDictionaryRequirement(bool           const userWantsDict,
+                               bool           const psFilter,
+                               unsigned int * const dictSizeP) {
 
-   (These filters exist only in Postscript Level 2 and higher).
-
-   The statements set /picstr and /inputf.
------------------------------------------------------------------------------*/
-    printf("/picstr\n  %u string\n", color ? 3 * width : width);
-    printf("def\n");
-    
-    printf("/inputf\n");
-    printf("  currentfile\n");
-    if (ascii85)
-        printf("  /ASCII85Decode filter\n");
-    else 
-        printf("  /ASCIIHexDecode filter\n");
-    if (rle)
-        printf("  /RunLengthDecode filter\n");
-    if (flate)
-        printf("  /FlateDecode filter\n");
-    printf("def\n");
+    if (userWantsDict) {
+        if (psFilter) {
+            /* The Postscript this program generates to use built-in
+               Postscript filters does not define any variables.
+            */
+            *dictSizeP = 0;
+        } else
+            *dictSizeP = 8;
+    } else
+        *dictSizeP = 0;
 }
 
 
@@ -788,26 +743,33 @@ setupReadstringNative(bool         const rle,
 
 
 static void
-putReadstringPsfilter(unsigned int const postscriptLevel,
-                      bool         const color) {
+putFilters(unsigned int const postscriptLevel,
+           bool         const rle,
+           bool         const flate,
+           bool         const ascii85,
+           bool         const color) {
 
-    if (postscriptLevel > 1) {
-        printf("{ inputf picstr readstring pop }\n");
-        if (color)
-            printf("false 3\n");
-    } else
-        printf("{ currentfile pstr readhexstring pop }\n");
+    assert(postscriptLevel > 1);
+    
+    if (ascii85)
+        printf("/ASCII85Decode filter ");
+    else 
+        printf("/ASCIIHexDecode filter ");
+    if (flate)
+        printf("/FlateDecode filter ");
+    if (rle) /* bmeps encodes rle before flate, so must decode after! */
+        printf("/RunLengthDecode filter ");
 }
 
 
 
 static void
 putReadstringNative(bool const color) {
+
     if (color) {
         printf("{ rpicstr readstring }\n");
         printf("{ gpicstr readstring }\n");
         printf("{ bpicstr readstring }\n");
-        printf("true 3\n");
     } else
         printf("{ picstr readstring }\n");
 }
@@ -815,7 +777,83 @@ putReadstringNative(bool const color) {
 
 
 static void
-putinit(unsigned int const postscriptLevel,
+putSetup(unsigned int const dictSize,
+         bool         const psFilter,
+         bool         const rle,
+         bool         const color,
+         unsigned int const icols,
+         unsigned int const padright,
+         unsigned int const bps) {
+/*----------------------------------------------------------------------------
+   Put the setup section in the Postscript program on Standard Output.
+-----------------------------------------------------------------------------*/
+    printf("%%%%BeginSetup\n");
+
+    if (dictSize > 0)
+        /* inputf {r,g,b,}pictsr readstring readrlestring rlestring */
+        printf("%u dict begin\n", dictSize);
+    
+    if (!psFilter)
+        setupReadstringNative(rle, color, icols, padright, bps);
+
+    printf("%%%%EndSetup\n");
+}
+
+
+
+static void
+putImage(bool const psFilter,
+         bool const color) {
+/*----------------------------------------------------------------------------
+   Put the image/colorimage statement in the Postscript program on
+   Standard Output.
+-----------------------------------------------------------------------------*/
+    if (color) {
+        if (psFilter)
+            printf("false 3\n");
+        else
+            printf("true 3\n");
+        printf("colorimage");
+    } else
+        printf("image");
+}
+
+
+
+static void
+putInitPsFilter(unsigned int const postscriptLevel,
+                bool         const rle,
+                bool         const flate,
+                bool         const ascii85,
+                bool         const color) {
+
+    bool const filterTrue = TRUE;
+
+    printf("{ currentfile ");
+
+    putFilters(postscriptLevel, rle, flate, ascii85, color);
+
+    putImage(filterTrue, color);
+    
+    printf(" } exec");
+}
+
+
+
+static void
+putInitReadstringNative(bool const color) {
+
+    bool const filterFalse = FALSE;
+
+    putReadstringNative(color);
+    
+    putImage(filterFalse, color);
+}
+
+
+
+static void
+putInit(unsigned int const postscriptLevel,
         char         const name[], 
         int          const icols, 
         int          const irows, 
@@ -833,13 +871,20 @@ putinit(unsigned int const postscriptLevel,
         bool         const flate,
         bool         const ascii85,
         bool         const setpage,
-        bool         const psfilter,
-        bool         const dict) {
+        bool         const psFilter,
+        unsigned int const dictSize) {
 /*----------------------------------------------------------------------------
    Write out to Standard Output the headers stuff for the Postscript
    program (everything up to the raster).
 -----------------------------------------------------------------------------*/
-    printf("%%!PS-Adobe-%u.0 EPSF-%u.0\n", postscriptLevel, postscriptLevel);
+    /* The numbers in the %! line often confuse people. They are NOT the
+       PostScript language level.  The first is the level of the DSC comment
+       spec being adhered to, the second is the level of the EPSF spec being
+       adhered to.  It is *incorrect* to claim EPSF compliance if the file
+       contains a setpagedevice.
+    */
+    printf("%%!PS-Adobe-3.0%s\n", setpage ? "" : " EPSF-3.0");
+    printf("%%%%LanguageLevel: %u\n", postscriptLevel);
     printf("%%%%Creator: pnmtops\n");
     printf("%%%%Title: %s.ps\n", name);
     printf("%%%%Pages: 1\n");
@@ -849,19 +894,12 @@ putinit(unsigned int const postscriptLevel,
         (int) (llx + scols + 0.5), (int) (lly + srows + 0.5));
     printf("%%%%EndComments\n");
 
-    if (dict)
-        printf("4 dict begin\n");
-    
-    if (psfilter)
-        setupReadstringPsFilter(rle, flate, ascii85, color, icols);
-    else 
-        setupReadstringNative(rle, color, icols, padright, bps);
+    putSetup(dictSize, psFilter, rle, color, icols, padright, bps);
 
-    printf("%%%%EndProlog\n");
     printf("%%%%Page: 1 1\n");
     if (setpage)
         printf("<< /PageSize [ %d %d ] /ImagingBBox null >> setpagedevice\n",
-                pagewid, pagehgt);
+               pagewid, pagehgt);
     printf("gsave\n");
     printf("%g %g translate\n", llx, lly);
     printf("%g %g scale\n", scols, srows);
@@ -870,48 +908,99 @@ putinit(unsigned int const postscriptLevel,
     printf("%d %d %d\n", icols, irows, bps);
     printf("[ %d 0 0 -%d 0 %d ]\n", icols, irows, irows);
 
-    if (psfilter)
-        putReadstringPsfilter(postscriptLevel, color);
+    if (psFilter)
+        putInitPsFilter(postscriptLevel, rle, flate, ascii85, color);
     else
-        putReadstringNative(color);
+        putInitReadstringNative(color);
 
-    if (color)
-        printf("colorimage\n");
-    else
-        printf("image\n");
+    printf("\n");
 }
 
 
 
 static void
-computeDepth(xelval         const input_maxval, 
+putEnd(bool         const showpage, 
+       bool         const psFilter,
+       bool         const ascii85,
+       unsigned int const dictSize,
+       bool         const vmreclaim) {
+
+    if (psFilter) {
+        if (ascii85)
+            printf("%s\n", "~>");
+        else
+            printf("%s\n", ">");
+    } else {
+        printf("currentdict /inputf undef\n");
+        printf("currentdict /picstr undef\n");
+        printf("currentdict /rpicstr undef\n");
+        printf("currentdict /gpicstr undef\n");
+        printf("currentdict /bpicstr undef\n");
+    }
+
+    if (dictSize > 0)
+        printf("end\n");
+
+    if (vmreclaim)
+        printf("1 vmreclaim\n");
+
+    printf("grestore\n");
+
+    if (showpage)
+        printf("showpage\n");
+    printf("%%%%Trailer\n");
+}
+
+
+
+static void
+computeDepth(xelval         const inputMaxval,
+             unsigned int   const postscriptLevel, 
              unsigned int * const bitspersampleP,
-             unsigned int * const ps_maxvalP) {
+             unsigned int * const psMaxvalP) {
 /*----------------------------------------------------------------------------
    Figure out how many bits will represent each sample in the Postscript
    program, and the maxval of the Postscript program samples.  The maxval
    is just the maximum value allowable in the number of bits.
 -----------------------------------------------------------------------------*/
-    unsigned int const bitsRequiredByMaxval = pm_maxvaltobits(input_maxval);
+    unsigned int const bitsRequiredByMaxval = pm_maxvaltobits(inputMaxval);
 
     if (bitsRequiredByMaxval <= 1)
         *bitspersampleP = 1;
-    else if (bitsRequiredByMaxval > 1 && bitsRequiredByMaxval <= 4)
+    else if (bitsRequiredByMaxval <= 2)
+        *bitspersampleP = 2;
+    else if (bitsRequiredByMaxval <= 4)
         *bitspersampleP = 4;
-    else 
+    else        
         *bitspersampleP = 8;
 
-    if (*bitspersampleP < bitsRequiredByMaxval)
-        pm_message("Maxval of input requires %u bit samples for full "
-                   "resolution, but we are using the Postscript maximum "
-                   "of %u", bitsRequiredByMaxval, *bitspersampleP);
+    /* There is supposedly a 12 bits per pixel Postscript format, but
+       what?  We produce a raster that is composed of bytes, each
+       coded as a pair of hexadecimal characters and representing 8,
+       4, 2, or 1 pixels.  We also have the RLE format, where some of
+       those bytes are run lengths.
+    */
 
-    *ps_maxvalP = pm_bitstomaxval(*bitspersampleP);
+    if (*bitspersampleP < bitsRequiredByMaxval) {
+        if (bitsRequiredByMaxval <= 12 && postscriptLevel >= 2)
+            pm_message("Maxval of input requires %u bit samples for full "
+                       "resolution, and Postscript level %u is capable "
+                       "of representing that many, but this program "
+                       "doesn't know how.  So we are using %u",
+                       bitsRequiredByMaxval, postscriptLevel, *bitspersampleP);
+        else
+            pm_message("Maxval of input requires %u bit samples for full "
+                       "resolution, but we are using the Postscript level %u "
+                       "maximum of %u",
+                       bitsRequiredByMaxval, postscriptLevel, *bitspersampleP);
+    }
+
+    *psMaxvalP = pm_bitstomaxval(*bitspersampleP);
 
     if (verbose)
         pm_message("Input maxval is %u.  Postscript raster will have "
                    "%u bits per sample, so maxval = %u",
-                   input_maxval, *bitspersampleP, *ps_maxvalP);
+                   inputMaxval, *bitspersampleP, *psMaxvalP);
 }    
 
 
@@ -919,7 +1008,7 @@ computeDepth(xelval         const input_maxval,
 static void
 convertRowNative(struct pam * const pamP, 
                  tuple *      const tuplerow, 
-                 unsigned int const ps_maxval, 
+                 unsigned int const psMaxval, 
                  bool         const rle, 
                  unsigned int const padright) {
 
@@ -929,9 +1018,9 @@ convertRowNative(struct pam * const pamP,
         unsigned int col;
         for (col= 0; col < pamP->width; ++col) {
             sample const scaledSample = 
-                pnm_scalesample(tuplerow[col][plane], pamP->maxval, ps_maxval);
+                pnm_scalesample(tuplerow[col][plane], pamP->maxval, psMaxval);
 
-            if ( rle )
+            if (rle)
                 rleputxelval(scaledSample);
             else
                 putxelval(scaledSample);
@@ -951,11 +1040,17 @@ convertRowNative(struct pam * const pamP,
 static void
 convertRowPsFilter(struct pam *     const pamP,
                    tuple *          const tuplerow,
-                   struct bmepsoe * const bmepsoeP) {
+                   struct bmepsoe * const bmepsoeP,
+                   unsigned int     const psMaxval) {
+
+    unsigned int const bitsPerSample = pm_maxvaltobits(psMaxval);
+    unsigned int const stragglers =
+        (((bitsPerSample * pamP->depth) % 8) * pamP->width) % 8;
+        /* Number of bits at the right edge that don't fill out a
+           whole byte
+        */
 
-    unsigned int const psMaxval = 255;
     unsigned int col;
-
     tuple scaledTuple;
     
     scaledTuple = pnm_allocpamtuple(pamP);
@@ -965,7 +1060,12 @@ convertRowPsFilter(struct pam *     const pamP,
         pnm_scaletuple(pamP, scaledTuple, tuplerow[col], psMaxval);
         
         for (plane = 0; plane < pamP->depth; ++plane)
-            outputBmepsSample(bmepsoeP, scaledTuple[plane]);
+            outputBmepsSample(bmepsoeP, scaledTuple[plane], bitsPerSample);
+    }
+    if (stragglers > 0) {
+        unsigned int i;
+        for (i = stragglers; i < 8; ++i)
+            oe_bit_add(bmepsoeP->oeP, 0);
     }
     pnm_freepamtuple(scaledTuple);
 }
@@ -979,6 +1079,7 @@ selectPostscriptLevel(bool           const levelIsGiven,
                       bool           const dict,
                       bool           const flate,
                       bool           const ascii85,
+                      bool           const psFilter,
                       unsigned int * const postscriptLevelP) {
 
     unsigned int const maxPermittedLevel = 
@@ -998,21 +1099,21 @@ selectPostscriptLevel(bool           const levelIsGiven,
         if (2 > maxPermittedLevel)
             pm_error("Color requires at least Postscript level 2");
     }
-    if (dict) {
-        minPossibleLevel = MAX(minPossibleLevel, 2);
-        if (2 > maxPermittedLevel)
-            pm_error("-dict requires at least Postscript level 2");
-    }
     if (flate) {
-        minPossibleLevel = MAX(minPossibleLevel, 2);
+        minPossibleLevel = MAX(minPossibleLevel, 3);
         if (2 > maxPermittedLevel)
-            pm_error("flate compression requires at least Postscript level 2");
+            pm_error("flate compression requires at least Postscript level 3");
     }
     if (ascii85) {
         minPossibleLevel = MAX(minPossibleLevel, 2);
         if (2 > maxPermittedLevel)
             pm_error("ascii85 encoding requires at least Postscript level 2");
     }
+    if (psFilter) {
+        minPossibleLevel = MAX(minPossibleLevel, 2);
+        if (2 > maxPermittedLevel)
+            pm_error("-psfilter requires at least Postscript level 2");
+    }
     if (levelIsGiven)
         *postscriptLevelP = levelGiven;
     else
@@ -1025,7 +1126,7 @@ static void
 convertPage(FILE * const ifP, 
             int    const turnflag, 
             int    const turnokflag, 
-            bool   const psfilter,
+            bool   const psFilter,
             bool   const rle, 
             bool   const flate,
             bool   const ascii85,
@@ -1054,7 +1155,7 @@ convertPage(FILE * const ifP,
            E.g. at 2 bits per sample with 10 columns, this would be 4.
         */
     int row;
-    unsigned int ps_maxval;
+    unsigned int psMaxval;
         /* The maxval of the Postscript program */
     float scols, srows;
     float llx, lly;
@@ -1062,6 +1163,8 @@ convertPage(FILE * const ifP,
     bool color;
     unsigned int postscriptLevel;
     struct bmepsoe * bmepsoeP;
+    unsigned int dictSize;
+        /* Size of Postscript dictionary we should define */
 
     pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
     
@@ -1075,12 +1178,12 @@ convertPage(FILE * const ifP,
     color = STRSEQ(inpam.tuple_type, PAM_PPM_TUPLETYPE);
     
     selectPostscriptLevel(levelIsGiven, levelGiven, color, 
-                          dict, flate, ascii85, &postscriptLevel);
+                          dict, flate, ascii85, psFilter, &postscriptLevel);
     
     if (color)
         pm_message("generating color Postscript program.");
 
-    computeDepth(inpam.maxval, &bitspersample, &ps_maxval);
+    computeDepth(inpam.maxval, postscriptLevel, &bitspersample, &psMaxval);
     {
         unsigned int const realBitsPerLine = inpam.width * bitspersample;
         unsigned int const paddedBitsPerLine = ((realBitsPerLine + 7) / 8) * 8;
@@ -1094,11 +1197,13 @@ convertPage(FILE * const ifP,
                          pagewid, pagehgt, scale, imagewidth, imageheight,
                          equalpixels,
                          &scols, &srows, &llx, &lly, &turned);
+
+    determineDictionaryRequirement(dict, psFilter, &dictSize);
     
-    putinit(postscriptLevel, name, inpam.width, inpam.height, 
+    putInit(postscriptLevel, name, inpam.width, inpam.height, 
             scols, srows, llx, lly, padright, bitspersample, 
             pagewid, pagehgt, color,
-            turned, rle, flate, ascii85, setpage, psfilter, dict);
+            turned, rle, flate, ascii85, setpage, psFilter, dictSize);
 
     createBmepsOutputEncoder(&bmepsoeP, stdout, rle, flate, ascii85);
     initNativeOutputEncoder(rle, bitspersample);
@@ -1107,22 +1212,22 @@ convertPage(FILE * const ifP,
 
     for (row = 0; row < inpam.height; ++row) {
         pnm_readpamrow(&inpam, tuplerow);
-        if (psfilter)
-            convertRowPsFilter(&inpam, tuplerow, bmepsoeP);
+        if (psFilter)
+            convertRowPsFilter(&inpam, tuplerow, bmepsoeP, psMaxval);
         else
-            convertRowNative(&inpam, tuplerow, ps_maxval, rle, padright);
+            convertRowNative(&inpam, tuplerow, psMaxval, rle, padright);
     }
 
     pnm_freepamrow(tuplerow);
 
-    if (psfilter)
+    if (psFilter)
         flushBmepsOutput(bmepsoeP);
     else
         flushNativeOutput(rle);
 
     destroyBmepsOutputEncoder(bmepsoeP);
 
-    putEnd(showpage, psfilter, ascii85, dict, vmreclaim);
+    putEnd(showpage, psFilter, ascii85, dictSize, vmreclaim);
 }
 
 
@@ -1160,23 +1265,23 @@ main(int argc, char * argv[]) {
 
     FILE* ifp;
     const char *name;  /* malloc'ed */
-    struct cmdline_info cmdline;
+    struct cmdlineInfo cmdline;
 
-    pnm_init( &argc, argv );
+    pnm_init(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
     verbose = cmdline.verbose;
 
-    ifp = pm_openr(cmdline.input_filespec);
+    ifp = pm_openr(cmdline.inputFileName);
 
-    if (STREQ(cmdline.input_filespec, "-"))
+    if (STREQ(cmdline.inputFileName, "-"))
         name = strdup("noname");
     else
-        name = basebasename(cmdline.input_filespec);
+        name = basebasename(cmdline.inputFileName);
     {
         int eof;  /* There are no more images in the input file */
-        unsigned int image_seq;
+        unsigned int imageSeq;
 
         /* I don't know if this works at all for multi-image PNM input.
            Before July 2000, it ignored everything after the first image,
@@ -1190,7 +1295,7 @@ main(int argc, char * argv[]) {
         */
 
         eof = FALSE;  /* There is always at least one image */
-        for (image_seq = 0; !eof; image_seq++) {
+        for (imageSeq = 0; !eof; ++imageSeq) {
             convertPage(ifp, cmdline.mustturn, cmdline.canturn, 
                         cmdline.psfilter,
                         cmdline.rle, cmdline.flate, cmdline.ascii85, 
