@@ -1,4 +1,3 @@
-
 /* pbmtog3.c - read a PBM image and produce a Group 3 FAX file
 **
 ** Copyright (C) 1989 by Paul Haeberli <paul@manray.sgi.com>.
@@ -15,14 +14,6 @@
    For specifications for Group 3 (G3) fax MH coding see ITU-T T.4
    This program generates only MH.  It is coded with future expansion for
    MR and MMR in mind.
-
-   This program exploits some i486 instructions to increase computational
-   speed if it knows its target is i486-compatible and that the compiler
-   and assembler are GNU.  To wit, it must find the macros 'i486' and
-   '__GNUC__' defined.  In some environments, i486 is not defined even
-   though the target is i486-compatible.  In that case, add a CADD=-Di486
-   to the Netpbm make command.  Some day, we should make the Netpbm
-   configurator ensure that i486 gets set.
 */
 
 #include <assert.h>
@@ -76,12 +67,14 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *inputFilespec;  /* Filespec of input file */
+    const char * inputFileName;
     unsigned int reversebits;
     unsigned int nofixedwidth;
     unsigned int verbose;
 };
 
+
+
 static void
 parseCommandLine(int argc, char ** const argv,
                  struct cmdlineInfo * const cmdlineP) {
@@ -89,12 +82,14 @@ parseCommandLine(int argc, char ** const argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry option_def[100];
-    /* Instructions to OptParseOptions2 on how to parse our options.  */
+    optEntry * option_def;
+        /* Instructions to OptParseOptions2 on how to parse our options.  */
     optStruct3 opt;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0,   "reversebits",      OPT_FLAG,  NULL, &cmdlineP->reversebits,
             0);
@@ -115,13 +110,12 @@ parseCommandLine(int argc, char ** const argv,
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (argc-1 == 0) 
-        cmdlineP->inputFilespec = "-";
+        cmdlineP->inputFileName = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdlineP->inputFilespec = argv[1];
-
+        cmdlineP->inputFileName = argv[1];
 }
 
 
@@ -143,10 +137,11 @@ makeBs(wordint      const bits,
 
     struct bitString retval;
     retval.intBuffer = bits;
-    retval.bitCount = bitCount;
+    retval.bitCount  = bitCount;
 
     return retval;
 }
+
     
 
 static __inline__ void
@@ -165,7 +160,7 @@ putbits(struct bitString const newBits) {
         sizeof(out.buffer.intBuffer)*8 - out.buffer.bitCount;
         /* Number of bits of unused space (at the high end) in buffer */
 
-    assert(newBits.bitCount < sizeof(out.buffer.intBuffer)*8);
+    assert(newBits.bitCount < sizeof(out.buffer.intBuffer) * 8);
     assert(newBits.intBuffer >> newBits.bitCount == 0);
 
     if (spaceLeft > newBits.bitCount) {
@@ -187,12 +182,12 @@ putbits(struct bitString const newBits) {
                        | (newBits.intBuffer >> nextBufBitCount));
         if (out.reverseBits)
             reversebuffer(outbytes, sizeof(outbytes));
-
+            
         rc = fwrite(outbytes, 1, sizeof(outbytes), stdout);
         if (rc != sizeof(outbytes))
             pm_error("Output error.  Unable to fwrite() to stdout");
         
-        out.buffer.intBuffer = newBits.intBuffer & ((1<<nextBufBitCount)-1); 
+        out.buffer.intBuffer = newBits.intBuffer & ((1<<nextBufBitCount) - 1); 
         out.buffer.bitCount = nextBufBitCount;
     }
 }
@@ -215,36 +210,41 @@ putcode(unsigned int const clr,
     /* Note that this requires ttable to be aligned white entry, black
        entry, white, black, etc.  
     */
-    putbits(makeBs(ttable[ix*2+clr].code, ttable[ix*2+clr].length));
+    putbits(makeBs(ttable[ix * 2 + clr].code, ttable[ix * 2 + clr].length));
 }
 
 
 
 static __inline__ void
-putcode2(int const clr, int const ix) {
+putcode2(int const clr,
+         int const ix) {
 /*----------------------------------------------------------------------------
    Output Make-up code and Terminating code at once.
 
    For run lengths above TC_MC threshold (usually 64).
 
    The codes are combined here to avoid calculations in putbits()
-   (on condition that word is wide enough).
+   wordint is usually wide enough, with 32 or 64 bits.
+   Provisions are made for 16 bit wordint (for debugging).
 
    Terminating code is max 12 bits, Make-up code is max 13 bits.
    (See ttable, mtable entries in pbmtog3.h)
 
    Also reduces object code size when putcode is compiled inline.
 -----------------------------------------------------------------------------*/
-    if (sizeof(wordint) > 24) {
-        unsigned int const l1 = ttable[ix%64*2+clr].length;
+    unsigned int const loIndex = ix % 64 * 2 + clr;
+    unsigned int const hiIndex = ix / 64 * 2 + clr;
+
+    if (sizeof(wordint) * 8 > 24) {
+        unsigned int const l1 = ttable[loIndex].length;
         
         putbits(
-            makeBs(mtable[ix/64*2+clr].code << l1 | ttable[ix%64*2+clr].code,
-                   mtable[ix/64*2+clr].length + l1)
+            makeBs(mtable[hiIndex].code << l1 | ttable[loIndex].code,
+                   mtable[hiIndex].length + l1)
             );
-    } else {
-        putbits(makeBs(mtable[ix/64*2+clr].code, mtable[ix/64*2+clr].length));
-        putbits(makeBs(ttable[ix%64*2+clr].code, ttable[ix%64*2+clr].length));
+    } else { /* typically 16 bit wordint used for debugging */
+        putbits(makeBs(mtable[hiIndex].code, mtable[hiIndex].length));
+        putbits(makeBs(ttable[loIndex].code, ttable[loIndex].length));
     }
 }
 
@@ -291,6 +291,7 @@ putspan(bit          const color,
 }
 
 
+
 static void
 puteol(void) {
 
@@ -327,225 +328,53 @@ puteol(void) {
    r2: c0000000000011111111001111100000 0->carry
   xor: ?0000000000100000001010000100000
 
-  The 1 bits in the xor above are the inflection points.  IA32
-  CPUs provide the BSR operand precisely for getting bit
-  indices in cases like this, so we use that.  For non-IA32 CPUs,
-  we use a table for the same effect.
+  The 1 bits in the xor above are the inflection points.
 */
 
-static bool const fastI386MethodAvailable =
-#if defined(__GNUC__) && defined(i386)
-    TRUE;
-#else
-    FALSE;
-#endif
-
-
-#if defined(__GNUC__) && defined(i386)
-
-#ifndef NDEBUG
-#define CHECK_DS_ES_DIVERGENCE { \
-    short int ds,es; \
-    __asm__("movw %%ds,%0\n\t" "movw %%es,%1\n\t" : "=r" (ds), "=r" (es)); \
-    if (ds != es) pm_error("ds es divergence"); \
-} 
-#else
-#define CHECK_DS_ES_DIVERGENCE
-#endif
-
-#endif
-
-
-static __inline__ void
-convertRowToRunLengths_gcc_i386(unsigned char * const bitrow, 
-                                int             const cols, 
-                                unsigned int *  const milepost,
-                                unsigned int *  const lengthP) {
-
-#if defined(__GNUC__) && defined(i386)
-
-    unsigned int   const bitsPerWord  = sizeof(unsigned int) * 8;
-    unsigned int * const bitrowByWord = (wordint *) bitrow;
-    int            const wordCount = (cols+bitsPerWord-1)/bitsPerWord; 
-        /* Number of full and partial words in the row */
-
-    int nonLeftWhite;
-        /* This is the number of pixels in the row not counting the 
-           contiguous bytes of white pixels at the left, minus 1.
-        */
-
-    if (cols % bitsPerWord != 0) {
-        /* Clean final word in row.  For loop simplicity */
-        wordint r1;
-        r1 = bytesToWordint((unsigned char *)&bitrowByWord[wordCount-1]);
-        r1 >>= bitsPerWord - cols%bitsPerWord;
-        r1 <<= bitsPerWord - cols%bitsPerWord;
-        wordintToBytes((wordintBytes *)&bitrowByWord[wordCount-1], r1);
-    }
-
-    CHECK_DS_ES_DIVERGENCE;
-
-    /* "Fast-forward" reading off of white words from beginning of line.
-       Operand scasl looks at es:edi.  This asm code assumes that es==ds.
-       Works with GNU/Linux EFL and djgpp.  CHECK_FOR_ES_DIVERGENCE
-       makes sure the assumption is valid (but the check is ommitted
-       with -DNDEBUG).
-       
-       edi is not on the clobber list.  (Fails compilation GCC 3.1.3)
-    */
-    __asm__ __volatile__ (
-        "cld\n\t"
-        "repe\n\t"
-        "scasl\n\t"
-        "jnz la0\n\t"
-        "dec %%ecx\n\t"
-        "la0:" 
-        : "=c" (nonLeftWhite) : "a" (0), "c" (wordCount), "D" (bitrowByWord) 
-        );
-
-    if (nonLeftWhite < 0) {
-        /* Whole row is white */
-        milepost[0] = cols; 
-        *lengthP = 1;
-    } else {
-        wordint carry;
-        wordint r1, r2;
-        unsigned int n;
-        int i;
-        int k; /* bit offset */
-
-        for (i = wordCount-(nonLeftWhite+1), carry = n = 0; 
-             i < wordCount; 
-             ++i) {
-            r1 = r2 = bytesToWordint((unsigned char *)&bitrowByWord[i]);
-            r2 = r1 ^ (carry << (bitsPerWord-1) | r2 >> 1);
-            carry = r1 & 0x1;
-            while (r2 != 0) {
-                /* TODO redundant; BSR reports ZF. */
-                __asm__ (
-                    "BSR %0,%1\n\t"
-                    "BTR %1,%0\n\t"
-                    : "=r" (r2), "=r" (k) : "0" (r2) 
-                    );
-                /* reports (k) most significant "1" bit of r2        */ 
-                /* counting from LSB = position 0, MSB = position 31 */
-                milepost[n++] = (i+1)*bitsPerWord - 1 - k;
-            } 
-        }
-        if (milepost[n-1] != cols) 
-            milepost[n++] = cols;
-        *lengthP = n;
-    }
-#endif
-}
-
-
-
-static __inline__ void
-convertRowToRunLengths_general(unsigned char * const bitrow, 
-                               int             const cols, 
-                               unsigned int *  const milepost,
-                               unsigned int *  const lengthP) {
-/*----------------------------------------------------------------------------
-   We access the row one byte at a time and use a table indexed by the
-   256 possible byte values.
-
-   Note that bitrow[] is both an input and output.
------------------------------------------------------------------------------*/
-    int const byteCount = pbm_packed_bytes(cols);
-
-    /* bsrt[x] is the number of leading 0 bits in an 8 bit binary 
-       representation of x.  Except bsrt[0] == 128.
-    */
-    static unsigned char const bsrt[256]= {
-        128, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4,
-          3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
-          2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
-          2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
-          1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-          1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-          1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-          1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 
-    };
-
-    unsigned int allZeroPrefixLen;
-
-    if (cols % 8 != 0) {
-        unsigned char r1;
-        /* Clean final byte.  For loop simplicity */
-        r1 = bitrow[byteCount-1];
-        r1 >>= 8 - cols%8;
-        r1 <<= 8 - cols%8;
-        bitrow[byteCount-1] = r1;
-    }
-
-    for (allZeroPrefixLen = 0; 
-         allZeroPrefixLen < byteCount && bitrow[allZeroPrefixLen] == 0x00; 
-         ++allZeroPrefixLen);
-   
-    if (allZeroPrefixLen == byteCount) {
-        /* All white line */
-        milepost[0] = cols; 
-        *lengthP = 1;
-    } else {
-        unsigned char r1, r2, carry;
-        unsigned int n;
-        unsigned int i;
-        int k;  /* bit offset within word */
-
-        k = n = 0;
-        carry = 0;
-
-        for (i = 0; i < byteCount; ++i) {
-            r1 = r2 = bitrow[i];
-            r2 = r1 ^ (carry << 7 | r2 >> 1);
-            carry = r1 & 0x01;
-            while (r2 != 0) { 
-                k = bsrt[r2];
-                milepost[n++] = i*8 + k;
-                r2 = r2 & ~(0x80 >> k);
-            }
-        }
-
-        if (milepost[n-1] != cols) 
-            milepost[n++] = cols;
-        *lengthP = n;
-    }
-}
-
-
-
 static __inline__ void
 convertRowToRunLengths(unsigned char * const bitrow, 
                        int             const cols, 
                        unsigned int *  const milepost,
                        unsigned int *  const lengthP) {
 
-    if (fastI386MethodAvailable)
-        convertRowToRunLengths_gcc_i386(bitrow, cols, milepost, lengthP);
-    else
-        convertRowToRunLengths_general(bitrow, cols, milepost, lengthP);
-}
+    unsigned int   const bitsPerWord  = sizeof(wordint) * 8;
+    wordint      * const bitrowByWord = (wordint *) bitrow;
+    int            const wordCount    = (cols + bitsPerWord - 1)/bitsPerWord; 
+        /* Number of full and partial words in the row */
+        
 
+    if (cols % bitsPerWord != 0) {
+        /* Clean final word in row.  For loop simplicity */
+        wordint r1;
+        r1 = bytesToWordint((unsigned char *)&bitrowByWord[wordCount - 1]);
+        r1 >>= bitsPerWord - cols % bitsPerWord;
+        r1 <<= bitsPerWord - cols % bitsPerWord;
+        wordintToBytes((wordintBytes *)&bitrowByWord[wordCount - 1], r1);
+    }
+    {
 
+        wordint carry;
+        wordint r1, r2;
+        unsigned int n;
+        int i,c,k;
 
-static void
-clearPadding(unsigned char * const bitrow,
-             int             const cols) { 
-
-    unsigned int i;
-    for (i = pbm_packed_bytes(cols); 
-         i < pbm_packed_bytes(cols) + sizeof(unsigned int);
-         ++i)
-        bitrow[i] = 0x00;
+        for (i = carry = n = 0; i < wordCount; ++i) {
+            r1 = r2 = bytesToWordint((unsigned char *)&bitrowByWord[i]);
+            r2 = r1 ^ (carry << (bitsPerWord-1) | r2 >> 1);
+            carry = r1 & 0x1;  k = 0;
+            while (r2 != 0) {
+                /* wordintClz(r2) reports most significant "1" bit of r2
+                   counting from MSB = position 0.
+                */
+                c = wordintClz(r2);
+                milepost[n++] = i * bitsPerWord + k + c;
+                r2 <<= c++; r2 <<= 1;  k += c; 
+            } 
+        }
+        if (n == 0 || milepost[n - 1] != cols) 
+            milepost[n++] = cols;
+        *lengthP = n;
+    }
 }
 
 
@@ -558,20 +387,21 @@ padToDesiredWidth(unsigned int * const milepost,
 
     if (existingCols < desiredCols) {
         /* adjustment for narrow input in fixed width mode
-           nRun%2==1 (0) means last (=rightmost) pixel is white (black)
+           nRun % 2 == 1 (0) means last (=rightmost) pixel is white (black)
            if white, extend the last span to outwidth
            if black, fill with a white span len (outwidth - readcols)
         */
         if (*nRunP % 2 == 0)
-            ++(*nRunP);
-        milepost[(*nRunP)-1] = desiredCols;
+            ++*nRunP;
+        milepost[*nRunP - 1] = desiredCols;
     }
 }
 
 
 
 int
-main( int argc, char * argv[]) {
+main(int    argc,
+     char * argv[]) {
 
     struct cmdlineInfo cmdline;
     FILE * ifP;
@@ -594,7 +424,7 @@ main( int argc, char * argv[]) {
 
     parseCommandLine(argc, argv, &cmdline);
      
-    ifP = pm_openr(cmdline.inputFilespec);
+    ifP = pm_openr(cmdline.inputFileName);
 
     pbm_readpbminit(ifP, &cols, &rows, &format);
     if (cmdline.nofixedwidth)
@@ -605,7 +435,6 @@ main( int argc, char * argv[]) {
     }
 
     MALLOCARRAY_NOFAIL(bitrow, pbm_packed_bytes(cols) + sizeof(wordint));
-    clearPadding(bitrow, cols);
 
     MALLOCARRAY_NOFAIL(milepost, readcols + 1);
 
@@ -624,7 +453,7 @@ main( int argc, char * argv[]) {
         padToDesiredWidth(milepost, &nRun, readcols, outwidth);
 
         for (i = p = 0; i < nRun; p = milepost[i++])
-            putspan(i%2 == 0 ? PBM_WHITE : PBM_BLACK, milepost[i]-p);
+            putspan(i%2 == 0 ? PBM_WHITE : PBM_BLACK, milepost[i] - p);
         /* TODO 2-dimensional coding MR, MMR */
         puteol();
     }
@@ -639,7 +468,7 @@ main( int argc, char * argv[]) {
         /* flush final partial buffer */
         unsigned int const bytesToWrite = (out.buffer.bitCount+7)/8;
         
-        unsigned char outbytes[4];
+        unsigned char outbytes[sizeof(wordint)];
         size_t rc;
         wordintToBytes(&outbytes, 
                        out.buffer.intBuffer << (sizeof(out.buffer.intBuffer)*8 
@@ -652,5 +481,5 @@ main( int argc, char * argv[]) {
     }
     pm_close(ifP);
 
-    return(0);
+    return 0;
 }
