@@ -1,4 +1,4 @@
-/* pbmtogo.c - read a portable bitmap and produce a GraphOn terminal raster file
+/* pbmtogo.c - read a PBM image and produce a GraphOn terminal raster file
 **	
 **	Rev 1.1 was based on pbmtolj.c
 **
@@ -37,9 +37,10 @@
 
 #include <stdio.h>
 #include <string.h>
+
+#include "pm_c_util.h"
 #include "pbm.h"
 
-#define TRUE 1
 #define BUFSIZE 132	/* GraphOn has 132 byte/1056 bit wide raster lines */
 #define REPEAT_CURRENT_LINE_MASK	0x00 
 #define SKIP_AND_PLOT_MASK		0x40 
@@ -87,6 +88,11 @@ main( argc, argv )
       ifp = stdin;
 
     pbm_readpbminit(ifp, &cols, &rows, &format);
+
+    if (cols > 1056)
+        pm_error("Image is wider (%u pixels) than a Graphon terminal "
+                 "(%u pixels)", cols, 1056);
+
     bitrow = pbm_allocrow(cols);
 
     /* Round cols up to the nearest multiple of 8. */
