@@ -42,6 +42,7 @@
 
 #include <string.h>
 
+#include "pm_c_util.h"
 #include "ppm.h"
 #include "ilbm.h"
 #include "mallocvar.h"
@@ -93,7 +94,6 @@ static short wrotemask = 0;
 static IFF_ID typeid;       /* ID_ILBM, ID_RGBN, ID_RGB8 */
 
 static pixel *transpColor = NULL;       /* transparent color */
-static short transpIndex = -1;
 static char *transpName = NULL;
 
 static bool debug = FALSE;
@@ -775,15 +775,19 @@ check_cmap(bmhd, cmap)
     if( bmhd ) {
         if( bmhd->masking == mskHasTransparentColor || 
             bmhd->masking == mskLasso ) {
-            transpIndex = bmhd->transparentColor;
+            unsigned short const transpIdx = bmhd->transparentColor;
             if( !transpName ) {
                 MALLOCVAR_NOFAIL(transpColor);
-                if( transpIndex >= cmap->ncolors ) {
-                    pm_message("using default transparent color (black)");
-                    PPM_ASSIGN(*transpColor, 0, 0, 0);
+                if (HAS_COLORMAP(cmap)) {
+                    if( transpIdx >= cmap->ncolors ) {
+                        pm_message("using default transparent color (black)");
+                        PPM_ASSIGN(*transpColor, 0, 0, 0);
+                    } else
+                        *transpColor = cmap->color[transpIdx];
+                } else {
+                    /* The color index is just a direct gray level */
+                    PPM_ASSIGN(*transpColor, transpIdx, transpIdx, transpIdx);
                 }
-                else
-                    *transpColor = cmap->color[transpIndex];
             }
         }
 
@@ -1674,7 +1678,7 @@ PCHG_Decompress(PCHG, CompHdr, compdata, compsize, comptree, data)
 #endif
             /* turn big-endian 2-byte shorts into native format */
             huffsize = treesize/2;
-            MALLOCVAR_NOFAIL(hufftree);
+            MALLOCARRAY_NOFAIL(hufftree, huffsize);
             for( i = 0; i < huffsize; i++ ) {
                 hufftree[i] = (short)BIG_WORD(comptree);
                 comptree += 2;
@@ -1931,7 +1935,7 @@ read_pchg(FILE *     const ifp,
                 get_big_long(ifp, iffid, &remainingChunksize);
 
             treesize = CompHdr.CompInfoSize;
-            MALLOCVAR_NOFAIL(comptree);
+            MALLOCARRAY_NOFAIL(comptree, treesize);
             read_bytes(ifp, treesize, comptree, iffid, &remainingChunksize);
 
             compsize = remainingChunksize;
@@ -2348,7 +2352,7 @@ main(int argc, char *argv[]) {
             cmap_to_ppm(cmap);
         } else
             pm_error("no colormap");
-    } else if( bodyChunkProcessed ) {
+    } else if( !bodyChunkProcessed ) {
         if( HAS_COLORMAP(cmap) ) {
             pm_message("input is a colormap file");
             check_cmap(bmhdP, cmap);
