@@ -51,12 +51,8 @@ static bool verbose;
  */
 typedef int code_int;
 
-#ifdef SIGNED_COMPARE_SLOW
-typedef unsigned long int count_int;
-typedef unsigned short int count_short;
-#else /*SIGNED_COMPARE_SLOW*/
 typedef long int          count_int;
-#endif /*SIGNED_COMPARE_SLOW*/
+
 
 struct cmap {
     /* This is the information for the GIF colormap (aka palette). */
@@ -87,21 +83,6 @@ struct cmap {
         /* The maxval for the colors in 'cht'. */
 };
 
-struct gif_dest {
-    /* This structure controls output of uncompressed GIF raster */
-
-    /* State for packing variable-width codes into a bitstream */
-    int n_bits;         /* current number of bits/code */
-    int maxcode;        /* maximum code, given n_bits */
-    int cur_accum;      /* holds bits not yet output */
-    int cur_bits;       /* # of bits in cur_accum */
-
-    /* State for GIF code assignment */
-    int ClearCode;      /* clear code (doesn't change) */
-    int EOFCode;        /* EOF code (ditto) */
-    int code_counter;   /* counts output symbols */
-};
-
 struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
@@ -548,23 +529,15 @@ write_comment_extension(FILE *fp, const char comment[]) {
 
 #define ARGVAL() (*++(*argv) || (--argc && *++argv))
 
-static int n_bits;                        /* number of bits/code */
-static int maxbits = BITS;                /* user settable max # bits/code */
-static code_int maxcode;                  /* maximum code, given n_bits */
-static code_int maxmaxcode = (code_int)1 << BITS; /* should NEVER generate this code */
-#ifdef COMPATIBLE               /* But wrong! */
-# define MAXCODE(n_bits)        ((code_int) 1 << (n_bits) - 1)
-#else /*COMPATIBLE*/
-# define MAXCODE(n_bits)        (((code_int) 1 << (n_bits)) - 1)
-#endif /*COMPATIBLE*/
+static code_int const maxmaxcode = (code_int)1 << BITS;
+    /* should NEVER generate this code */
+#define MAXCODE(n_bits)        (((code_int) 1 << (n_bits)) - 1)
 
-static count_int htab [HSIZE];
+static long htab [HSIZE];
 static unsigned short codetab [HSIZE];
 #define HashTabOf(i)       htab[i]
 #define CodeTabOf(i)    codetab[i]
 
-static code_int hsize = HSIZE;                 /* for dynamic table sizing */
-
 /*
  * To save much memory, we overlay the table used by compress() with those
  * used by decompress().  The tab_prefix table is the same size and type
@@ -606,8 +579,6 @@ static long int out_count = 0;           /* # of codes output (for debugging) */
  * questions about this implementation to ames!jaw.
  */
 
-static int g_init_bits;
-
 static int ClearCode;
 static int EOFCode;
 
@@ -615,208 +586,274 @@ static int EOFCode;
 *                          BYTE OUTPUTTER                 
 ***************************************************************************/
 
-struct outputter {
+typedef struct {
     FILE * fileP;  /* The file to which to output */
     unsigned int count;
         /* Number of bytes so far in the current data block */
     unsigned char buffer[256];
         /* The current data block, under construction */
-};
-
-static struct outputter outputter;
+} byteBuffer;
 
 
 
-static void
-outputter_init(FILE * const fileP) {
-    outputter.fileP = fileP;
-    outputter.count = 0;
+static byteBuffer *
+byteBuffer_create(FILE * const fileP) {
+
+    byteBuffer * byteBufferP;
+
+    MALLOCVAR_NOFAIL(byteBufferP);
+
+    byteBufferP->fileP = fileP;
+    byteBufferP->count = 0;
+
+    return byteBufferP;
 }
 
 
 
 static void
-outputter_flush() {
+byteBuffer_destroy(byteBuffer * const byteBufferP) {
+
+    free(byteBufferP);
+}
+
+
+
+static void
+byteBuffer_flush(byteBuffer * const byteBufferP) {
 /*----------------------------------------------------------------------------
    Write the current data block to the output file, then reset the current 
    data block to empty.
 -----------------------------------------------------------------------------*/
-    if (outputter.count > 0 ) {
+    if (byteBufferP->count > 0 ) {
         if (verbose)
-            pm_message("Writing %u byte block", outputter.count);
-        fputc(outputter.count, outputter.fileP);
-        fwrite(outputter.buffer, 1, outputter.count, outputter.fileP);
-        outputter.count = 0;
+            pm_message("Writing %u byte block", byteBufferP->count);
+        fputc(byteBufferP->count, byteBufferP->fileP);
+        fwrite(byteBufferP->buffer, 1, byteBufferP->count, byteBufferP->fileP);
+        byteBufferP->count = 0;
     }
 }
 
 
 
 static void
-outputter_flushFile() {
+byteBuffer_flushFile(byteBuffer * const byteBufferP) {
     
-    fflush(outputter.fileP);
+    fflush(byteBufferP->fileP);
     
-    if (ferror(outputter.fileP))
+    if (ferror(byteBufferP->fileP))
         pm_error("error writing output file");
 }
 
 
 
 static void
-outputter_out(unsigned char const c) {
+byteBuffer_out(byteBuffer *  const byteBufferP,
+               unsigned char const c) {
 /*----------------------------------------------------------------------------
   Add a byte to the end of the current data block, and if it is now 254
   characters, flush the data block to the output file.
 -----------------------------------------------------------------------------*/
-    outputter.buffer[outputter.count++] = c;
-    if (outputter.count >= 254)
-        outputter_flush();
+    byteBufferP->buffer[byteBufferP->count++] = c;
+    if (byteBufferP->count >= 254)
+        byteBuffer_flush(byteBufferP);
 }
 
 
 
+struct gif_dest {
+    /* This structure controls output of uncompressed GIF raster */
+
+    byteBuffer * byteBufferP;  /* Where the full bytes go */
+
+    /* State for packing variable-width codes into a bitstream */
+    int n_bits;         /* current number of bits/code */
+    int maxcode;        /* maximum code, given n_bits */
+    int cur_accum;      /* holds bits not yet output */
+    int cur_bits;       /* # of bits in cur_accum */
+
+    /* State for GIF code assignment */
+    int ClearCode;      /* clear code (doesn't change) */
+    int EOFCode;        /* EOF code (ditto) */
+    int code_counter;   /* counts output symbols */
+};
+
+
+
 static unsigned long const masks[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F,
                                        0x001F, 0x003F, 0x007F, 0x00FF,
                                        0x01FF, 0x03FF, 0x07FF, 0x0FFF,
                                        0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };
 
+typedef struct {
+    byteBuffer * byteBufferP;
+    unsigned int initBits;
+    unsigned int n_bits;                        /* number of bits/code */
+    code_int maxcode;                  /* maximum code, given n_bits */
+    unsigned long curAccum;
+    int curBits;
+} codeBuffer;
+
+
+
+static codeBuffer *
+codeBuffer_create(FILE *       const ofP,
+                  unsigned int const initBits) {
+
+    codeBuffer * codeBufferP;
+
+    MALLOCVAR_NOFAIL(codeBufferP);
+
+    codeBufferP->initBits    = initBits;
+    codeBufferP->n_bits      = codeBufferP->initBits;
+    codeBufferP->maxcode     = MAXCODE(codeBufferP->n_bits);
+    codeBufferP->byteBufferP = byteBuffer_create(ofP);
+    codeBufferP->curAccum    = 0;
+    codeBufferP->curBits     = 0;
+
+    return codeBufferP;
+}
+
+
+
 static void
-output(code_int const code) {
+codeBuffer_destroy(codeBuffer * const codeBufferP) {
+
+    byteBuffer_destroy(codeBufferP->byteBufferP);
+
+    free(codeBufferP);
+}
+
+
+
+static void
+codeBuffer_output(codeBuffer * const codeBufferP,
+                  code_int     const code) {
 /*----------------------------------------------------------------------------
-   Output one GIF code to the file.
+   Output one GIF code to the file, through the code buffer.
 
-   The code is represented as cur_bits+n_bits/8 bytes in the file.
+   The code is represented as n_bits bits in the file -- the lower
+   n_bits bits of 'code'.
 
-   Inputs:
-      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
-              that n_bits =< (long)wordsize - 1.
-   Assumptions:
-      Chars are 8 bits long.
+   If the code is EOF, flush the code buffer to the file.
+
+   In some cases, change n_bits and recalculate maxcode to go with it.
 -----------------------------------------------------------------------------*/
     /*
       Algorithm:
-        Maintain a BITS character long buffer (so that 8 codes will
-        fit in it exactly).  Use the VAX insv instruction to insert each
-        code in turn.  When the buffer fills up empty it and start over.
+      Maintain a BITS character long buffer (so that 8 codes will
+      fit in it exactly).  Use the VAX insv instruction to insert each
+      code in turn.  When the buffer fills up empty it and start over.
     */
     
-    /* Note: this initialization of static variables by the loader must go
-       if we want to use the compressor more than once per invocation
-       (e.g. we want to create a multi-image gif).
-    */
-    static unsigned long curAccum;
-    static int curBits;
+    codeBufferP->curAccum &= masks[codeBufferP->curBits];
 
-    curBits = 0;  /* initial value */
-    curAccum = 0;  /* initial value */
-    curAccum &= masks[curBits];
-
-    if (curBits > 0)
-        curAccum |= ((long)code << curBits);
+    if (codeBufferP->curBits > 0)
+        codeBufferP->curAccum |= ((long)code << codeBufferP->curBits);
     else
-        curAccum = code;
+        codeBufferP->curAccum = code;
 
-    curBits += n_bits;
+    codeBufferP->curBits += codeBufferP->n_bits;
 
-    while (curBits >= 8) {
-        outputter_out((unsigned int)(curAccum & 0xff));
-        curAccum >>= 8;
-        curBits -= 8;
+    while (codeBufferP->curBits >= 8) {
+        byteBuffer_out(codeBufferP->byteBufferP,
+                       codeBufferP->curAccum & 0xff);
+        codeBufferP->curAccum >>= 8;
+        codeBufferP->curBits -= 8;
     }
 
-    /* If the next entry is going to be too big for the code size,
-       then increase it, if possible.
-    */
-   if (free_ent > maxcode || clear_flg) {
-       if (clear_flg) {
-           maxcode = MAXCODE (n_bits = g_init_bits);
-           clear_flg = 0;
-       } else {
-           ++n_bits;
-           if (n_bits == maxbits)
-               maxcode = maxmaxcode;
-           else
-               maxcode = MAXCODE(n_bits);
-       }
-   }
-   
-   if (code == EOFCode) {
-       /* We're at EOF.  Write the rest of the buffer. */
-       while (curBits > 0) {
-           outputter_out((unsigned int)(curAccum & 0xff));
-           curAccum >>= 8;
-           curBits -= 8;  
-           /* ?? what if cur_bits < 8?  Apparently only an issue 
-              if we extend this program to do multiple images */
-       }
-       outputter_flush();
-       
-       outputter_flushFile();
-   }
+    if (clear_flg) {
+        codeBufferP->n_bits = codeBufferP->initBits;
+        codeBufferP->maxcode = MAXCODE(codeBufferP->n_bits);
+        clear_flg = 0;
+    } else if (free_ent > codeBufferP->maxcode) {
+        /* The next entry is going to be too big for the code size, so
+           increase it, if possible.
+        */
+        ++codeBufferP->n_bits;
+        if (codeBufferP->n_bits == BITS)
+            codeBufferP->maxcode = maxmaxcode;
+        else
+            codeBufferP->maxcode = MAXCODE(codeBufferP->n_bits);
+    }
+    
+    if (code == EOFCode) {
+        /* We're at EOF.  Output the possible partial byte in the buffer */
+        if (codeBufferP->curBits > 0) {
+            byteBuffer_out(codeBufferP->byteBufferP,
+                           codeBufferP->curAccum & 0xff);
+            codeBufferP->curBits = 0;
+        }
+        byteBuffer_flush(codeBufferP->byteBufferP);
+        
+        byteBuffer_flushFile(codeBufferP->byteBufferP);
+    }
 }
 
 
 
 static void
-cl_hash(hsize)          /* reset code table */
-register count_int hsize;
-{
+cl_hash(long const hsize) {
+    /* reset code table */
 
-        register count_int *htab_p = htab+hsize;
+    long const m1 = -1;
 
-        register long i;
-        register long m1 = -1;
+    long * htab_p;
+    long i;
 
-        i = hsize - 16;
-        do {                            /* might use Sys V memset(3) here */
-                *(htab_p-16) = m1;
-                *(htab_p-15) = m1;
-                *(htab_p-14) = m1;
-                *(htab_p-13) = m1;
-                *(htab_p-12) = m1;
-                *(htab_p-11) = m1;
-                *(htab_p-10) = m1;
-                *(htab_p-9) = m1;
-                *(htab_p-8) = m1;
-                *(htab_p-7) = m1;
-                *(htab_p-6) = m1;
-                *(htab_p-5) = m1;
-                *(htab_p-4) = m1;
-                *(htab_p-3) = m1;
-                *(htab_p-2) = m1;
-                *(htab_p-1) = m1;
-                htab_p -= 16;
-        } while ((i -= 16) >= 0);
+    htab_p = htab + hsize;  /* initial value */
 
-        for ( i += 16; i > 0; --i )
-                *--htab_p = m1;
+    i = hsize - 16;
+    do {                            /* might use Sys V memset(3) here */
+        *(htab_p-16) = m1;
+        *(htab_p-15) = m1;
+        *(htab_p-14) = m1;
+        *(htab_p-13) = m1;
+        *(htab_p-12) = m1;
+        *(htab_p-11) = m1;
+        *(htab_p-10) = m1;
+        *(htab_p-9) = m1;
+        *(htab_p-8) = m1;
+        *(htab_p-7) = m1;
+        *(htab_p-6) = m1;
+        *(htab_p-5) = m1;
+        *(htab_p-4) = m1;
+        *(htab_p-3) = m1;
+        *(htab_p-2) = m1;
+        *(htab_p-1) = m1;
+        htab_p -= 16;
+    } while ((i -= 16) >= 0);
+
+    for (i += 16; i > 0; --i)
+        *--htab_p = m1;
 }
 
 
-/*
- * Clear out the hash table
- */
-static void
-cl_block() {
-    /* table clear for block compress */
 
-    cl_hash ( (count_int) hsize );
+static void
+cl_block(codeBuffer * const codeBufferP) {
+/*----------------------------------------------------------------------------
+  Clear out the hash table
+-----------------------------------------------------------------------------*/
+    cl_hash(HSIZE);
     free_ent = ClearCode + 2;
     clear_flg = 1;
     
-    output((code_int)ClearCode);
+    codeBuffer_output(codeBufferP, (code_int)ClearCode);
 }
 
 
 
 static void
-write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
-                 gray ** const alpha, gray const alpha_maxval, 
+write_raster_LZW(pixel **      const pixels,
+                 pixval        const input_maxval,
+                 gray **       const alpha,
+                 gray          const alpha_maxval, 
                  struct cmap * const cmapP, 
-                 int const init_bits, FILE * const outfile) {
+                 int           const initBits,
+                 FILE *        const ofP) {
 /*----------------------------------------------------------------------------
-   Write the raster to file 'outfile'.
+   Write the raster to file 'ofP'.
 
    The raster to write is 'pixels', which has maxval 'input_maxval',
    modified by alpha mask 'alpha', which has maxval 'alpha_maxval'.
@@ -826,22 +863,18 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
 
    Write the raster using LZW compression.
 -----------------------------------------------------------------------------*/
-
-    code_int ent;
-    code_int disp;
-    code_int hsize_reg;
-    int hshift;
-    bool eof;
     gray const alpha_threshold = (alpha_maxval + 1) / 2;
         /* gray levels below this in the alpha mask indicate transparent
            pixels in the output image.
         */
+    code_int ent;
+    code_int disp;
+    int hshift;
+    bool eof;
+    codeBuffer * codeBufferP;
+    
+    codeBufferP = codeBuffer_create(ofP, initBits);
     
-    /*
-     * Set up the globals:  g_init_bits - initial number of bits
-     */
-    g_init_bits = init_bits;
-
     /*
      * Set up the necessary values
      */
@@ -849,27 +882,23 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
     out_count = 0;
     clear_flg = 0;
     in_count = 1;
-    maxcode = MAXCODE(n_bits = g_init_bits);
 
-    ClearCode = (1 << (init_bits - 1));
+    ClearCode = (1 << (initBits - 1));
     EOFCode = ClearCode + 1;
     free_ent = ClearCode + 2;
 
-    outputter_init(outfile);
-
     ent = gifNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
 
     {
         long fcode;
         hshift = 0;
-        for ( fcode = (long) hsize;  fcode < 65536L; fcode *= 2L )
+        for (fcode = HSIZE; fcode < 65536L; fcode *= 2L)
             ++hshift;
         hshift = 8 - hshift;                /* set hash code range bound */
     }
-    hsize_reg = hsize;
-    cl_hash( (count_int) hsize_reg);            /* clear hash table */
+    cl_hash(HSIZE);            /* clear hash table */
 
-    output((code_int)ClearCode);
+    codeBuffer_output(codeBufferP, (code_int)ClearCode);
 
     eof = FALSE;
     while (!eof) {
@@ -881,7 +910,7 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
                                 input_maxval, alpha, alpha_threshold, cmapP);
         if (gifpixel == EOF) eof = TRUE;
         if (!eof) {
-            long const fcode = (long) (((long) gifpixel << maxbits) + ent);
+            long const fcode = (long) (((long) gifpixel << BITS) + ent);
             code_int i;
                 /* xor hashing */
 
@@ -889,41 +918,41 @@ write_raster_LZW(pixel ** const pixels, pixval const input_maxval,
 
             i = (((code_int)gifpixel << hshift) ^ ent);    
 
-            if ( HashTabOf (i) == fcode ) {
+            if (HashTabOf (i) == fcode) {
                 ent = CodeTabOf (i);
                 continue;
-            } else if ( (long)HashTabOf (i) < 0 )      /* empty slot */
+            } else if ((long)HashTabOf(i) < 0)      /* empty slot */
                 goto nomatch;
-            disp = hsize_reg - i;        /* secondary hash (after G. Knott) */
-            if ( i == 0 )
+            disp = HSIZE - i;        /* secondary hash (after G. Knott) */
+            if (i == 0)
                 disp = 1;
         probe:
-            if ( (i -= disp) < 0 )
-                i += hsize_reg;
+            if ((i -= disp) < 0)
+                i += HSIZE;
 
-            if ( HashTabOf (i) == fcode ) {
-                ent = CodeTabOf (i);
+            if (HashTabOf(i) == fcode) {
+                ent = CodeTabOf(i);
                 continue;
             }
-            if ( (long)HashTabOf (i) > 0 )
+            if ((long)HashTabOf(i) > 0)
                 goto probe;
         nomatch:
-            output((code_int)ent);
+            codeBuffer_output(codeBufferP, (code_int)ent);
             ++out_count;
             ent = gifpixel;
-            if ( free_ent < maxmaxcode ) {  /* } */
-                CodeTabOf (i) = free_ent++; /* code -> hashtable */
-                HashTabOf (i) = fcode;
+            if (free_ent < maxmaxcode) {
+                CodeTabOf(i) = free_ent++; /* code -> hashtable */
+                HashTabOf(i) = fcode;
             } else
-                cl_block(outfile);
+                cl_block(codeBufferP);
         }
     }
-    /*
-     * Put out the final code.
-     */
-    output((code_int)ent);
+    /* Put out the final code. */
+    codeBuffer_output(codeBufferP, (code_int)ent);
     ++out_count;
-    output((code_int) EOFCode);
+    codeBuffer_output(codeBufferP, (code_int) EOFCode);
+
+    codeBuffer_destroy(codeBufferP);
 }
 
 
@@ -940,7 +969,7 @@ outputUncompressed(struct gif_dest * const dinfoP,
     dinfoP->cur_bits += dinfoP->n_bits;
 
     while (dinfoP->cur_bits >= 8) {
-        outputter_out(dinfoP->cur_accum & 0xFF);
+        byteBuffer_out(dinfoP->byteBufferP, dinfoP->cur_accum & 0xFF);
         dinfoP->cur_accum >>= 8;
         dinfoP->cur_bits -= 8;
     }
@@ -955,8 +984,6 @@ writeRasterUncompressedInit(FILE *            const ofP,
    Initialize pseudo-compressor
 -----------------------------------------------------------------------------*/
 
-    outputter_init(ofP);
-
     /* init all the state variables */
     dinfoP->n_bits = i_bits;
     dinfoP->maxcode = MAXCODE(dinfoP->n_bits);
@@ -964,6 +991,7 @@ writeRasterUncompressedInit(FILE *            const ofP,
     dinfoP->EOFCode = dinfoP->ClearCode + 1;
     dinfoP->code_counter = dinfoP->ClearCode + 2;
     /* init output buffering vars */
+    dinfoP->byteBufferP = byteBuffer_create(ofP);
     dinfoP->cur_accum = 0;
     dinfoP->cur_bits = 0;
     /* GIF specifies an initial Clear code */
@@ -1002,9 +1030,11 @@ writeRasterUncompressedTerm(struct gif_dest * const dinfoP) {
     outputUncompressed(dinfoP, dinfoP->EOFCode);
 
     if (dinfoP->cur_bits > 0)
-        outputter_out(dinfoP->cur_accum & 0xFF);
+        byteBuffer_out(dinfoP->byteBufferP, dinfoP->cur_accum & 0xFF);
 
-    outputter_flush();
+    byteBuffer_flush(dinfoP->byteBufferP);
+
+    byteBuffer_destroy(dinfoP->byteBufferP);
 }
 
 
@@ -1033,7 +1063,6 @@ writeRasterUncompressed(FILE *         const ofP,
     struct gif_dest gifDest;
 
     writeRasterUncompressedInit(ofP, &gifDest, initBits);
-    pm_message("before raster, position is %lu", ftell(outputter.fileP));
 
     eof = FALSE;
     while (!eof) {
@@ -1049,7 +1078,6 @@ writeRasterUncompressed(FILE *         const ofP,
             writeRasterUncompressedPixel(&gifDest, gifpixel);
     }
     writeRasterUncompressedTerm(&gifDest);
-    pm_message("after writing raster, position is %lu", ftell(outputter.fileP));
 }
 
 
@@ -1175,6 +1203,14 @@ gifEncode(FILE *        const ofP,
     unsigned int const initCodeSize = bitsPerPixel <= 1 ? 2 : bitsPerPixel;
         /* The initial code size */
 
+    if (gWidth > 65535)
+        pm_error("Image width %u too large for GIF format.  (Max 65535)",
+                 gWidth);  
+    
+    if (gHeight > 65535)
+        pm_error("Image height %u too large for GIF format.  (Max 65535)",
+                 gHeight);  
+
     writeGifHeader(ofP, gWidth, gHeight, gInterlace, background,
                    bitsPerPixel, cmapP, comment);
 
