@@ -7,11 +7,15 @@
 
 /* COPYRIGHT INFORMATION IS AT THE END OF THIS FILE */
 
-#define _XOPEN_SOURCE 1
+#define _XOPEN_SOURCE 500
     /* This makes sure popen() is in stdio.h.  In GNU libc 2.1.3, 
      _POSIX_C_SOURCE = 2 is sufficient, but on AIX 4.3, the higher level
      _XOPEN_SOURCE is required.  2000.09.09 
+
+     This also makes sure strdup() is in string.h.
     */
+#define _BSD_SOURCE 1
+    /* Make sure string.h defines strncasecmp */
 
 
 /*==============*
@@ -22,6 +26,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "nstring.h"
 #include "mallocvar.h"
@@ -89,23 +94,6 @@
 
 #define NUM_OPTIONS           33
 
-/*=======================*
- * STRUCTURE DEFINITIONS *
- *=======================*/
-
-typedef struct InputFileEntryStruct {
-    char    left[256];
-    char    right[256];
-    boolean glob;	    /* if FALSE, left is complete name */
-    int	    startID;
-    int	    endID;
-    int	    skip;
-    int	    numPadding;	    /* -1 if there is none */
-    int	    numFiles;
-    boolean repeat;
-} InputFileEntry;
-
-
 /*==================*
  * GLOBAL VARIABLES *
  *==================*/
@@ -295,8 +283,8 @@ GetFrameRate(const char * const p)
 
 
 static void
-mergeInputSource(struct inputSource * const baseSourceP,
-                 struct inputSource * const addedSourceP) {
+mergeInputSource(struct inputSource *       const baseSourceP,
+                 const struct inputSource * const addedSourceP) {
 
     unsigned int i;
 
@@ -306,12 +294,6 @@ mergeInputSource(struct inputSource * const baseSourceP,
     for (i = 0; i < addedSourceP->numInputFileEntries; ++i)
         baseSourceP->inputFileEntries[baseSourceP->numInputFileEntries++] =
             addedSourceP->inputFileEntries[i];
-
-    free(addedSourceP);
-    /* Note the space allocated for the *addedSourceP input file
-       entries themselves is still allocated, and used by 
-       *baseSourceP.
-    */
 }
 
 
@@ -558,7 +540,12 @@ setInputSource(int                   const function,
                struct inputSource *  const inputSourceP,
                struct inputSource *  const gopInputSourceP,
                struct inputSource *  const frameInputSourceP) {
-
+/*----------------------------------------------------------------------------
+   Given three the three input sources described by the parameter
+   file, 'inputSourceP', 'gopInputSourceP', and 'frameInputSourceP',
+   return as *inputSourcePP the appropriate one of them for the
+   function 'function', and destroy the other two.
+-----------------------------------------------------------------------------*/
     switch (function) {
     case ENCODE_FRAMES:
         *inputSourcePP = inputSourceP;
@@ -587,6 +574,360 @@ setInputSource(int                   const function,
  *=====================*/
 
 
+
+static void
+removeTrailingWhite(const char *  const rawLine,
+                    const char ** const editedLineP) {
+
+    char * buffer;
+    char * p;
+
+    buffer = strdup(rawLine);
+
+    if (!buffer)
+        pm_error("Unable to get memory to process parameter file");
+
+    p = buffer + strlen(buffer) - 1;  /* Point to last character */
+    
+    /* Position p to just before the trailing white space (might be one
+       character before beginning of string!)
+    */
+    while (p >= buffer && isspace(*p))
+        --p;
+    
+    ++p;  /* Move to first trailing whitespace character */
+    
+    *p = '\0';  /* Chop off all the trailing whitespace */
+
+    *editedLineP = buffer;
+}
+
+
+
+static void
+readNiqTable(FILE * const fpointer) {
+
+    unsigned int row;
+    for (row = 0; row < 8; ++row) {
+        char input[256];
+        unsigned int col;
+        fgets(input, sizeof(input), fpointer);
+        if (8 != sscanf(input, "%d %d %d %d %d %d %d %d",
+                        &niqtable[row*8+0],  &niqtable[row*8+1],
+                        &niqtable[row*8+2],  &niqtable[row*8+3],
+                        &niqtable[row*8+4],  &niqtable[row*8+5],
+                        &niqtable[row*8+6],  &niqtable[row*8+7])) {
+            pm_error("Line %d of NIQTABLE doesn't have 8 elements!", 
+                     row);
+        }
+        for ( col = 0; col < 8; col++ ) {
+            if ((niqtable[row*8+col]<1) || (niqtable[row*8+col]>255)) {
+                pm_message(
+                    "Warning:  NIQTable Element %1d,%1d (%d) "
+                    "corrected to 1-255.",
+                    row + 1, col + 1, niqtable[row * 8 + col]);
+                niqtable[row * 8 + col] = (niqtable[row*8+col] < 1) ? 1 : 255;
+            }
+        }
+    }
+
+    customNIQtable = niqtable;
+}
+
+
+
+static void
+processRanges(const char * const arg) {
+
+    int numRanges;
+    int a, b;
+
+    numRanges = sscanf(arg, "%d %d", &a, &b);
+
+    if (numRanges == 2)
+        SetSearchRange(a, b);
+    else if (sscanf(arg, "%d [%d]", &a, &b) == 2)
+        SetSearchRange(a, b);
+    else
+        SetSearchRange(a, a);
+}
+
+
+
+static void
+processParamLine(char const input[],
+                 FILE * const fpointer,
+                 bool * const yuvUsedP,
+                 struct inputSource * const inputSourceP,
+                 struct inputSource * const frameInputSourceP,
+                 struct inputSource * const gopInputSourceP,
+                 struct params * const paramP) {
+
+    switch(input[0]) {
+    case 'A':
+        if (STRNEQ(input, "ASPECT_RATIO", 12)) {
+            aspectRatio = GetAspectRatio(SkipSpacesTabs(&input[12]));
+            optionSeen[OPTION_ASPECT_RATIO] = TRUE;
+        }
+        break;
+        
+    case 'B':
+        if (STRNEQ(input, "BQSCALE", 7)) {
+            SetBQScale(atoi(SkipSpacesTabs(&input[7])));
+            optionSeen[OPTION_BQSCALE] = TRUE;
+        } else if (STRNEQ(input, "BASE_FILE_FORMAT", 16)) {
+            const char * arg = SkipSpacesTabs(&input[16]);
+            SetFileFormat(arg);
+            if (STRNEQ(arg, "YUV", 3) || STREQ(arg, "Y"))
+                *yuvUsedP = TRUE;
+            optionSeen[OPTION_BASE_FORMAT] = TRUE;
+        } else if (STRNEQ(input, "BSEARCH_ALG", 11)) {
+            SetBSearchAlg(SkipSpacesTabs(&input[11]));
+            optionSeen[OPTION_BSEARCH_ALG] = TRUE;
+        } else if (STRNEQ(input, "BIT_RATE", 8)) {
+            setBitRate(SkipSpacesTabs(&input[8]));
+            optionSeen[OPTION_BIT_RATE] = TRUE;
+        } else if (STRNEQ(input, "BUFFER_SIZE", 11)) {
+            setBufferSize(SkipSpacesTabs(&input[11]));
+            optionSeen[OPTION_BUFFER_SIZE] = TRUE;                  
+        }
+        break;
+
+    case 'C':
+        if (STRNEQ(input, "CDL_FILE", 8)) {
+            strcpy(specificsFile, SkipSpacesTabs(&input[8]));
+            specificsOn = TRUE;
+            optionSeen[OPTION_SPECIFICS] = TRUE;
+        } else if (STRNEQ(input, "CDL_DEFINES", 11)) {
+            strcpy(specificsDefines, SkipSpacesTabs(&input[11]));
+            optionSeen[OPTION_DEFS_SPECIFICS] = TRUE;
+        }
+        break;
+
+    case 'F':
+        if (STRNEQ(input, "FRAME_INPUT_DIR", 15)) {
+            const char * const arg = SkipSpacesTabs(&input[15]);
+            if (STRNCASEEQ(arg, "stdin", 5))
+                SetStdinInput(frameInputSourceP);
+
+            strcpy(currentFramePath, arg);
+        } else if (STRNEQ(input, "FRAME_INPUT", 11)) {
+            ReadInputFileNames(fpointer, "FRAME_END_INPUT", 
+                               frameInputSourceP->stdinUsed ? 
+                               NULL : frameInputSourceP);
+            optionSeen[OPTION_FRAME_INPUT] = TRUE;
+        } else if (STRNEQ(input, "FORCE_I_ALIGN", 13)) {
+            forceIalign = TRUE;
+        } else if (STRNEQ(input, "FORCE_ENCODE_LAST_FRAME", 23)) {
+            /* NO-OP.  We used to drop trailing B frames by default and you
+               needed this option to change the last frame to I so you could
+               encode all the frames.  Now we just do that all the time.  
+               Why wouldn't we?
+            */
+        } else if (STRNEQ(input, "FRAME_RATE", 10)) {
+            frameRate = GetFrameRate(SkipSpacesTabs(&input[10]));
+            frameRateRounded = (int) VidRateNum[frameRate];
+            if ((frameRate % 3) == 1)
+                frameRateInteger = FALSE;
+            optionSeen[OPTION_FRAME_RATE] = TRUE;
+        }
+        break;
+        
+    case 'G':
+        if (STRNEQ(input, "GOP_SIZE", 8)) {
+            SetGOPSize(atoi(SkipSpacesTabs(&input[8])));
+            optionSeen[OPTION_GOP] = TRUE;
+        } else if (STRNEQ(input, "GOP_INPUT_DIR", 13)) {
+            const char * const arg = SkipSpacesTabs(&input[13]);
+            if (STRNCASEEQ(arg, "stdin", 5))
+                SetStdinInput(gopInputSourceP);
+
+            strcpy(currentGOPPath, arg);
+        } else if (STRNEQ(input, "GOP_INPUT", 9)) {
+            ReadInputFileNames(fpointer, "GOP_END_INPUT", 
+                               gopInputSourceP->stdinUsed ? 
+                               NULL : gopInputSourceP);
+            optionSeen[OPTION_GOP_INPUT] = TRUE;
+        } else if (STRNEQ(input, "GAMMA", 5)) {
+            GammaCorrection = TRUE;
+            sscanf(SkipSpacesTabs(&input[5]), "%f", &GammaValue);
+            optionSeen[OPTION_GAMMA] = TRUE;
+        }
+        break;
+        
+    case 'I':
+        if (STRNEQ(input, "IQSCALE", 7)) {
+            SetIQScale(atoi(SkipSpacesTabs(&input[7])));
+            optionSeen[OPTION_IQSCALE] = TRUE;
+        } else if (STRNEQ(input, "INPUT_DIR", 9)) {
+            const char * const arg = SkipSpacesTabs(&input[9]);
+            if (STRNCASEEQ(arg, "stdin", 5))
+                SetStdinInput(inputSourceP);
+            strcpy(currentPath, arg);
+            optionSeen[OPTION_INPUT_DIR] = TRUE;
+        } else if (STRNEQ(input, "INPUT_CONVERT", 13)) {
+            strcpy(inputConversion, SkipSpacesTabs(&input[13]));
+            optionSeen[OPTION_INPUT_CONVERT] = TRUE;
+        } else if (STREQ(input, "INPUT")) {
+            ReadInputFileNames(fpointer, "END_INPUT", 
+                               inputSourceP->stdinUsed ?
+                               NULL : inputSourceP);
+            optionSeen[OPTION_INPUT] = TRUE;
+        } else if (STRNEQ(input, "IO_SERVER_CONVERT", 17)) {
+            strcpy(ioConversion, SkipSpacesTabs(&input[17]));
+            optionSeen[OPTION_IO_CONVERT] = TRUE;
+        } else if (STRNEQ(input, "IQTABLE", 7)) {
+            processIqtable(fpointer);
+
+            optionSeen[OPTION_IQTABLE] = TRUE;
+        }
+        break;
+
+    case 'K':
+        if (STRNEQ(input, "KEEP_TEMP_FILES", 15))
+            keepTempFiles = TRUE;
+        break;
+        
+    case 'N':
+      if (STRNEQ(input, "NIQTABLE", 8)) {
+          readNiqTable(fpointer);
+
+          optionSeen[OPTION_NIQTABLE] = TRUE;
+      }
+      break;
+
+    case 'O':
+        if (STRNEQ(input, "OUTPUT", 6)) {
+            const char * const arg = SkipSpacesTabs(&input[6]);
+            if ( whichGOP == -1 )
+                strcpy(outputFileName, arg);
+            else
+                sprintf(outputFileName, "%s.gop.%d", arg, whichGOP);
+            
+            optionSeen[OPTION_OUTPUT] = TRUE;
+        }
+        break;
+        
+    case 'P':
+        if (STRNEQ(input, "PATTERN", 7)) {
+            SetFramePattern(SkipSpacesTabs(&input[7]));
+            optionSeen[OPTION_PATTERN] = TRUE;
+        } else if (STRNEQ(input, "PIXEL", 5)) {
+            SetPixelSearch(SkipSpacesTabs(&input[5]));
+            optionSeen[OPTION_PIXEL] = TRUE;
+        } else if (STRNEQ(input, "PQSCALE", 7)) {
+            SetPQScale(atoi(SkipSpacesTabs(&input[7])));
+            optionSeen[OPTION_PQSCALE] = TRUE;
+        } else if (STRNEQ(input, "PSEARCH_ALG", 11)) {
+            SetPSearchAlg(SkipSpacesTabs(&input[11]));
+            optionSeen[OPTION_PSEARCH_ALG] = TRUE;
+        } else if (STRNEQ(input, "PARALLEL_TEST_FRAMES", 20)) {
+            SetParallelPerfect(FALSE);
+            parallelTestFrames = atoi(SkipSpacesTabs(&input[20]));
+        } else if (STRNEQ(input, "PARALLEL_TIME_CHUNKS", 20)) {
+            SetParallelPerfect(FALSE);
+            parallelTimeChunks = atoi(SkipSpacesTabs(&input[20]));
+        } else if (STRNEQ(input, "PARALLEL_CHUNK_TAPER", 20)) {
+            SetParallelPerfect(FALSE);
+            parallelTimeChunks = -1;
+        } else if (STRNEQ(input, "PARALLEL_PERFECT", 16)) {
+            SetParallelPerfect(TRUE);
+        } else if (STRNEQ(input, "PARALLEL", 8)) {
+            ReadMachineNames(fpointer);
+            optionSeen[OPTION_PARALLEL] = TRUE;
+        }
+        break;
+        
+    case 'R':
+        if (STRNEQ(input, "RANGE", 5)) {
+            processRanges(SkipSpacesTabs(&input[5]));
+            optionSeen[OPTION_RANGE] = TRUE;
+        } else if (STRNEQ(input, "REFERENCE_FRAME", 15)) {
+            SetReferenceFrameType(SkipSpacesTabs(&input[15]));
+            optionSeen[OPTION_REF_FRAME] = TRUE;
+        } else if (STRNEQ(input, "RSH", 3)) {
+            SetRemoteShell(SkipSpacesTabs(&input[3]));
+        } else if (STRNEQ(input, "RESIZE", 6)) {
+            const char * const arg = SkipSpacesTabs(&input[6]);
+            sscanf(arg, "%dx%d", &outputWidth, &outputHeight);
+            outputWidth &= ~(DCTSIZE * 2 - 1);
+            outputHeight &= ~(DCTSIZE * 2 - 1);
+            optionSeen[OPTION_RESIZE] = TRUE;
+        }
+        break;
+
+    case 'S':
+        if (STRNEQ(input, "SLICES_PER_FRAME", 16)) {
+            SetSlicesPerFrame(atoi(SkipSpacesTabs(&input[16])));
+            optionSeen[OPTION_SPF] = TRUE;
+        } else if (STRNEQ(input, "SLAVE_CONVERT", 13)) {
+            strcpy(slaveConversion, SkipSpacesTabs(&input[13]));
+            optionSeen[OPTION_SLAVE_CONVERT] = TRUE;
+        } else if (STRNEQ(input, "SPECIFICS_FILE", 14)) {
+            strcpy(specificsFile, SkipSpacesTabs(&input[14]));
+            specificsOn = TRUE;
+            optionSeen[OPTION_SPECIFICS] = TRUE;
+        } else if (STRNEQ(input, "SPECIFICS_DEFINES", 16)) {
+            strcpy(specificsDefines, SkipSpacesTabs(&input[17]));
+            optionSeen[OPTION_DEFS_SPECIFICS] = TRUE;
+        } else if (STRNEQ(input, "SEQUENCE_SIZE", 13)) {
+            mult_seq_headers = atoi(SkipSpacesTabs(&input[13]));
+        } else if (STRNEQ(input, "SIZE", 4)) {
+            const char * const arg = SkipSpacesTabs(&input[4]);
+            sscanf(arg, "%dx%d", &yuvWidth, &yuvHeight);
+            realWidth = yuvWidth;
+            realHeight = yuvHeight;
+            Fsize_Validate(&yuvWidth, &yuvHeight);
+            optionSeen[OPTION_YUV_SIZE] = TRUE;
+        }
+        break;
+
+    case 'T':
+        if (STRNEQ(input, "TUNE", 4)) {
+            tuneingOn = TRUE;
+            ParseTuneParam(SkipSpacesTabs(&input[4]));
+        }
+        break;
+
+    case 'U':
+        if (STRNEQ(input, "USER_DATA", 9)) {
+            strcpy(userDataFileName, SkipSpacesTabs(&input[9]));
+            optionSeen[OPTION_USER_DATA] = TRUE;
+        }
+        break;
+        
+    case 'W':
+        if (STRNEQ(input, "WARN_UNDERFLOW", 14))
+            paramP->warnUnderflow = TRUE;
+        if (STRNEQ(input, "WARN_OVERFLOW", 13))
+            paramP->warnOverflow = TRUE;
+        break;
+        
+    case 'Y':
+        if (STRNEQ(input, "YUV_SIZE", 8)) {
+            const char * const arg = SkipSpacesTabs(&input[8]);
+            sscanf(arg, "%dx%d", &yuvWidth, &yuvHeight);
+            realWidth = yuvWidth;
+            realHeight = yuvHeight;
+            Fsize_Validate(&yuvWidth, &yuvHeight);
+            optionSeen[OPTION_YUV_SIZE] = TRUE;
+        } else if (STRNEQ(input, "Y_SIZE", 6)) {
+            const char * const arg = SkipSpacesTabs(&input[6]);
+            sscanf(arg, "%dx%d", &yuvWidth, &yuvHeight);
+            realWidth = yuvWidth;
+            realHeight = yuvHeight;
+            Fsize_Validate(&yuvWidth, &yuvHeight);
+            optionSeen[OPTION_YUV_SIZE] = TRUE;
+        } else if (STRNEQ(input, "YUV_FORMAT", 10)) {
+            strcpy(yuvConversion,  SkipSpacesTabs(&input[10]));
+            optionSeen[OPTION_YUV_FORMAT] = TRUE;
+        }
+        break;
+    }
+}
+
+
+
 /*===========================================================================*
  *
  * ReadParamFile
@@ -606,10 +947,8 @@ ReadParamFile(const char *    const fileName,
               struct params * const paramP) {
 
   FILE *fpointer;
-  char    input[256];
-  const char *charPtr;
-  boolean yuvUsed = FALSE;
-  int row, col;
+  char    buffer[256];
+  bool yuvUsed;
   struct inputSource * inputSourceP;
       /* Contents of INPUT section */
   struct inputSource * frameInputSourceP;
@@ -630,365 +969,31 @@ ReadParamFile(const char *    const fileName,
   sprintf(currentPath, ".");
   sprintf(currentGOPPath, ".");
   sprintf(currentFramePath, ".");
-#ifndef HPUX
   SetRemoteShell("rsh");
-#else
-  SetRemoteShell("remsh");
-#endif
   keepTempFiles = FALSE;
   paramP->warnOverflow = FALSE;
   paramP->warnUnderflow = FALSE;
 
   initOptionSeen();
 
-  while ( fgets(input, 256, fpointer) != NULL ) {
-    /* skip comments */
-    if ( input[0] == '#' ) {            
-      continue;
-    }
+  yuvUsed = FALSE;  /* initial value */
 
-    {
-      int len = strlen(input);
-      if (input[len-1] == '\n') {
-	len--;
-        input[len] = '\0'; /* get rid of newline */
-	/* Junk whitespace */
-	while ((len >= 0) && ((input[len-1] == ' ') || (input[len-1] == '\t'))) {
-	  input[--len] = '\0';
-	}
-      }
-    }
+  while (fgets(buffer, 256, fpointer) != NULL) {
+      const char * input;
 
-    if (strlen(SkipSpacesTabs(input)) == 0) continue;
+      removeTrailingWhite(buffer, &input);
 
-    switch(input[0]) {
-    case 'A':
-      if ( strncmp(input, "ASPECT_RATIO", 12) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[12]);
-        aspectRatio = GetAspectRatio(charPtr);
-        optionSeen[OPTION_ASPECT_RATIO] = TRUE;
-      }
-      break;
+      if (strlen(input) == 0) {
+          /* Ignore blank line */
+      } else if (input[0] == '#') {
+          /* Ignore comment */
+      } else
+          processParamLine(input, fpointer, &yuvUsed,
+                           inputSourceP, frameInputSourceP, gopInputSourceP,
+                           paramP);
+              /* May read additional lines from file */
 
-    case 'B':
-      if ( strncmp(input, "BQSCALE", 7) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[7]);
-        SetBQScale(atoi(charPtr));
-        optionSeen[OPTION_BQSCALE] = TRUE;
-      } else if ( strncmp(input, "BASE_FILE_FORMAT", 16) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[16]);
-        SetFileFormat(charPtr);
-        if ( (strncmp(charPtr,"YUV",3) == 0) || 
-            (strcmp(charPtr,"Y") == 0) ) {
-          yuvUsed = TRUE;
-        }
-        optionSeen[OPTION_BASE_FORMAT] = TRUE;
-      } else if ( strncmp(input, "BSEARCH_ALG", 11) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[11]);
-        SetBSearchAlg(charPtr);
-        optionSeen[OPTION_BSEARCH_ALG] = TRUE;
-      } else if ( strncmp(input, "BIT_RATE", 8) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[8]);
-        setBitRate(charPtr);
-        optionSeen[OPTION_BIT_RATE] = TRUE;
-      } else if ( strncmp(input, "BUFFER_SIZE", 11) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[11]);
-        setBufferSize(charPtr);
-        optionSeen[OPTION_BUFFER_SIZE] = TRUE;                  
-      }
-      break;
-
-    case 'C':
-      if ( strncmp(input, "CDL_FILE", 8) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[8]);
-        strcpy(specificsFile, charPtr);
-        specificsOn=TRUE;
-        optionSeen[OPTION_SPECIFICS] = TRUE;
-      } else if ( strncmp(input, "CDL_DEFINES", 11) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[11]);
-        strcpy(specificsDefines, charPtr);
-        optionSeen[OPTION_DEFS_SPECIFICS] = TRUE;
-      }
-      break;
-
-    case 'F':
-      if (strncmp(input, "FRAME_INPUT_DIR", 15) == 0) {
-          charPtr = SkipSpacesTabs(&input[15]);
-          if ( strncmp(charPtr, "stdin", 5) == 0 ||
-               strncmp(charPtr, "STDIN", 5) == 0 ) {
-              SetStdinInput(frameInputSourceP);
-          }
-          strcpy(currentFramePath, charPtr);
-      } else if ( strncmp(input, "FRAME_INPUT", 11) == 0 ) {
-          ReadInputFileNames(fpointer, "FRAME_END_INPUT", 
-                             frameInputSourceP->stdinUsed ? 
-                             NULL : frameInputSourceP);
-          optionSeen[OPTION_FRAME_INPUT] = TRUE;
-      } else if ( strncmp(input, "FORCE_I_ALIGN", 13) == 0 ) {
-        forceIalign = TRUE;
-      } else if ( strncmp(input, "FORCE_ENCODE_LAST_FRAME", 23) == 0 ) {
-        /* NO-OP.  We used to drop trailing B frames by default and you
-           needed this option to change the last frame to I so you could
-           encode all the frames.  Now we just do that all the time.  
-           Why wouldn't we?
-        */
-      } else if ( strncmp(input, "FRAME_RATE", 10) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[10]);
-        frameRate = GetFrameRate(charPtr);
-        frameRateRounded = (int) VidRateNum[frameRate];
-        if ( (frameRate % 3) == 1) {
-          frameRateInteger = FALSE;
-        }
-        optionSeen[OPTION_FRAME_RATE] = TRUE;
-      }
-      break;
-
-    case 'G':
-        if ( strncmp(input, "GOP_SIZE", 8) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[8]);
-            SetGOPSize(atoi(charPtr));
-            optionSeen[OPTION_GOP] = TRUE;
-        } else if ( strncmp(input, "GOP_INPUT_DIR", 13) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[13]);
-            if ( strncmp(charPtr, "stdin", 5) == 0 ||
-                 strncmp(charPtr, "STDIN", 5) == 0 ) {
-                SetStdinInput(gopInputSourceP);
-            }
-            strcpy(currentGOPPath, charPtr);
-        } else if ( strncmp(input, "GOP_INPUT", 9) == 0 ) {
-            ReadInputFileNames(fpointer, "GOP_END_INPUT", 
-                               gopInputSourceP->stdinUsed ? 
-                               NULL : gopInputSourceP);
-            optionSeen[OPTION_GOP_INPUT] = TRUE;
-        } else if ( strncmp(input, "GAMMA", 5) == 0) {
-            charPtr = SkipSpacesTabs(&input[5]);
-            GammaCorrection = TRUE;
-            sscanf(charPtr,"%f",&GammaValue);
-            optionSeen[OPTION_GAMMA] = TRUE;
-        }
-        break;
-
-    case 'I':
-        if ( strncmp(input, "IQSCALE", 7) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[7]);
-            SetIQScale(atoi(charPtr));
-            optionSeen[OPTION_IQSCALE] = TRUE;
-        } else if ( strncmp(input, "INPUT_DIR", 9) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[9]);
-            if ( strncmp(charPtr, "stdin", 5) == 0 ||
-                 strncmp(charPtr, "STDIN", 5) == 0 )
-                SetStdinInput(inputSourceP);
-            strcpy(currentPath, charPtr);
-            optionSeen[OPTION_INPUT_DIR] = TRUE;
-        } else if ( strncmp(input, "INPUT_CONVERT", 13) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[13]);
-            strcpy(inputConversion, charPtr);
-            optionSeen[OPTION_INPUT_CONVERT] = TRUE;
-        } else if ( strcmp(input, "INPUT") == 0 ) { /* yes, strcmp */
-            ReadInputFileNames(fpointer, "END_INPUT", 
-                               inputSourceP->stdinUsed ?
-                               NULL : inputSourceP);
-            optionSeen[OPTION_INPUT] = TRUE;
-        } else if ( strncmp(input, "IO_SERVER_CONVERT", 17) == 0 ) {
-            charPtr = SkipSpacesTabs(&input[17]);
-            strcpy(ioConversion, charPtr);
-            optionSeen[OPTION_IO_CONVERT] = TRUE;
-        } else if ( strncmp(input, "IQTABLE", 7) == 0 ) {
-            processIqtable(fpointer);
-
-            optionSeen[OPTION_IQTABLE] = TRUE;
-        }
-        break;
-
-    case 'K':
-        if (strncmp(input, "KEEP_TEMP_FILES", 15) == 0)
-            keepTempFiles = TRUE;
-        break;
-
-    case 'N':
-      if ( strncmp(input, "NIQTABLE", 8) == 0 ) {
-        for ( row = 0; row < 8; row ++ ) {
-          fgets(input, 256, fpointer);
-          charPtr = input;
-          if (8!=sscanf(charPtr,"%d %d %d %d %d %d %d %d",
-                        &niqtable[row*8+0],  &niqtable[row*8+1],
-                        &niqtable[row*8+2],  &niqtable[row*8+3],
-                        &niqtable[row*8+4],  &niqtable[row*8+5],
-                        &niqtable[row*8+6],  &niqtable[row*8+7])) {
-            fprintf(stderr, "Line %d of NIQTABLE doesn't have 8 elements!\n", 
-                    row);
-            exit(1);
-          }
-          for ( col = 0; col < 8; col++ ) {
-            if ((niqtable[row*8+col]<1) || (niqtable[row*8+col]>255)) {
-              fprintf(stderr,
-                      "Warning:  NIQTable Element %1d,%1d (%d) "
-                      "corrected to 1-255.\n",
-                      row+1, col+1, niqtable[row*8+col]);
-              niqtable[row*8+col]=(niqtable[row*8+col]<1)?1:255;
-            }}
-        }
-
-        customNIQtable = niqtable;
-        optionSeen[OPTION_NIQTABLE] = TRUE;
-      }
-      break;
-
-    case 'O':
-      if ( strncmp(input, "OUTPUT", 6) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[6]);
-        if ( whichGOP == -1 ) {
-          strcpy(outputFileName, charPtr);
-        } else {
-          sprintf(outputFileName, "%s.gop.%d",
-                  charPtr, whichGOP);
-        }
-
-        optionSeen[OPTION_OUTPUT] = TRUE;
-      }
-      break;
-
-    case 'P':
-      if ( strncmp(input, "PATTERN", 7) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[7]);
-        SetFramePattern(charPtr);
-        optionSeen[OPTION_PATTERN] = TRUE;
-      } else if ( strncmp(input, "PIXEL", 5) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[5]);
-        SetPixelSearch(charPtr);
-        optionSeen[OPTION_PIXEL] = TRUE;
-      } else if ( strncmp(input, "PQSCALE", 7) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[7]);
-        SetPQScale(atoi(charPtr));
-        optionSeen[OPTION_PQSCALE] = TRUE;
-      } else if ( strncmp(input, "PSEARCH_ALG", 11) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[11]);
-        SetPSearchAlg(charPtr);
-        optionSeen[OPTION_PSEARCH_ALG] = TRUE;
-      } else if ( strncmp(input, "PARALLEL_TEST_FRAMES", 20) == 0 ) {
-        SetParallelPerfect(FALSE);
-        charPtr = SkipSpacesTabs(&input[20]);
-        parallelTestFrames = atoi(charPtr);
-      } else if ( strncmp(input, "PARALLEL_TIME_CHUNKS", 20) == 0 ) {
-        SetParallelPerfect(FALSE);
-        charPtr = SkipSpacesTabs(&input[20]);
-        parallelTimeChunks = atoi(charPtr);
-      } else if ( strncmp(input, "PARALLEL_CHUNK_TAPER", 20) == 0 ) {
-        SetParallelPerfect(FALSE);
-        parallelTimeChunks = -1;
-      } else if ( strncmp(input, "PARALLEL_PERFECT", 16) == 0 ) {
-        SetParallelPerfect(TRUE);
-      } else if ( strncmp(input, "PARALLEL", 8) == 0 ) {
-        ReadMachineNames(fpointer);
-        optionSeen[OPTION_PARALLEL] = TRUE;
-      }
-      break;
-
-    case 'R':
-      if ( strncmp(input, "RANGE", 5) == 0 ) {
-        int num_ranges=0,a,b;
-        charPtr = SkipSpacesTabs(&input[5]);
-        optionSeen[OPTION_RANGE] = TRUE;
-        num_ranges=sscanf(charPtr,"%d %d",&a,&b);
-        if (num_ranges==2) {
-          SetSearchRange(a,b);
-        } else if (sscanf(charPtr,"%d [%d]",&a,&b)==2) {
-          SetSearchRange(a,b);
-        } else SetSearchRange(a,a);
-      } else if ( strncmp(input, "REFERENCE_FRAME", 15) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[15]);
-        SetReferenceFrameType(charPtr);
-        optionSeen[OPTION_REF_FRAME] = TRUE;
-      } else if ( strncmp(input, "RSH", 3) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[3]);
-        SetRemoteShell(charPtr);
-      } else if ( strncmp(input, "RESIZE", 6) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[6]);                    
-        sscanf(charPtr, "%dx%d", &outputWidth, &outputHeight);
-        outputWidth &= ~(DCTSIZE * 2 - 1);
-        outputHeight &= ~(DCTSIZE * 2 - 1);
-        optionSeen[OPTION_RESIZE] = TRUE;
-      }
-      break;
-
-    case 'S':
-      if ( strncmp(input, "SLICES_PER_FRAME", 16) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[16]);
-        SetSlicesPerFrame(atoi(charPtr));
-        optionSeen[OPTION_SPF] = TRUE;
-      } else if ( strncmp(input, "SLAVE_CONVERT", 13) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[13]);
-        strcpy(slaveConversion, charPtr);
-        optionSeen[OPTION_SLAVE_CONVERT] = TRUE;
-      } else if ( strncmp(input, "SPECIFICS_FILE", 14) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[14]);
-        strcpy(specificsFile, charPtr);
-        specificsOn=TRUE;
-        optionSeen[OPTION_SPECIFICS] = TRUE;
-      } else if ( strncmp(input, "SPECIFICS_DEFINES", 16) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[17]);
-        strcpy(specificsDefines, charPtr);
-        optionSeen[OPTION_DEFS_SPECIFICS] = TRUE;
-      } else if (strncmp(input, "SEQUENCE_SIZE", 13) == 0) {
-        charPtr = SkipSpacesTabs(&input[13]);
-        mult_seq_headers = atoi(charPtr);
-      } else if (strncmp(input, "SIZE", 4) == 0 ) {
-        charPtr = SkipSpacesTabs(&input[4]);
-        sscanf(charPtr, "%dx%d", &yuvWidth, &yuvHeight);
-        realWidth = yuvWidth;
-        realHeight = yuvHeight;
-        Fsize_Validate(&yuvWidth, &yuvHeight);
-        optionSeen[OPTION_YUV_SIZE] = TRUE;
-      }
-      break;
-
-    case 'T':
-      if ( strncmp(input, "TUNE", 4) == 0) {
-        tuneingOn = TRUE;
-        charPtr = SkipSpacesTabs(&input[4]);
-	ParseTuneParam(charPtr);
-      }
-      break;
-
-    case 'U':
-        if (strncmp(input, "USER_DATA", 9) == 0) {
-            charPtr = SkipSpacesTabs(&input[9]);
-            strcpy(userDataFileName, charPtr);
-            optionSeen[OPTION_USER_DATA] = TRUE;
-        }
-        break;
-        
-    case 'W':
-        if (strncmp(input, "WARN_UNDERFLOW", 14) == 0)
-            paramP->warnUnderflow = TRUE;
-        if (strncmp(input, "WARN_OVERFLOW", 13) == 0)
-            paramP->warnOverflow = TRUE;
-        break;
-        
-    case 'Y':
-        if (strncmp(input, "YUV_SIZE", 8) == 0) {
-            charPtr = SkipSpacesTabs(&input[8]);
-            sscanf(charPtr, "%dx%d", &yuvWidth, &yuvHeight);
-            realWidth = yuvWidth;
-            realHeight = yuvHeight;
-            Fsize_Validate(&yuvWidth, &yuvHeight);
-            optionSeen[OPTION_YUV_SIZE] = TRUE;
-        }
-        else if (strncmp(input, "Y_SIZE", 6) == 0) {
-            charPtr = SkipSpacesTabs(&input[6]);
-            sscanf(charPtr, "%dx%d", &yuvWidth, &yuvHeight);
-            realWidth = yuvWidth;
-            realHeight = yuvHeight;
-            Fsize_Validate(&yuvWidth, &yuvHeight);
-            optionSeen[OPTION_YUV_SIZE] = TRUE;
-        }
-        else if (strncmp(input, "YUV_FORMAT", 10) == 0) {
-            charPtr = SkipSpacesTabs(&input[10]);
-            strcpy(yuvConversion, charPtr);
-            optionSeen[OPTION_YUV_FORMAT] = TRUE;
-        }
-        break;
-    }
+      strfree(input);
   }
 
   fclose(fpointer);
