@@ -19,6 +19,7 @@
 #include "ppm.h"
 #include "tga.h"
 #include "shhopt.h"
+#include "nstring.h"
 
 #define MAXCOLORS 16384
 
@@ -72,15 +73,15 @@ parseCommandLine(int argc, char ** argv,
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (argc - 1 == 0)
-        cmdlineP->input_filename = "=";  /* he wants stdin */
+        cmdlineP->input_filename = "-";  /* he wants stdin */
     else if (argc - 1 == 1)
         cmdlineP->input_filename = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted "
-                 "is the input file specificaton");
+                 "is the input file specification");
 
     if (alpha_spec &&
-        strcmp(cmdlineP->alpha_filename, "-") == 0 )
+        STREQ(cmdlineP->alpha_filename, "-"))
         cmdlineP->alpha_stdout = 1;
     else 
         cmdlineP->alpha_stdout = 0;
