@@ -9,7 +9,7 @@
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
 **
-** Upgraded to support XPM version 3 by
+** Upgraded to handle XPM version 3 by
 **   Arnaud Le Hors (lehors@mirsa.inria.fr)
 **   Tue Apr 9 1991
 **
@@ -38,7 +38,7 @@
 #include "nstring.h"
 #include "mallocvar.h"
 
-#define MAX_LINE 2048
+#define MAX_LINE (8 * 1024)
   /* The maximum size XPM input line we can handle. */
 
 /* number of xpmColorKeys */
@@ -101,10 +101,10 @@ parse_command_line(int argc, char ** argv,
         cmdline_p->input_filespec = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specificaton");
+                 "is the input file specification");
 
     if (cmdline_p->alpha_filename && 
-        strcmp(cmdline_p->alpha_filename, "-") == 0 )
+        STREQ(cmdline_p->alpha_filename, "-"))
         cmdline_p->alpha_stdout = TRUE;
     else 
         cmdline_p->alpha_stdout = FALSE;
@@ -121,7 +121,9 @@ static bool backup;
 
 
 static void
-getline(char * const line, int const size, FILE * const stream) {
+getline(char * const line,
+        size_t const size,
+        FILE * const stream) {
 /*----------------------------------------------------------------------------
    Read the next line from the input file 'stream', through the one-line
    buffer lastInputLine[].
@@ -136,7 +138,7 @@ getline(char * const line, int const size, FILE * const stream) {
 
    Exit program if the line doesn't fit in the buffer.
 -----------------------------------------------------------------------------*/
-    if (size > MAX_LINE+1)
+    if (size > sizeof(lastInputLine))
         pm_error("INTERNAL ERROR: getline() received 'size' parameter "
                  "which is out of bounds");
 
@@ -148,8 +150,8 @@ getline(char * const line, int const size, FILE * const stream) {
             pm_error("EOF or read error on input file");
         if (strlen(line) == size - 1)
             pm_error("Input file has line that is too long (longer than "
-                     "%d bytes).", size - 1);
-        strncpy(lastInputLine, line, MAX_LINE);
+                     "%u bytes).", (unsigned)size - 1);
+        STRSCPY(lastInputLine, line);
     }
 }
 
@@ -159,7 +161,7 @@ static unsigned int
 getNumber(char * const p, unsigned int const size) {
 
     unsigned int retval;
-    char * q;
+    unsigned char * q;
     
     retval = 0;
     for (q = p; q < p+size; ++q)
@@ -291,7 +293,7 @@ interpretXpm3ColorTableLine(char line[], int const seqNum,
                number if so)
             */
             for (key = 1; 
-                 key <= NKEYS && strcmp(xpmColorKeys[key - 1], str2) != 0; 
+                 key <= NKEYS && !STREQ(xpmColorKeys[key - 1], str2); 
                  key++);
             isKey = (key <= NKEYS);
 
@@ -463,7 +465,8 @@ readXpm1Header(FILE * const stream, int * const widthP, int * const heightP,
     char line[MAX_LINE+1], str1[MAX_LINE+1], str2[MAX_LINE+1];
     char *t1;
     char *t2;
-    int format, v;
+    int format;
+    unsigned int v;
     int i, j;
     bool processedStaticChar;  
         /* We have read up to and interpreted the "static char..." line */
@@ -481,15 +484,15 @@ readXpm1Header(FILE * const stream, int * const widthP, int * const heightP,
                 t1 = str1;
             else
                 ++t1;
-            if (!strcmp(t1, "format"))
+            if (STREQ(t1, "format"))
                 format = v;
-            else if (!strcmp(t1, "width"))
+            else if (STREQ(t1, "width"))
                 *widthP = v;
-            else if (!strcmp(t1, "height"))
+            else if (STREQ(t1, "height"))
                 *heightP = v;
-            else if (!strcmp(t1, "ncolors"))
+            else if (STREQ(t1, "ncolors"))
                 *ncolorsP = v;
-            else if (!strcmp(t1, "pixel"))
+            else if (STREQ(t1, "pixel"))
                 *chars_per_pixelP = v;
         } else if (!strncmp(line, "static char", 11)) {
             if ((t1 = strrchr(line, '_')) == NULL)
@@ -611,7 +614,7 @@ interpretXpmLine(char   const line[],
    Stop processing the line either at the end of the line or when
    the output would go beyond maxCursor, whichever comes first.
 -----------------------------------------------------------------------------*/
-    char *lineCursor;
+    char * lineCursor;
 
     lineCursor = strchr(line, '"');  /* position to 1st quote in line */
     if (lineCursor == NULL) {
@@ -624,7 +627,9 @@ interpretXpmLine(char   const line[],
     } else {
         ++lineCursor; /* Skip to first character after quote */
 
-        /* Handle pixels until a close quote, eol, or the end of the image. */
+        /* Handle pixels until a close quote, eol, or we've returned all
+           the pixels Caller wants.
+        */
         while (*lineCursor && *lineCursor != '"' && *cursorP <= maxCursor) {
             int colorNumber;
             int i;
