@@ -242,10 +242,11 @@ determineOutputType(struct pam * const composedPamP,
                     struct pam * const overlayPamP) {
 
     composedPamP->height = underlayPamP->height;
-    composedPamP->width = underlayPamP->width;
+    composedPamP->width  = underlayPamP->width;
 
     composedPamP->format = commonFormat(underlayPamP->format, 
                                         overlayPamP->format);
+    composedPamP->plainformat = FALSE;
     commonTupletype(underlayPamP->tuple_type, overlayPamP->tuple_type,
                     composedPamP->tuple_type, 
                     sizeof(composedPamP->tuple_type));
@@ -304,6 +305,25 @@ warnOutOfFrame( int const originLeft,
 
 
 
+static void
+validateComputableHeight(int const originTop, 
+                         int const overRows) {
+
+    if (originTop < 0) {
+        if (originTop < -INT_MAX)
+            pm_error("Overlay starts too far above the underlay image to be "
+                     "computable.  Overlay can be at most %d rows above "
+                     "the underlay.", INT_MAX);
+    } else {
+        if (INT_MAX - originTop <= overRows)
+            pm_error("Too many total rows involved to be computable.  "
+                     "You must have a shorter overlay image or compose it "
+                     "higher on the underlay image.");
+    }
+}
+
+
+
 static void
 computeOverlayPosition(int                const underCols, 
                        int                const underRows,
@@ -339,6 +359,8 @@ computeOverlayPosition(int                const underCols,
     *originLeftP = xalign + cmdline.xoff;
     *originTopP  = yalign + cmdline.yoff;
 
+    validateComputableHeight(*originTopP, overRows);
+
     warnOutOfFrame(*originLeftP, *originTopP, 
                    overCols, overRows, underCols, underRows);    
 }
