@@ -388,6 +388,16 @@ struct rowCutter {
 
 
 
+/* In a typical multi-image stream, all the images have the same
+   dimensions, so this program creates and destroys identical row
+   cutters for each image in the stream.  If that turns out to take a
+   significant amount of resource to do, we should create a cache:
+   keep the last row cutter made, tagged by the parameters used to
+   create it.  If the parameters are the same for the next image, we
+   just use that cached row cutter; otherwise, we discard it and
+   create a new one then.
+*/
+
 static void
 createRowCutter(struct pam *        const inpamP,
                 struct pam *        const outpamP,
@@ -416,12 +426,12 @@ createRowCutter(struct pam *        const inpamP,
     MALLOCARRAY_NOFAIL(outputPointers, outpamP->width);
 
     /* Put in left padding */
-    for (col = leftcol; col < 0; ++col)
+    for (col = leftcol; col < 0 && col-leftcol < outpamP->width; ++col)
         outputPointers[col-leftcol] = blackTuple;
- 
+
     /* Put in extracted columns */
-    for (col = MAX(leftcol, 0); 
-         col <= MIN(rightcol, inpamP->width-1); 
+    for (col = MAX(leftcol, 0);
+         col <= MIN(rightcol, inpamP->width-1);
          ++col) {
         int const outcol = col - leftcol;
 
@@ -429,15 +439,18 @@ createRowCutter(struct pam *        const inpamP,
     }
 
     /* Put in right padding */
-    for (col = MIN(rightcol, inpamP->width-1) + 1; col <= rightcol; ++col)
-        outputPointers[col-leftcol] = blackTuple;
-    
+    for (col = MIN(rightcol, inpamP->width-1) + 1; col <= rightcol; ++col) {
+        if (col - leftcol >= 0) {
+            outputPointers[col-leftcol] = blackTuple;
+        }
+    }
+
     /* Direct input pixels that are getting cut off to the discard tuple */
 
-    for (col = 0; col < leftcol; ++col)
+    for (col = 0; col < MIN(leftcol, inpamP->width); ++col)
         inputPointers[col] = discardTuple;
 
-    for (col = rightcol + 1; col < inpamP->width; ++col)
+    for (col = MAX(0, rightcol + 1); col < inpamP->width; ++col)
         inputPointers[col] = discardTuple;
 
     MALLOCVAR_NOFAIL(rowCutterP);
@@ -469,23 +482,17 @@ destroyRowCutter(struct rowCutter * const rowCutterP) {
 
 
 
-int
-main(int argc, char *argv[]) {
+static void
+cutOneImage(FILE *             const ifP,
+            struct cmdlineInfo const cmdline,
+            FILE *             const ofP) {
 
-    FILE* ifP;
     int row;
     int leftcol, rightcol, toprow, bottomrow;
-    struct cmdlineInfo cmdline;
     struct pam inpam;   /* Input PAM image */
     struct pam outpam;  /* Output PAM image */
     struct rowCutter * rowCutterP;
 
-    pnm_init(&argc, argv);
-
-    parseCommandLine(argc, argv, &cmdline);
-
-    ifP = pm_openr(cmdline.inputFilespec);
-    
     pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
     
     computeCutBounds(inpam.width, inpam.height, 
@@ -497,7 +504,17 @@ main(int argc, char *argv[]) {
     if (!cmdline.pad)
         rejectOutOfBounds(inpam.width, inpam.height, leftcol, rightcol, 
                           toprow, bottomrow);
-
+    else {
+        if (cmdline.left > cmdline.right) 
+            pm_error("You have specified a left edge (%d) that is to the right\n"
+                     "of the right edge you specified (%d)", 
+                     cmdline.left, cmdline.right);
+        
+        if (cmdline.top > cmdline.bottom) 
+            pm_error("You have specified a top edge (%d) that is below\n"
+                     "the bottom edge you specified (%d)", 
+                     cmdline.top, cmdline.bottom);
+    }
     if (cmdline.verbose) {
         pm_message("Image goes from Row 0, Column 0 through Row %d, Column %d",
                    inpam.height-1, inpam.width-1);
@@ -506,7 +523,7 @@ main(int argc, char *argv[]) {
     }
 
     outpam = inpam;    /* Initial value -- most fields should be same */
-    outpam.file   = stdout;
+    outpam.file   = ofP;
     outpam.width  = rightcol-leftcol+1;
     outpam.height = bottomrow-toprow+1;
 
@@ -537,9 +554,33 @@ main(int argc, char *argv[]) {
     /* Write out bottom padding */
     if ((bottomrow - (inpam.height-1)) > 0)
         writeBlackRows(&outpam, bottomrow - (inpam.height-1));
+}
 
-    pm_close(inpam.file);
-    pm_close(outpam.file);
+
+
+int
+main(int argc, char *argv[]) {
+
+    FILE * const ofP = stdout;
+
+    struct cmdlineInfo cmdline;
+    FILE* ifP;
+    bool eof;
+
+    pnm_init(&argc, argv);
+
+    parseCommandLine(argc, argv, &cmdline);
+
+    ifP = pm_openr(cmdline.inputFilespec);
+
+    eof = FALSE;
+    while (!eof) {
+        cutOneImage(ifP, cmdline, ofP);
+        pnm_nextimage(ifP, &eof);
+    }
+
+    pm_close(ifP);
+    pm_close(ofP);
     
     return 0;
 }
