@@ -145,6 +145,7 @@ parseCommandLine(int argc, char ** argv,
 }
 
 
+
 static struct pam
 makeOutputPam(unsigned int const width,
               unsigned int const height) {
@@ -331,7 +332,7 @@ doHilbert(FILE *       const ifP,
     int *x,*y;
     int sum;
 
-    grays = pnm_readpam(ifP, &graypam, sizeof(graypam));
+    grays = pnm_readpam(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
 
     bitpam = makeOutputPam(graypam.width, graypam.height);
 
@@ -424,7 +425,8 @@ fsConvertRow(struct converter * const converterP,
     do {
         samplen sum;
 
-        sum = pm_ungamma709(grayrow[col][0]) + thiserr[col + 1];
+        sum = MIN(2*stateP->threshval, pm_ungamma709(grayrow[col][0])) +
+            thiserr[col + 1];
         if (sum >= stateP->threshval) {
             /* We've accumulated enough light to justify a white output
                pixel.
@@ -570,13 +572,13 @@ clusterConvertRow(struct converter * const converterP,
                   tuple                    bitrow[]) {
 
     struct clusterState * const stateP = converterP->stateP;
-    unsigned int const radius = stateP->radius;
+    unsigned int const diameter = 2 * stateP->radius;
 
     unsigned int col;
 
     for (col = 0; col < converterP->cols; ++col) {
         float const threshold = 
-            stateP->clusterMatrix[row % radius][col % radius];
+            stateP->clusterMatrix[row % diameter][col % diameter];
         bitrow[col][0] = 
             grayrow[col][0] > threshold ? PAM_BW_WHITE : PAM_BLACK;
     }
@@ -588,10 +590,11 @@ static void
 clusterDestroy(struct converter * const converterP) {
 
     struct clusterState * const stateP = converterP->stateP;
+    unsigned int const diameter = 2 * stateP->radius;
 
     unsigned int row;
 
-    for (row = 0; row < stateP->radius; ++row)
+    for (row = 0; row < diameter; ++row)
         free(stateP->clusterMatrix[row]);
 
     free(stateP->clusterMatrix);
@@ -614,6 +617,7 @@ createClusterConverter(struct pam *    const graypamP,
     */
     
     int const clusterNormalizer = radius * radius * 2;
+    unsigned int const diameter = 2 * radius;
 
     struct converter converter;
     struct clusterState * stateP;
@@ -627,13 +631,13 @@ createClusterConverter(struct pam *    const graypamP,
 
     stateP->radius = radius;
 
-    MALLOCARRAY_NOFAIL(stateP->clusterMatrix, radius);
-    for (row = 0; row < radius; ++row) {
+    MALLOCARRAY_NOFAIL(stateP->clusterMatrix, diameter);
+    for (row = 0; row < diameter; ++row) {
         unsigned int col;
 
-        MALLOCARRAY_NOFAIL(stateP->clusterMatrix[row], radius);
+        MALLOCARRAY_NOFAIL(stateP->clusterMatrix[row], diameter);
         
-        for (col = 0; col < radius; ++col) {
+        for (col = 0; col < diameter; ++col) {
             switch (ditherType) {
             case DT_REGULAR: 
                 switch (radius) {
@@ -691,7 +695,7 @@ main(int argc, char *argv[]) {
         tuple * bitrow;
         int row;
 
-        pnm_readpaminit(ifP, &graypam, sizeof(graypam));
+        pnm_readpaminit(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
 
         bitpam = makeOutputPam(graypam.width, graypam.height);
         
