@@ -388,16 +388,19 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
      * in a form easy for the program to use.
      */
-    const char *inputFileName;  /* Filespec of input file */
+    const char * inputFileName;  /* Filespec of input file */
     unsigned int nomix;
     basicFunction_t filterFunction; /* NULL if not using resample method */
     basicFunction_t windowFunction;
+        /* Meaningful only when filterFunction != NULL */
     double filterRadius;           
+        /* Meaningful only when filterFunction != NULL */
     enum scaleType scaleType;
     /* 'xsize' and 'ysize' are numbers of pixels.  Their meaning depends upon
        'scaleType'.  for SCALE_BOXFIT and SCALE_BOXFILL, they are the box 
        dimensions.  For SCALE_SEPARATE, they are the separate dimensions, or
-       zero to indicate unspecified.
+       zero to indicate unspecified.  For SCALE_PIXELMAX, they are
+       meaningless.
     */
     unsigned int xsize;
     unsigned int ysize;
@@ -680,6 +683,7 @@ parseCommandLine(int argc,
     } else if (reduce != -1) {
         cmdlineP->scaleType = SCALE_SEPARATE;
         parseFilespecOnlyParms(argc, argv, cmdlineP);
+        cmdlineP->xsize = cmdlineP->ysize = 0;
         cmdlineP->xscale = cmdlineP->yscale = 
             ((double) 1.0) / ((double) reduce);
         pm_message("reducing by %d gives scale factor of %f.", 
@@ -2092,7 +2096,7 @@ main(int argc, char **argv ) {
     outpam = inpam;  /* initial value */
     outpam.file = stdout;
 
-    if (PNM_FORMAT_TYPE(inpam.format) == PBM_TYPE) {
+    if (PNM_FORMAT_TYPE(inpam.format) == PBM_TYPE && !cmdline.nomix) {
         outpam.format = PGM_TYPE;
         outpam.maxval = PGM_MAXMAXVAL;
         pm_message("promoting from PBM to PGM");
