@@ -38,7 +38,7 @@ struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *inputFilespec;  /* Filespecs of input files */
+    const char * inputFileName;
     unsigned int width;
     unsigned int height;
     unsigned int cutoff;
@@ -65,7 +65,7 @@ parseCommandLine(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def;
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -113,11 +113,12 @@ parseCommandLine(int argc, char ** argv,
         cmdlineP->type = MEDIAN_UNSPECIFIED;
 
     if (argc-1 < 1)
-        cmdlineP->inputFilespec = "-";
+        cmdlineP->inputFileName = "-";
     else {
-        cmdlineP->inputFilespec = argv[1];
+        cmdlineP->inputFileName = argv[1];
         if (argc-1 > 1)
-            pm_error ("Too many arguments.");
+            pm_error ("Too many arguments.  The only argument is "
+                      "the optional input file name");
     }
 }
 
@@ -397,7 +398,7 @@ main(int    argc,
 
     parseCommandLine(argc, argv, &cmdline);
     
-    ifP = pm_openr(cmdline.inputFilespec);
+    ifP = pm_openr(cmdline.inputFileName);
 
     ccolso2 = cmdline.width / 2;
     crowso2 = cmdline.height / 2;
@@ -410,7 +411,7 @@ main(int    argc,
     grayrow = pgm_allocrow(cols);
 
     /* Allocate pointers to mask row buffer. */
-    rowptr = pgm_allocarray(1, cmdline.height);
+    rowptr = (gray **) pm_allocrow(cmdline.height, sizeof(gray *));
 
     /* Read in and write out initial rows that won't get changed. */
     for (row = 0; row < cmdline.height - 1; ++row) {
@@ -449,7 +450,7 @@ main(int    argc,
 
     pgm_freearray(grays, cmdline.height);
     pgm_freerow(grayrow);
-    pgm_freearray(rowptr, cmdline.height);
+    pm_freerow(rowptr);
 
     return 0;
 }
