@@ -50,6 +50,11 @@ struct cmdlineInfo {
     float wpercent;
     enum brightMethod brightMethod;
     unsigned int keephues;
+    float maxExpansion;
+        /* The maximum allowed expansion factor for expansion specified
+           by per centile.  This is a factor, not a per cent increase.
+           E.g. 50% increase means a factor of 1.50.
+        */
 };
 
 
@@ -73,7 +78,9 @@ parseCommandLine (int argc, char ** argv,
     optStruct3 opt;
 
     unsigned int luminosity, colorvalue, saturation;
-
+    unsigned int maxexpandSpec;
+    float maxexpand;
+    
     unsigned int option_def_index;
 
     MALLOCARRAY_NOFAIL(option_def, 100);
@@ -87,6 +94,8 @@ parseCommandLine (int argc, char ** argv,
             &cmdlineP->bvalue,     &cmdlineP->bvalueSpec, 0);
     OPTENT3(0,   "wvalue",        OPT_UINT,   
             &cmdlineP->wvalue,     &cmdlineP->wvalueSpec, 0);
+    OPTENT3(0,   "maxexpand",     OPT_FLOAT,   
+            &maxexpand,            &maxexpandSpec, 0);
     OPTENT3(0,   "keephues",      OPT_FLAG,   
             NULL,                  &cmdlineP->keephues, 0);
     OPTENT3(0,   "luminosity",    OPT_FLAG,   
@@ -139,6 +148,14 @@ parseCommandLine (int argc, char ** argv,
         else
             cmdlineP->brightMethod = BRIGHT_LUMINOSITY;
     }
+    if (maxexpandSpec) {
+        if (maxexpand < 0)
+            pm_error("-maxexpand must be positive.  You specified %f",
+                     maxexpand);
+        else
+            cmdlineP->maxExpansion = 1 + (float)maxexpand/100;
+    } else
+        cmdlineP->maxExpansion = 1e6;  /* essentially infinite */
 
     if (argc-1 > 1)
         pm_error("Program takes at most one argument: the input file "
@@ -273,14 +290,78 @@ computeTopPercentile(unsigned int         hist[],
 
 
 static void
-computeEndValues(FILE *   const ifp,
-                 int      const cols,
-                 int      const rows,
-                 xelval   const maxval,
-                 int      const format,
+computeAdjustmentForExpansionLimit(xelval   const maxval,
+                                   xelval   const unlBvalue,
+                                   xelval   const unlWvalue,
+                                   float    const maxExpansion,
+                                   xelval * const bLowerP,
+                                   xelval * const wRaiseP) {
+/*----------------------------------------------------------------------------
+   Assuming 'unlBvalue' and 'unlWvalue' are the appropriate bvalue and
+   wvalue to normalize the image to 0 .. maxval, compute the amount
+   by which the bvalue must be raised and the wvalue lowered from that
+   in order to cap the expansion factor at 'maxExpansion'.
+
+   E.g. if 'maxval' is 100, 'unlBvalue' is 20 and 'unlWvalue' is 70, that
+   implies an expansion factor of 100/50 (because the range goes from
+   70-20, which is 50, to 100 - 0, which is 100).  If 'maxEpansion' is
+   1.333, these values are unacceptable.  To get down to the desired 1.333
+   factor, we need the span of bvalue to wvalue to be 75, not 50.  So
+   we need to raise the bvalue and lower the wvalue by a total of 25.
+   We apportion that adjustment to bvalue and wvalue in proportion to
+   how close each is already to it's end (which we call the margin).
+   'unlBvalue' is 20 from its end, while 'unlWvalue' is 30 from its end,
+   so we want to lower the bvalue by 10 and raise the wvalue by 15.
+   Ergo we return *bLowerP = 10 and *wRaise = 15.
+-----------------------------------------------------------------------------*/
+    unsigned int const newRange = maxval - 0;
+        /* The range of sample values after normalization, if we used
+           the unlimited bvalue and wvalue
+        */
+    unsigned int const oldRange = unlWvalue - unlBvalue;
+        /* The range of sample values in the original image that normalize
+           to 0 .. maxval, if we used the unlimited bvalue and wvalue
+        */
+    float const unlExpansion = (float)newRange/oldRange;
+    
+    if (unlExpansion <= maxExpansion) {
+        /* No capping is necessary.  Unlimited values are already within
+           range.
+           */
+        *bLowerP = 0;
+        *wRaiseP = 0;
+    } else {
+        unsigned int const totalWidening = newRange/maxExpansion - oldRange;
+            /* Amount by which the (bvalue, wvalue) range must be widened
+               to limit expansion to 'maxExpansion'
+            */
+        unsigned int const bMargin = unlBvalue - 0;
+        unsigned int const wMargin = maxval - unlWvalue;
+
+        /* Apportion 'totalWidening' between the black and and the
+           white end
+        */
+        *bLowerP =
+            ROUNDU((float)bMargin / (bMargin + wMargin) * totalWidening);
+        *wRaiseP =
+            ROUNDU((float)wMargin / (bMargin + wMargin) * totalWidening);
+
+        pm_message("limiting expansion of %.1f%% to %.1f%%",
+                   (unlExpansion - 1) * 100, (maxExpansion -1) * 100);
+    }
+}
+
+
+
+static void
+computeEndValues(FILE *             const ifp,
+                 int                const cols,
+                 int                const rows,
+                 xelval             const maxval,
+                 int                const format,
                  struct cmdlineInfo const cmdline,
-                 xelval * const bvalueP,
-                 xelval * const wvalueP) {
+                 xelval *           const bvalueP,
+                 xelval *           const wvalueP) {
 /*----------------------------------------------------------------------------
    Figure out what original values will be translated to full white and
    full black -- thus defining to what all the other values get translated.
@@ -296,29 +377,43 @@ computeEndValues(FILE *   const ifp,
     if (hist == NULL)
         pm_error("Unable to allocate storage for intensity histogram.");
     else {
+        xelval unlimitedBvalue, unlimitedWvalue;
+        unsigned int bLower, wRaise;
+
         buildHistogram(ifp, cols, rows, maxval, format, hist,
                        cmdline.brightMethod);
 
         if (cmdline.bvalueSpec && !cmdline.bpercentSpec) {
-            *bvalueP = cmdline.bvalue;
+            unlimitedBvalue = cmdline.bvalue;
         } else {
+            xelval percentBvalue;
             computeBottomPercentile(hist, maxval, cols*rows, cmdline.bpercent, 
-                                bvalueP);
-            if (cmdline.bvalueSpec && cmdline.bvalue < *bvalueP) {
-                *bvalueP = cmdline.bvalue;
-            }
+                                    &percentBvalue);
+            if (cmdline.bvalueSpec)
+                unlimitedBvalue = MIN(percentBvalue, cmdline.bvalue);
+            else
+                unlimitedBvalue = percentBvalue;
         }
 
         if (cmdline.wvalueSpec && !cmdline.wpercentSpec) {
-            *wvalueP = cmdline.wvalue;
+            unlimitedWvalue = cmdline.wvalue;
         } else {
+            xelval percentWvalue;
             computeTopPercentile(hist, maxval, cols*rows, cmdline.wpercent, 
-                                 wvalueP);
-            if (cmdline.wvalueSpec && cmdline.wvalue > *wvalueP) {
-                *wvalueP = cmdline.wvalue;
-            }
+                                 &percentWvalue);
+            if (cmdline.wvalueSpec)
+                unlimitedWvalue = MAX(percentWvalue, cmdline.wvalue);
+            else
+                unlimitedWvalue = percentWvalue;
         }
 
+        computeAdjustmentForExpansionLimit(
+            maxval, unlimitedBvalue, unlimitedWvalue, cmdline.maxExpansion,
+            &bLower, &wRaise);
+
+        *bvalueP = unlimitedBvalue - bLower;
+        *wvalueP = unlimitedWvalue + wRaise;
+
         free(hist);
     }
 }
@@ -513,7 +608,7 @@ main(int argc, char *argv[]) {
         for (row = 0; row < rows; ++row) {
             pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
             writeRowNormalized(xelrow, cols, maxval, format,
-                               cmdline.keephues, cmdline.brightMethod,
+                               cmdline.brightMethod, cmdline.keephues,
                                newBrightness, rowbuf);
         }
         free(newBrightness);
