@@ -49,6 +49,7 @@
 #include <time.h>
 #include <assert.h>
 
+#include "pm_config.h"
 #include "pam.h"
 #include "shhopt.h"
 #include "mallocvar.h"
@@ -299,7 +300,9 @@ struct randomState {
 };
 
 
+#ifndef LITERAL_FN_DEF_MATCH
 static coord2Color randomColor;
+#endif
 
 static tuple
 randomColor(outGenerator * const outGenP,
@@ -336,7 +339,9 @@ randomColor(outGenerator * const outGenP,
 
 
 
+#ifndef LITERAL_FN_DEF_MATCH
 static outGenStateTerm termRandomColor;
+#endif
 
 static void
 termRandomColor(outGenerator * const outGenP) {
@@ -402,7 +407,9 @@ struct patternPixelState {
 
 
 
+#ifndef LITERAL_FN_DEF_MATCH
 static coord2Color patternPixel;
+#endif
 
 static tuple
 patternPixel(outGenerator * const outGenP,
@@ -431,7 +438,9 @@ patternPixel(outGenerator * const outGenP,
 
 
 
+#ifndef LITERAL_FN_DEF_MATCH
 static outGenStateTerm termPatternPixel;
+#endif
 
 static void
 termPatternPixel(outGenerator * const outGenP) {
@@ -471,6 +480,7 @@ initPatternPixel(outGenerator *     const outGenP,
     outGenP->pam.plainformat = stateP->patPam.plainformat;
     outGenP->pam.depth       = stateP->patPam.depth;
     outGenP->pam.maxval      = stateP->patPam.maxval;
+    strcpy(outGenP->pam.tuple_type, stateP->patPam.tuple_type);
 
     if (cmdline.verbose)
         reportImageParameters("Pattern file", &stateP->patPam);
@@ -489,7 +499,6 @@ createoutputGenerator(struct cmdlineInfo const cmdline,
 
     outGenP->pam.size   = sizeof(struct pam);
     outGenP->pam.len    = PAM_STRUCT_SIZE(tuple_type);
-    outGenP->pam.bytes_per_sample = pnm_bytespersample(outGenP->pam.maxval);
     outGenP->pam.file   = stdout;
     outGenP->pam.height = inPamP->height + 3 * abs(cmdline.guidesize);
         /* Allow room for guides. */
@@ -505,6 +514,8 @@ createoutputGenerator(struct cmdlineInfo const cmdline,
         initRandomColor(outGenP, inPamP, cmdline);
     }
 
+    outGenP->pam.bytes_per_sample = pnm_bytespersample(outGenP->pam.maxval);
+
     *outputGeneratorPP = outGenP;
 }
 
