@@ -10,6 +10,7 @@
 ** implied warranty.
 */
 
+#include <math.h>
 #include "pgm.h"
 #include "shhopt.h"
 
@@ -108,19 +109,20 @@ parseCommandLine(int argc, char ** argv,
 
 
 
-int main(int argc, char *argv[]) {
+int 
+main(int argc, char *argv[]) {
 
     struct cmdlineInfo cmdline;
     gray *grayrow;
     int rowso2, colso2;
-    int row;
+    unsigned int row;
 
     pgm_init( &argc, argv );
 
     parseCommandLine(argc, argv, &cmdline);
 
-    colso2 = cmdline.cols / 2;
-    rowso2 = cmdline.rows / 2;
+    colso2 = MAX(1, cmdline.cols / 2);
+    rowso2 = MAX(1, cmdline.rows / 2);
     
     pgm_writepgminit(stdout, cmdline.cols, cmdline.rows, cmdline.maxval, 0);
     grayrow = pgm_allocrow(cmdline.cols);
@@ -139,16 +141,16 @@ int main(int argc, char *argv[]) {
                 break;
 
             case RT_RECT: {
-                float const r = abs( rowso2 - row ) / (float) rowso2;
-                float const c = abs( colso2 - col ) / (float) colso2;
+                float const r = fabs((int)(rowso2 - row)) / rowso2;
+                float const c = fabs((int)(colso2 - col)) / colso2;
                 grayrow[col] = 
-                    cmdline.maxval - ( r + c ) / 2.0 * cmdline.maxval;
+                    cmdline.maxval - (r + c) / 2.0 * cmdline.maxval;
             }
             break;
             
             case RT_ELLIP: {
-                float const r = abs( rowso2 - row ) / (float) rowso2;
-                float const c = abs( colso2 - col ) / (float) colso2;
+                float const r = fabs((int)(rowso2 - row)) / rowso2;
+                float const c = fabs((int)(colso2 - col)) / colso2;
                 float v;
 
                 v = r * r + c * c;
@@ -164,5 +166,5 @@ int main(int argc, char *argv[]) {
 
     pgm_freerow(grayrow);
     pm_close(stdout);
-    exit(0);
+    return 0;
 }
