@@ -35,7 +35,9 @@
 
 #include <math.h>
 #include <assert.h>
+#include <unistd.h>
 
+#include "pm_c_util.h"
 #include "ppm.h"
 #include "mallocvar.h"
 
@@ -64,18 +66,6 @@ typedef struct {
 #define nrand 4               /* Gauss() sample count */
 #define Cast(low, high) ((low)+(((high)-(low)) * ((rand() & 0x7FFF) / arand)))
 
-/* Utility definition to get an  array's  element  count  (at  compile
-   time).   For  example:  
-
-       int  arr[] = {1,2,3,4,5};
-       ... 
-       printf("%d", ELEMENTS(arr));
-
-   would print a five.  ELEMENTS("abc") can also be used to  tell  how
-   many  bytes are in a string constant INCLUDING THE TRAILING NULL. */
-   
-#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))
-
 /* prototypes */
 static void fourn ARGS((float data[], int nn[], int ndim, int isign));
 static void initgauss ARGS((unsigned int seed));
@@ -295,7 +285,7 @@ initseed(void) {
 
     int i;
 
-    i = time(NULL) ^ 0xF37C;
+    i = time(NULL) ^ getpid();
     srand(i);
     for (i = 0; i < 7; ++i)
         rand();
@@ -617,7 +607,7 @@ makeLand(int *  const irP,
         {133,  82, 0}, {131,  80, 0}, {129,  78, 0}
     };
 
-    unsigned int const ix = ((r - 128) * (ELEMENTS(pgnd) - 1)) / 127;
+    unsigned int const ix = ((r - 128) * (ARRAY_SIZE(pgnd) - 1)) / 127;
     
     *irP = pgnd[ix][0];
     *igP = pgnd[ix][1];
