@@ -4,6 +4,13 @@
 */
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+#if 0
+} /* to fake out automatic code indenters */
+#endif
+
 struct ppm_fs_info {
     /* thisXerr and nextXerr are dynamically allocated arrays each of whose
        dimension is the width of the image plus 2
@@ -49,4 +56,12 @@ ppm_fs_update(    ppm_fs_info *fi, int col, pixel *pP);
 
 
 void
-ppm_fs_update3(ppm_fs_info *fi, int col, pixval r, pixval g, pixval b);
+ppm_fs_update3(ppm_fs_info * const fi, 
+               int           const col, 
+               pixval        const r, 
+               pixval        const g, 
+               pixval        const b);
+
+#ifdef __cplusplus
+}
+#endif
