@@ -10,7 +10,8 @@
 */
 #define _FILE_OFFSET_BITS 64
 #define _LARGE_FILES  
-#define _XOPEN_SOURCE
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <string.h>
 #include <limits.h>
@@ -55,6 +56,20 @@ allocationDepth(const struct pam * const pamP) {
 
 
 
+static const char **
+pamCommentP(const struct pam * const pamP) {
+
+    const char ** retval;
+
+    if (pamP->len >= PAM_STRUCT_SIZE(comment_p))
+        retval = pamP->comment_p;
+    else
+        retval = NULL;
+
+    return retval;
+}
+
+
 static void
 validateComputableSize(struct pam * const pamP) {
 /*----------------------------------------------------------------------------
@@ -75,7 +90,7 @@ validateComputableSize(struct pam * const pamP) {
 
     if (depth > INT_MAX/sizeof(sample))
         pm_error("image depth (%u) too large to be processed", depth);
-    else if (depth * sizeof(sample) > INT_MAX/pamP->width)
+    else if (pamP->width > 0 && depth * sizeof(sample) > INT_MAX/pamP->width)
         pm_error("image width and depth (%u, %u) too large "
                  "to be processed.", pamP->width, depth);
     else if (pamP->width * (depth * sizeof(sample)) >
@@ -172,21 +187,12 @@ pnm_createBlackTuple(const struct pam * const pamP,
    Create a "black" tuple.  By that we mean a tuple all of whose elements
    are zero.  If it's an RGB, grayscale, or b&w pixel, that means it's black.
 -----------------------------------------------------------------------------*/
+    unsigned int i;
+
     *blackTupleP = pnm_allocpamtuple(pamP);
-    if (pamP->format == PAM_FORMAT) {
-        /* In this format, we don't know the meaning of "black", so we
-           just punt.
-           */
-        int i;
-        for (i = 0; i < pamP->depth; i++) 
-            (*blackTupleP)[i] = 0;
-    } else {
-        xel black_xel;
-        black_xel = pnm_blackxel(pamP->maxval, pamP->format);
-        (*blackTupleP)[0] = PPM_GETR(black_xel);
-        (*blackTupleP)[1] = PPM_GETG(black_xel);
-        (*blackTupleP)[2] = PPM_GETB(black_xel);
-    }
+
+    for (i = 0; i < pamP->depth; ++i) 
+        (*blackTupleP)[i] = 0;
 }
 
 
@@ -544,15 +550,49 @@ process_header_line(char                const buffer[],
 
 
 
+static void
+appendComment(char **      const commentsP,
+              const char * const commentHeader) {
+
+    const char * const commentLine = &commentHeader[1];
+
+    size_t commentLen = strlen(*commentsP) + strlen(commentLine) + 1;
+
+    assert(commentHeader[0] == '#');
+
+    REALLOCARRAY(*commentsP, commentLen);
+
+    if (*commentsP == NULL)
+        pm_error("Couldn't get storage for %u characters of comments from "
+                 "the PAM header", commentLen);
+
+    strcat(*commentsP, commentLine);
+}
+
+
+
+static void
+disposeOfComments(const struct pam * const pamP,
+                  const char *       const comments) {
+
+    const char ** const retP = pamCommentP(pamP);
+
+    if (retP)
+        *retP = comments;
+    else
+        strfree(comments);
+}
+
+
+
 static void
 readpaminitrest(struct pam * const pamP) {
 /*----------------------------------------------------------------------------
    Read the rest of the PAM header (after the first line -- the magic
    number line).  Fill in all the information in *pamP.
 -----------------------------------------------------------------------------*/
-    char buffer[256];
-    char *rc;
     struct headerSeen headerSeen;
+    char * comments;
 
     headerSeen.width  = FALSE;
     headerSeen.height = FALSE;
@@ -562,6 +602,8 @@ readpaminitrest(struct pam * const pamP) {
 
     pamP->tuple_type[0] = '\0';
 
+    comments = strdup("");
+
     { 
         int c;
         /* Read off rest of 1st line -- probably just the newline after the 
@@ -571,19 +613,25 @@ readpaminitrest(struct pam * const pamP) {
     }    
 
     while (!headerSeen.endhdr) {
+        char buffer[256];
+        char * rc;
         rc = fgets(buffer, sizeof(buffer), pamP->file);
         if (rc == NULL)
             pm_error("EOF or error reading file while trying to read the "
                      "PAM header");
         else {
-            if (buffer[0] == '#');
-                /* Ignore it; it's a comment */
+            buffer[256-1-1] = '\n';  /* In case fgets() truncated */
+            if (buffer[0] == '#')
+                appendComment(&comments, buffer);
             else if (stripeq(buffer, ""));
                 /* Ignore it; it's a blank line */
             else 
                 process_header_line(buffer, pamP, &headerSeen);
         }
     }
+
+    disposeOfComments(pamP, comments);
+
     if (!headerSeen.height)
         pm_error("No HEIGHT header line in PAM header");
     if (!headerSeen.width)
@@ -622,9 +670,9 @@ pnm_readpaminitrestaspnm(FILE * const fileP,
 -----------------------------------------------------------------------------*/
     struct pam pam;
 
-    pam.size        = PAM_STRUCT_SIZE(tuple_type);
+    pam.size        = sizeof(struct pam);
     pam.file        = fileP;
-    pam.len         = sizeof(struct pam);
+    pam.len         = PAM_STRUCT_SIZE(tuple_type);
     pam.format      = PAM_FORMAT;
 
     readpaminitrest(&pam);
@@ -703,14 +751,18 @@ pnm_readpaminit(FILE *       const file,
         pamP->maxval = (sample) maxval;
         pamP->depth = 3;
         strcpy(pamP->tuple_type, PAM_PPM_TUPLETYPE);
+        if (pamCommentP(pamP))
+            *pamP->comment_p = strdup("");
     } break;
 
     case PGM_TYPE: {
         gray maxval;
-        pgm_readpgminitrest(pamP->file, &pamP->width, &pamP->height, &maxval );
+        pgm_readpgminitrest(pamP->file, &pamP->width, &pamP->height, &maxval);
         pamP->maxval = (sample) maxval;
         pamP->depth = 1;
         strcpy(pamP->tuple_type, PAM_PGM_TUPLETYPE);
+        if (pamCommentP(pamP))
+            *pamP->comment_p = strdup("");
     } break;
 
     case PBM_TYPE:
@@ -718,6 +770,8 @@ pnm_readpaminit(FILE *       const file,
         pamP->maxval = (sample) 1;
         pamP->depth = 1;
         strcpy(pamP->tuple_type, PAM_PBM_TUPLETYPE);
+        if (pamCommentP(pamP))
+            *pamP->comment_p = strdup("");
         break;
         
     default:
@@ -732,6 +786,38 @@ pnm_readpaminit(FILE *       const file,
 }
 
 
+
+static void
+writeComments(const struct pam * const pamP) {
+/*----------------------------------------------------------------------------
+   Write comments for a PAM header, insofar as *pamP specifies comments.
+-----------------------------------------------------------------------------*/
+    const char ** const commentP = pamCommentP(pamP);
+
+    if (commentP) {
+        const char * const comment = *commentP;
+
+        const char * p;
+        bool startOfLine;
+
+        for (p = &comment[0], startOfLine = TRUE; *p; ++p) {
+            if (startOfLine)
+                fputc('#', pamP->file);
+                
+            fputc(*p, pamP->file);
+
+            if (*p == '\n')
+                startOfLine = TRUE;
+            else
+                startOfLine = FALSE;
+        }
+        if (!startOfLine)
+            fputc('\n', pamP->file);
+    }
+}
+
+
+
 /* About the 'plainformat' member on image input operations:
 
    'plainformat' is meaningless when reading an image, but we always
@@ -792,6 +878,7 @@ pnm_writepaminit(struct pam * const pamP) {
             pm_error("There is no plain version of PAM.  -plain option "
                      "is not allowed");
         fprintf(pamP->file, "P7\n");
+        writeComments(pamP);
         fprintf(pamP->file, "WIDTH %u\n",   (unsigned)pamP->width);
         fprintf(pamP->file, "HEIGHT %u\n",  (unsigned)pamP->height);
         fprintf(pamP->file, "DEPTH %u\n",   pamP->depth);
@@ -978,7 +1065,7 @@ pm_feed_from_pamtuples(int    const pipeToFeedFd,
     outpam.file = fdopen(pipeToFeedFd, "w");
 
     /* The following signals (and normally kills) the process with
-       SIGPIPE if the pipe does not take all 'inputLength' bytes.
+       SIGPIPE if the pipe does not take all the data.
     */
     pnm_writepam(&outpam, *inputTuplesP->tuplesP);
 
@@ -993,10 +1080,11 @@ pm_accept_to_pamtuples(int    const pipeToSuckFd,
 
     struct pamtuples * const outputTuplesP = accepterParm;
 
-    struct pam inpam;
+    struct pam * const inpamP = outputTuplesP->pamP;
 
     *outputTuplesP->tuplesP =
-        pnm_readpam(fdopen(pipeToSuckFd, "r"), &inpam, sizeof(inpam));
+        pnm_readpam(fdopen(pipeToSuckFd, "r"), inpamP,
+                    PAM_STRUCT_SIZE(tuple_type));
 
-    pm_close(inpam.file);
+    pm_close(inpamP->file);
 }
