@@ -14,8 +14,10 @@
 
 #include <string.h>
 
-#include "pam.h"
+#include "mallocvar.h"
+#include "nstring.h"
 #include "shhopt.h"
+#include "pam.h"
 
 #define MAX_INPUTS 16
     /* The most input PAMs we allow user to specify */
@@ -27,7 +29,7 @@ struct cmdlineInfo {
     const char *tupletype;       /* Tuple type for output PAM */
     unsigned int nInput;
         /* The number of input PAMs.  At least 1, at most 16. */
-    char * inputFilespec[MAX_INPUTS];
+    const char * inputFileName[MAX_INPUTS];
         /* The PAM files to combine, in order. */
 };
 
@@ -35,12 +37,12 @@ struct cmdlineInfo {
 
 static void
 parseCommandLine(int argc, char ** argv,
-                 struct cmdlineInfo *cmdlineP) {
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec strings we return are stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -49,6 +51,8 @@ parseCommandLine(int argc, char ** argv,
     unsigned int option_def_index;
     unsigned int tupletypeSpec;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+    
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "tupletype",  OPT_STRING, &cmdlineP->tupletype, 
             &tupletypeSpec, 0);
@@ -69,16 +73,36 @@ parseCommandLine(int argc, char ** argv,
 
     cmdlineP->nInput = 0;  /* initial value */
     { 
-        int argn;
-        for (argn = 1; argn < argc; argn++) {
+        unsigned int argn;
+        bool stdinUsed;
+        for (argn = 1, stdinUsed = false; argn < argc; ++argn) {
             if (cmdlineP->nInput >= MAX_INPUTS) 
-                pm_error("You may not specify more than %d input images.",
+                pm_error("You may not specify more than %u input images.",
                          MAX_INPUTS);
-            cmdlineP->inputFilespec[cmdlineP->nInput++] = argv[argn];
+            cmdlineP->inputFileName[cmdlineP->nInput++] = argv[argn];
+            if (streq(argv[argn], "-")) {
+                if (stdinUsed)
+                    pm_error("You cannot specify Standard Input ('-') "
+                             "for more than one input file");
+                stdinUsed = true;
+            }
         }
     }
     if (cmdlineP->nInput < 1)
-        pm_error("You must specify at least one input PAM image.");
+        cmdlineP->inputFileName[cmdlineP->nInput++] = "-";
+}
+
+
+
+static void
+openAllStreams(unsigned int  const nInput,
+               const char ** const inputFileName,
+               FILE **       const ifP) {
+
+    unsigned int inputSeq;
+
+    for (inputSeq = 0; inputSeq < nInput; ++inputSeq)
+        ifP[inputSeq] = pm_openr(inputFileName[inputSeq]);
 }
 
 
@@ -123,28 +147,27 @@ outputRaster(const struct pam       inpam[],
 
 
 
-int
-main(int argc, char *argv[]) {
+static void
+processOneImageInAllStreams(unsigned int const nInput,
+                            FILE *       const ifP[],
+                            FILE *       const ofP,
+                            const char * const tupletype) {
 
-    struct cmdlineInfo cmdline;
-    FILE* ifp[MAX_INPUTS];
-    struct pam inpam[MAX_INPUTS];   /* Input PAM imageS */
+    struct pam inpam[MAX_INPUTS];   /* Input PAM images */
     struct pam outpam;  /* Output PAM image */
+
     unsigned int inputSeq;
-    int outputDepth;
-
-    pnm_init(&argc, argv);
-
-    parseCommandLine(argc, argv, &cmdline);
+        /* The horizontal sequence -- i.e. the sequence of the
+           input stream, not the sequence of an image within a
+           stream.
+        */
 
+    unsigned int outputDepth;
     outputDepth = 0;  /* initial value */
     
+    for (inputSeq = 0; inputSeq < nInput; ++inputSeq) {
 
-    for (inputSeq = 0; inputSeq < cmdline.nInput; ++inputSeq) {
-
-        ifp[inputSeq] = pm_openr(cmdline.inputFilespec[inputSeq]);
-
-        pnm_readpaminit(ifp[inputSeq], &inpam[inputSeq], 
+        pnm_readpaminit(ifP[inputSeq], &inpam[inputSeq], 
                         PAM_STRUCT_SIZE(tuple_type));
 
         if (inputSeq > 0) {
@@ -164,27 +187,62 @@ main(int argc, char *argv[]) {
         outputDepth += inpam[inputSeq].depth;
     }
 
-    outpam = inpam[0];     /* Initial value */
-    outpam.depth = outputDepth;
-    outpam.file = stdout;
+    outpam        = inpam[0];     /* Initial value */
+    outpam.depth  = outputDepth;
+    outpam.file   = ofP;
     outpam.format = PAM_FORMAT;
-    strcpy(outpam.tuple_type, cmdline.tupletype);
+    strcpy(outpam.tuple_type, tupletype);
 
-    pm_message("Writing %d channel PAM image", outpam.depth);
+    pm_message("Writing %u channel PAM image", outpam.depth);
 
     pnm_writepaminit(&outpam);
 
-    outputRaster(inpam, cmdline.nInput, outpam);
-
-    exit(0);
+    outputRaster(inpam, nInput, outpam);
 }
 
 
 
+static void
+nextImageAllStreams(unsigned int const nInput,
+                    FILE *       const ifP[],
+                    bool *       const eofP) {
+/*----------------------------------------------------------------------------
+   Advance all the streams ifP[] to the next image.
+
+   Return *eofP == TRUE iff at least one stream has no next image.
+-----------------------------------------------------------------------------*/
+    unsigned int inputSeq;
+
+    for (inputSeq = 0; inputSeq < nInput; ++inputSeq) {
+        bool eof;
+        pnm_nextimage(ifP[inputSeq], &eof);
+        if (eof)
+            *eofP = eof;
+    }
+}
 
 
 
+int
+main(int argc, char *argv[]) {
 
+    struct cmdlineInfo cmdline;
+    FILE * ifP[MAX_INPUTS];
+    bool eof;
 
+    pnm_init(&argc, argv);
 
+    parseCommandLine(argc, argv, &cmdline);
 
+    openAllStreams(cmdline.nInput, cmdline.inputFileName, ifP);
+
+    eof = FALSE;
+    while (!eof) {
+        processOneImageInAllStreams(cmdline.nInput, ifP, stdout,
+                                    cmdline.tupletype);
+
+        nextImageAllStreams(cmdline.nInput, ifP, &eof);
+    }
+
+    return 0;
+}
